/*  ============================================================================
 *           Copyright (C) 2011, 2012 Texas Instruments Incorporated.          *
 *
 *   Use of this software is controlled by the terms and conditions found in the
 *   license agreement under which this software has been supplied.
* ============================================================================
This file setup Packet DMA and QMSS for AIF2
* =============================================================================
 * Created by :                                                             *
 *            Brighton Feng                                                 *
 *            Texas Instruments                                             * 
 *            August 4, 2011                                                 *
Updated on June 18, 2012 for TCI6614 by Brighton
 * ============================================================================
 */

#include <stdio.h>
#include <string.h>
#include "KeyStone_common.h"
#include "Aif_test_config.h"
#include "AIF_PktDMA_init.h"
#ifdef _BIG_ENDIAN
#include <ti/drv/qmss/firmware/acc48_be_bin.h>
#else
#include <ti/drv/qmss/firmware/acc48_le_bin.h>
#endif

CSL_Cppidma_global_configRegs * aifDmaCfgRegs= 
	(CSL_Cppidma_global_configRegs * )CSL_AIF2_CFG_CPPI_DMA_GLOBAL_CFG_REGS;
CSL_Cppidma_tx_channel_configRegs * aifDmaTxChCfgRegs= 
	(CSL_Cppidma_tx_channel_configRegs * )CSL_AIF2_CFG_CPPI_DMA_TX_CFG_REGS;
CSL_Cppidma_rx_channel_configRegs * aifDmaRxChCfgRegs= 
	(CSL_Cppidma_rx_channel_configRegs * )CSL_AIF2_CFG_CPPI_DMA_RX_CFG_REGS;
volatile Uint32 * aifDmaTxChPriority = 
	(volatile Uint32 *)CSL_AIF2_CFG_CPPI_DMA_TX_SCHEDULER_CFG_REGS;
CSL_Cppidma_rx_flow_configRegs * aifDmaRxFlowCfgRegs= 
	(CSL_Cppidma_rx_flow_configRegs * )CSL_AIF2_CFG_CPPI_DMA_RX_FLOW_CFG_REGS;
PKT_DMA_Regs aifPktDmaRegs=
{
	(CSL_Cppidma_global_configRegs * )CSL_AIF2_CFG_CPPI_DMA_GLOBAL_CFG_REGS,
	(CSL_Cppidma_tx_channel_configRegs * )CSL_AIF2_CFG_CPPI_DMA_TX_CFG_REGS,
	(CSL_Cppidma_rx_channel_configRegs * )CSL_AIF2_CFG_CPPI_DMA_RX_CFG_REGS,
	(volatile Uint32 *)CSL_AIF2_CFG_CPPI_DMA_TX_SCHEDULER_CFG_REGS,
	(CSL_Cppidma_rx_flow_configRegs * )CSL_AIF2_CFG_CPPI_DMA_RX_FLOW_CFG_REGS
};

#pragma DATA_SECTION(monoPacket_SL2,"QMSS_Data:Descriptor_SL2")
#pragma DATA_ALIGN (monoPacket_SL2, 128)
Uint8 monoPacket_SL2[SL2_NUM_MONO_PACKET][AIF_MONO_PACKET_SIZE];

#pragma DATA_SECTION(monoPacket_LL2,"QMSS_Data:Descriptor_LL2")
#pragma DATA_ALIGN (monoPacket_LL2, 64)
Uint8 monoPacket_LL2[LL2_NUM_MONO_PACKET][AIF_MONO_PACKET_SIZE];

#pragma DATA_SECTION(monoPacket_DDR,"QMSS_Data:Descriptor_DDR")
#pragma DATA_ALIGN (monoPacket_DDR, 128)
Uint8 monoPacket_DDR[DDR_NUM_MONO_PACKET][AIF_MONO_PACKET_SIZE];

/* descriptor memory region configuration.
descriptor Base address must be specified in ascending order,
no overlap is allowed between regions*/
Qmss_DescMemRegionCfg descMemRegionsCfg[]=
{   /*The base address of descriptor region     Size of each descriptor   Number of descriptors 
                                                16*n,1<=n<=8192           2^(5+n), 0<=n<=15*/
    {(Uint32)monoPacket_SL2,                    AIF_MONO_PACKET_SIZE,     SL2_NUM_MONO_PACKET},/*Monolithic descriptor in SL2*/
    {((Uint32)monoPacket_LL2)+0x10000000,       AIF_MONO_PACKET_SIZE,     LL2_NUM_MONO_PACKET},/*Monolithic descriptor in Core0 LL2*/
    {((Uint32)monoPacket_LL2)+0x11000000,       AIF_MONO_PACKET_SIZE,     LL2_NUM_MONO_PACKET},/*Monolithic descriptor in Core1 LL2*/
    {((Uint32)monoPacket_LL2)+0x12000000,       AIF_MONO_PACKET_SIZE,     LL2_NUM_MONO_PACKET},/*Monolithic descriptor in Core2 LL2*/
    {((Uint32)monoPacket_LL2)+0x13000000,       AIF_MONO_PACKET_SIZE,     LL2_NUM_MONO_PACKET},/*Monolithic descriptor in Core3 LL2*/
    {(Uint32)monoPacket_DDR,                    AIF_MONO_PACKET_SIZE,     DDR_NUM_MONO_PACKET}/*Monolithic descriptor in DDR*/
};

FreeMonoQueueCfg freeQueueCfgTable[]=
{
    /*free queue number*//*address of first descriptor*/  /*size of descriptor*//*number of descriptors*/
    {DDR_FDQ,        (Uint32)monoPacket_DDR,              AIF_MONO_PACKET_SIZE, DDR_NUM_MONO_PACKET},/*DDR monolithic Free Descriptor Queue*/
    {SL2_FDQ,        (Uint32)monoPacket_SL2,              AIF_MONO_PACKET_SIZE, SL2_NUM_MONO_PACKET},/*SL2 monolithic Free Descriptor Queue*/
    {CORE0_LL2_FDQ,  ((Uint32)monoPacket_LL2)+0x10000000, AIF_MONO_PACKET_SIZE, LL2_NUM_MONO_PACKET},/*CORE0_LL2 monolithic Free Descriptor Queue*/
    {CORE1_LL2_FDQ,  ((Uint32)monoPacket_LL2)+0x11000000, AIF_MONO_PACKET_SIZE, LL2_NUM_MONO_PACKET},/*CORE1_LL2 monolithic Free Descriptor Queue*/
    {CORE2_LL2_FDQ,  ((Uint32)monoPacket_LL2)+0x12000000, AIF_MONO_PACKET_SIZE, LL2_NUM_MONO_PACKET},/*CORE2_LL2 monolithic Free Descriptor Queue*/
    {CORE3_LL2_FDQ,  ((Uint32)monoPacket_LL2)+0x13000000, AIF_MONO_PACKET_SIZE, LL2_NUM_MONO_PACKET}/*CORE3_LL2 monolithic Free Descriptor Queue*/
};

/*flow configuration table*/
PktDma_RxFlowCfg flowCfgTable[]=
{
    /*dest_qnum*/                             /*sop_offset*//*ps_loc*//*desc_type*//*error retry*//*psinfo*/ /*einfo*//*size_thresh0(1,2)_en*//*size_thresh0,1,2*//*fdq0_sz0_qnum, sz1_qnum, sz2_qnum, sz3_qnum*//*fdq1_qnum*//*fdq2_qnum*//*fdq3_qnum*//*dest_tag_lo*//*dest_tag_hi*//*src_tag_lo*//*src_tag_hi*//*dest_tag_lo_sel*//*dest_tag_hi_sel*//*src_tag_lo_sel*//*src_tag_hi_sel*/
    {AIF_RX_FLOW0_DST_Q+AIF_RX_FLOW_DDR      , 16,          0,  Cppi_DescType_MONOLITHIC,     0,   1     ,     0,      0, 0, 0,                0, 0, 0,           DDR_FDQ,         0, 0, 0,                        DDR_FDQ,     DDR_FDQ,     DDR_FDQ,         0,          0,              0,              0,              0,                  0,                  0,                  0},
    {AIF_RX_FLOW0_DST_Q+AIF_RX_FLOW_SL2      , 16,          0,  Cppi_DescType_MONOLITHIC,     0,   1     ,     0,      0, 0, 0,                0, 0, 0,           SL2_FDQ,         0, 0, 0,                        SL2_FDQ,     SL2_FDQ,     SL2_FDQ,         0,          0,              0,              0,              0,                  0,                  0,                  0},
    {AIF_RX_FLOW0_DST_Q+AIF_RX_FLOW_CORE0_LL2, 16,          0,  Cppi_DescType_MONOLITHIC,     0,   1     ,     0,      0, 0, 0,                0, 0, 0,           CORE0_LL2_FDQ,   0, 0, 0,                   CORE0_LL2_FDQ, CORE0_LL2_FDQ, CORE0_LL2_FDQ,    0,          0,              0,              0,              0,                  0,                  0,                  0},
    {AIF_RX_FLOW0_DST_Q+AIF_RX_FLOW_CORE1_LL2, 16,          0,  Cppi_DescType_MONOLITHIC,     0,   1     ,     0,      0, 0, 0,                0, 0, 0,           CORE1_LL2_FDQ,   0, 0, 0,                   CORE1_LL2_FDQ, CORE1_LL2_FDQ, CORE1_LL2_FDQ,    0,          0,              0,              0,              0,                  0,                  0,                  0},
    {AIF_RX_FLOW0_DST_Q+AIF_RX_FLOW_CORE2_LL2, 16,          0,  Cppi_DescType_MONOLITHIC,     0,   1     ,     0,      0, 0, 0,                0, 0, 0,           CORE2_LL2_FDQ,   0, 0, 0,                   CORE2_LL2_FDQ, CORE2_LL2_FDQ, CORE2_LL2_FDQ,    0,          0,              0,              0,              0,                  0,                  0,                  0},
    {AIF_RX_FLOW0_DST_Q+AIF_RX_FLOW_CORE3_LL2, 16,          0,  Cppi_DescType_MONOLITHIC,     0,   1     ,     0,      0, 0, 0,                0, 0, 0,           CORE3_LL2_FDQ,   0, 0, 0,                   CORE3_LL2_FDQ, CORE3_LL2_FDQ, CORE3_LL2_FDQ,    0,          0,              0,              0,              0,                  0,                  0,                  0}
};

char * flow_str[]=
{
	"DDR"      ,
	"SL2"      ,
	"CORE0_LL2",
	"CORE1_LL2",
	"CORE2_LL2",
	"CORE3_LL2"
};

/*flow look up table for each channels,
specify which flow be used for each channel*/
unsigned char flowTable[]=
{
	AIF_RX_FLOW_CORE1_LL2, 
	AIF_RX_FLOW_CORE2_LL2, 
	AIF_RX_FLOW_CORE3_LL2, 
	AIF_RX_FLOW_SL2,
	AIF_RX_FLOW_SL2,
	AIF_RX_FLOW_SL2,
	AIF_RX_FLOW_SL2,
	AIF_RX_FLOW_SL2,

	AIF_RX_FLOW_CORE1_LL2, 
	AIF_RX_FLOW_CORE2_LL2, 
	AIF_RX_FLOW_CORE3_LL2, 
	AIF_RX_FLOW_SL2,
	AIF_RX_FLOW_SL2,
	AIF_RX_FLOW_SL2,
	AIF_RX_FLOW_SL2,
	AIF_RX_FLOW_SL2,

	AIF_RX_FLOW_CORE1_LL2, 
	AIF_RX_FLOW_CORE2_LL2, 
	AIF_RX_FLOW_CORE3_LL2, 
	AIF_RX_FLOW_SL2,
	AIF_RX_FLOW_SL2,
	AIF_RX_FLOW_SL2,
	AIF_RX_FLOW_SL2,
	AIF_RX_FLOW_SL2,

	AIF_RX_FLOW_CORE1_LL2, 
	AIF_RX_FLOW_CORE2_LL2, 
	AIF_RX_FLOW_CORE3_LL2, 
	AIF_RX_FLOW_SL2,
	AIF_RX_FLOW_SL2,
	AIF_RX_FLOW_SL2,
	AIF_RX_FLOW_SL2,
	AIF_RX_FLOW_SL2,

	AIF_RX_FLOW_CORE1_LL2, 
	AIF_RX_FLOW_CORE2_LL2, 
	AIF_RX_FLOW_CORE3_LL2, 
	AIF_RX_FLOW_SL2,
	AIF_RX_FLOW_SL2,
	AIF_RX_FLOW_SL2,
	AIF_RX_FLOW_SL2,
	AIF_RX_FLOW_SL2,

	AIF_RX_FLOW_CORE1_LL2, 
	AIF_RX_FLOW_CORE2_LL2, 
	AIF_RX_FLOW_CORE3_LL2, 
	AIF_RX_FLOW_SL2,
	AIF_RX_FLOW_SL2,
	AIF_RX_FLOW_SL2,
	AIF_RX_FLOW_SL2,
	AIF_RX_FLOW_SL2,

	AIF_RX_FLOW_CORE1_LL2, 
	AIF_RX_FLOW_CORE2_LL2, 
	AIF_RX_FLOW_CORE3_LL2, 
	AIF_RX_FLOW_SL2,
	AIF_RX_FLOW_SL2,
	AIF_RX_FLOW_SL2,
	AIF_RX_FLOW_SL2,
	AIF_RX_FLOW_SL2,

	AIF_RX_FLOW_CORE1_LL2, 
	AIF_RX_FLOW_CORE2_LL2, 
	AIF_RX_FLOW_CORE3_LL2, 
	AIF_RX_FLOW_SL2,
	AIF_RX_FLOW_SL2,
	AIF_RX_FLOW_SL2,
	AIF_RX_FLOW_SL2,
	AIF_RX_FLOW_SL2,

	AIF_RX_FLOW_CORE0_LL2, 
	AIF_RX_FLOW_CORE1_LL2, 
	AIF_RX_FLOW_CORE2_LL2, 
	AIF_RX_FLOW_CORE3_LL2, 
	AIF_RX_FLOW_SL2,
	AIF_RX_FLOW_SL2,
	AIF_RX_FLOW_SL2,
	AIF_RX_FLOW_SL2,

	AIF_RX_FLOW_CORE0_LL2, 
	AIF_RX_FLOW_CORE1_LL2, 
	AIF_RX_FLOW_CORE2_LL2, 
	AIF_RX_FLOW_CORE3_LL2, 
	AIF_RX_FLOW_SL2,
	AIF_RX_FLOW_SL2,
	AIF_RX_FLOW_SL2,
	AIF_RX_FLOW_SL2,

	AIF_RX_FLOW_CORE0_LL2, 
	AIF_RX_FLOW_CORE1_LL2, 
	AIF_RX_FLOW_CORE2_LL2, 
	AIF_RX_FLOW_CORE3_LL2, 
	AIF_RX_FLOW_SL2,
	AIF_RX_FLOW_SL2,
	AIF_RX_FLOW_SL2,
	AIF_RX_FLOW_SL2,

	AIF_RX_FLOW_CORE0_LL2, 
	AIF_RX_FLOW_CORE1_LL2, 
	AIF_RX_FLOW_CORE2_LL2, 
	AIF_RX_FLOW_CORE3_LL2, 
	AIF_RX_FLOW_SL2,
	AIF_RX_FLOW_SL2,
	AIF_RX_FLOW_SL2,
	AIF_RX_FLOW_SL2,

	AIF_RX_FLOW_CORE0_LL2, 
	AIF_RX_FLOW_CORE1_LL2, 
	AIF_RX_FLOW_CORE2_LL2, 
	AIF_RX_FLOW_CORE3_LL2, 
	AIF_RX_FLOW_SL2,
	AIF_RX_FLOW_SL2,
	AIF_RX_FLOW_SL2,
	AIF_RX_FLOW_SL2,

	AIF_RX_FLOW_CORE0_LL2, 
	AIF_RX_FLOW_CORE1_LL2, 
	AIF_RX_FLOW_CORE2_LL2, 
	AIF_RX_FLOW_CORE3_LL2, 
	AIF_RX_FLOW_SL2,
	AIF_RX_FLOW_SL2,
	AIF_RX_FLOW_SL2,
	AIF_RX_FLOW_SL2,

	AIF_RX_FLOW_CORE0_LL2, 
	AIF_RX_FLOW_CORE1_LL2, 
	AIF_RX_FLOW_CORE2_LL2, 
	AIF_RX_FLOW_CORE3_LL2, 
	AIF_RX_FLOW_SL2,
	AIF_RX_FLOW_SL2,
	AIF_RX_FLOW_SL2,
	AIF_RX_FLOW_SL2,

	AIF_RX_FLOW_CORE0_LL2, 
	AIF_RX_FLOW_CORE1_LL2, 
	AIF_RX_FLOW_CORE2_LL2, 
	AIF_RX_FLOW_CORE3_LL2, 
	AIF_RX_FLOW_SL2,
	AIF_RX_FLOW_SL2,
	AIF_RX_FLOW_SL2,
	AIF_RX_FLOW_SL2,

	AIF_RX_FLOW_DDR,
	AIF_RX_FLOW_DDR,
	AIF_RX_FLOW_DDR,
	AIF_RX_FLOW_DDR,

	AIF_RX_FLOW_SL2,
	AIF_RX_FLOW_SL2,
	AIF_RX_FLOW_SL2,
	AIF_RX_FLOW_SL2

};

/*Descriptor accumulation buffer*/
Uint32 uiaAifDescAccList[(QUEUE_ACC_THRESH+1)*2];
Uint32 uiQueueAccThresh = QUEUE_ACC_THRESH;

Uint32 uiNumUsedDescRegion;
Uint32 uiNumInitialFdq;
Uint32 uiNumUsedRxFlow;

void QMSS_Accumulation_config()
{
	Uint32 uiReturnCode;
	Qmss_AccCmdCfg loQuAccCfg;

	/*set PDSP tick interval 25us*/
	uiReturnCode= KeyStone_Qmss_config_Acc_Timer(Qmss_PdspId_PDSP1, 4167);
	if(1!=uiReturnCode)
		printf("ACC timer configure return code = %d\n", uiReturnCode);

	loQuAccCfg.channel = 32;
	loQuAccCfg.command = Qmss_AccCmd_ENABLE_CHANNEL;
	loQuAccCfg.queueEnMask = 0x3F;
	loQuAccCfg.listAddress = GLOBAL_ADDR(uiaAifDescAccList);

	/*this value must be multiple of 32 for multiple queue mode*/
	loQuAccCfg.queMgrIndex = AIF_RX_FLOW0_DST_Q;

	/*Accumulation threshold match to number of channel,
	accumulation interrupt should be generated when all channel receive
	one packet*/
	if(uiNumAifChannel<QUEUE_ACC_THRESH)
		uiQueueAccThresh = uiNumAifChannel;
	loQuAccCfg.maxPageEntries = uiQueueAccThresh+1;

	loQuAccCfg.timerLoadCount = 2;
	loQuAccCfg.interruptPacingMode = Qmss_AccPacingMode_FIRST_NEW_PACKET;
	loQuAccCfg.listEntrySize = Qmss_AccEntrySize_REG_D;
	loQuAccCfg.listCountMode = Qmss_AccCountMode_ENTRY_COUNT;
	loQuAccCfg.multiQueueMode = Qmss_AccQueueMode_MULTI_QUEUE;
	KeyStone_Qmss_Config_Acc_Channel(Qmss_PdspId_PDSP1, &loQuAccCfg);

}

/*AIF Queue initialization*/
void AIF_Queue_init()
{
	uiNumUsedDescRegion= sizeof(descMemRegionsCfg)/sizeof(Qmss_DescMemRegionCfg);

	/*-----------------QMSS queue configuration------------------*/
	KeyStone_QMSS_Linking_RAM_init(NULL, 0);

	KeyStone_QMSS_Descriptor_Regions_init(descMemRegionsCfg, uiNumUsedDescRegion);

	uiNumInitialFdq= sizeof(freeQueueCfgTable)/sizeof(FreeMonoQueueCfg);
	KeyStone_Mono_Descriptor_Queues_init(freeQueueCfgTable, uiNumInitialFdq);

	/*-----------------QMSS PDSP fireware configuration------------------*/
#ifdef _BIG_ENDIAN
	KeyStone_Qmss_Download_Firmware(Qmss_PdspId_PDSP1, &acc48_be, sizeof(acc48_be));
#else
	KeyStone_Qmss_Download_Firmware(Qmss_PdspId_PDSP1, &acc48_le, sizeof(acc48_le));
#endif

	/*clear QMSS interrupts*/
	gpQM_INTD_regs->STATUS_CLR_REG0= 0xFFFFFFFF;
	gpQM_INTD_regs->STATUS_CLR_REG1= 0xFFFF;

	KeyStone_Qmss_Config_Reclaim_Queue (Qmss_PdspId_PDSP1, RECLAMATION_QUEUE);
	
	QMSS_Accumulation_config();

}

/*AIF packet DMA initialization
must be called after AIF PSC enabled*/
void AIF_PktDMA_init()
{
	int i;
	
	uiNumUsedRxFlow= sizeof(flowTable);
	if(uiNumUsedRxFlow>AIF_PKTDMA_MAX_RX_FLOW_NUM)
		uiNumUsedRxFlow= AIF_PKTDMA_MAX_RX_FLOW_NUM;
	if(uiNumUsedRxFlow>uiNumAifChannel)
		uiNumUsedRxFlow= uiNumAifChannel;

	/*-----------------AIF PktDMA configuration------------------*/
	KeyStone_pktDma_Global_Control(aifDmaCfgRegs, 256, 0, 0);

	aifDmaCfgRegs->EMULATION_CONTROL_REG= 
		(0<<CSL_CPPIDMA_GLOBAL_CONFIG_EMULATION_CONTROL_REG_LOOPBACK_EN_SHIFT)
		|(1<<CSL_CPPIDMA_GLOBAL_CONFIG_EMULATION_CONTROL_REG_SOFT_SHIFT)
		|(0<<CSL_CPPIDMA_GLOBAL_CONFIG_EMULATION_CONTROL_REG_FREE_SHIFT);

	for(i= 0; i<uiNumAifChannel; i++)
	{
		KeyStone_pktDma_RxCh_enable(aifDmaRxChCfgRegs, i);
		KeyStone_pktDma_TxCh_enable(aifDmaTxChCfgRegs, i);

#if 0
		/*Enable Tx AIF Specific Monolithic Packet Mode*/
		aifDmaTxChCfgRegs->TX_CHANNEL_GLOBAL_CONFIG[i].TX_CHANNEL_GLOBAL_CONFIG_REG_B= 
			(1<<CSL_CPPIDMA_TX_CHANNEL_CONFIG_TX_CHANNEL_GLOBAL_CONFIG_REG_B_TX_AIF_MONO_MODE_SHIFT);
#endif
	}

	for(i=0; i< uiNumUsedRxFlow; i++)
	{
		KeyStone_pktDma_configureRxFlow(
			&aifDmaRxFlowCfgRegs->RX_FLOW_CONFIG[i], 
			&flowCfgTable[flowTable[i]], 1);
	}
	
	AIF_Check_Free_Queues(); 	//for debug
}

char * fdqName[] =
{
	"DDR_FDQ",
	"SL2_FDQ",
	"CORE0_LL2_FDQ",
	"CORE1_LL2_FDQ",
	"CORE2_LL2_FDQ",
	"CORE3_LL2_FDQ"
};
void Check_Queue(Uint32 uiQIdx, char * qName, Uint32 uiExpectEntryCount)
{
	Uint32 uiEntryCount;
	
	uiEntryCount= 
			gpQueueStatusConfigRegs[uiQIdx].REG_A_EntryCount;
	if(uiEntryCount!=uiExpectEntryCount)
		printf("%s entry count = %d (initial value %d)\n", 
			qName, uiEntryCount, uiExpectEntryCount);
}

/*check the descriptors in the queue, for debug purpose*/
void AIF_Check_Free_Queues()
{
	int i;
	
	_mfence(); 	//force all memory operation complete
	_mfence(); 	//force all memory operation complete
	for(i=0; i<AIF_NUM_RX_FLOW; i++)
	{
		Check_Queue(AIF_FDQ_BASE+i, fdqName[i],
			freeQueueCfgTable[i].uiDescriptorNumber);
	}

}

char * rxQName[] =
{
	"DDR",
	"SL2",
	"CORE0_LL2",
	"CORE1_LL2",
	"CORE2_LL2",
	"CORE3_LL2"
};
/*check the descriptors in the queue, for debug purpose*/
void AIF_Check_TxRx_Queues()
{
	int i;
	Uint32 uiQuNum, uiEntryCount, uiDescriptor;

	
	_mfence(); 	//force all memory operation complete
	_mfence(); 	//force all memory operation complete
	for(i= 0; i< uiNumAifChannel; i++)
	{
		uiEntryCount= 
				gpQueueStatusConfigRegs[QMSS_AIF_QUEUE_BASE+i].REG_A_EntryCount;
		if(uiEntryCount)
		{
			printf("AIF TX queue %d entry count = %d\n", i, uiEntryCount);
		}
			
	}

	for(i=0; i<sizeof(flowCfgTable)/sizeof(PktDma_RxFlowCfg) ; i++)
	{
		uiQuNum= flowCfgTable[i].rx_dest_qnum;
		uiEntryCount= gpQueueStatusConfigRegs[uiQuNum].REG_A_EntryCount;
		if(uiEntryCount)
			printf("number of descriptors in %s RX queue = %d\n", 
				rxQName[i], uiEntryCount);

		uiDescriptor= KeyStone_queuePop(uiQuNum);
		while(uiDescriptor)
		{
			KeyStone_ReturnQueue(uiDescriptor);
			uiDescriptor= KeyStone_queuePop(uiQuNum);
		}
	}

	uiEntryCount= 
			gpQueueStatusConfigRegs[0].REG_A_EntryCount;
	if(uiEntryCount)
		printf("Queue 0 entry count = %d\n", uiEntryCount);

}


