
/*  ============================================================================
 *   Copyright (c) Texas Instruments Inc 2011
 *
 *   Use of this software is controlled by the terms and conditions found in the
 *   license agreement under which this software has been supplied.
* ============================================================================
Example to show the initialization of Multicore Navigator on KeyStone DSP
* =============================================================================
 * Created by :                                                             *
 *            Brighton Feng                                                 *
 *            Texas Instruments                                             * 
 *            August 4, 2011                                                 *
 * ============================================================================
 */
#ifndef _AIF_PKT_DMA_INIT_H_
#define _AIF_PKT_DMA_INIT_H_

#include <csl_qm_queue.h>
#include "KeyStone_Navigator_init_drv.h"

#define QUEUE_ACC_THRESH            128

#define LTE_20MHz_SAMPLES_PER_SYMBOL 		(512+2048)

/*size of descriptor must be multiple of 16B*/
#define AIF_MONO_PACKET_SIZE 	(4*LTE_20MHz_SAMPLES_PER_SYMBOL+16)

/*the linking RAM entry number for each descriptor region 
must be power of 2, and no less than 32.
Actual descriptor number may less than linking entry number, this may 
leave hole in linking RAM, but it still works*/
#define SL2_NUM_MONO_PACKET 			(128+32)
#define LL2_NUM_MONO_PACKET 			(64+8)

/*intend to support 24(AxC) * 2(TX/RX) * 7(Symbols) * 2(ping/pong) = 672 */
#define DDR_NUM_MONO_PACKET 			(1024) 	

typedef enum
{
	AIF_RX_FLOW_DDR        ,
	AIF_RX_FLOW_SL2        ,
	AIF_RX_FLOW_CORE0_LL2  ,
	AIF_RX_FLOW_CORE1_LL2  ,
	AIF_RX_FLOW_CORE2_LL2  ,
	AIF_RX_FLOW_CORE3_LL2  ,
	AIF_NUM_RX_FLOW
}AifRxFlow;

#define TEST_QUEUE_BASE 	2048

/*Low priority accumulation will be used for these queue,
so, their index start at 32 boundary, and use 32 queues per channel*/
#define AIF_RX_FLOW0_DST_Q 	(TEST_QUEUE_BASE)
#define AIF_FDQ_BASE    	(TEST_QUEUE_BASE+ 32)

typedef enum
{
	DDR_FDQ= (AIF_FDQ_BASE),
	SL2_FDQ,
	CORE0_LL2_FDQ,
	CORE1_LL2_FDQ,
	CORE2_LL2_FDQ,
	CORE3_LL2_FDQ,

	QUEUE_AFTER_FDQ	
}AifFdq;
#define RECLAMATION_QUEUE 		(QUEUE_AFTER_FDQ)

extern CSL_Cppidma_global_configRegs * aifDmaCfgRegs;
extern CSL_Cppidma_rx_channel_configRegs * aifDmaRxChCfgRegs;
extern CSL_Cppidma_rx_flow_configRegs * aifDmaRxFlowCfgRegs;
extern CSL_Cppidma_tx_channel_configRegs * aifDmaTxChCfgRegs;
extern volatile Uint32 * aifDmaTxChPriority;
extern PKT_DMA_Regs aifPktDmaRegs;

extern Uint32 uiNumUsedDescRegion;
extern Uint32 uiNumInitialFdq;
extern Uint32 uiNumUsedRxFlow;
extern Uint32 uiaAifDescAccList[];
extern Uint32 uiQueueAccThresh;
extern char * flow_str[];
extern char * fdqName[];
extern unsigned char flowTable[];

/*AIF Queue initialization*/
extern void AIF_Queue_init();

extern void AIF_PktDMA_init();

/*check the descriptors in the queue, for debug purpose*/
extern void AIF_Check_Free_Queues();

/*check the descriptors in the queue, for debug purpose*/
extern void AIF_Check_TxRx_Queues();

#endif
