/****************************************************************************\
 *           Copyright (C) 2011, 2012 Texas Instruments Incorporated.             *
 *                           All Rights Reserved                            *
 *                                                                          *
 * GENERAL DISCLAIMER                                                       *
 * ------------------                                                       *
 * All software and related documentation is provided "AS IS" and without   *
 * warranty or support of any kind and Texas Instruments expressly disclaims*
 * all other warranties, express or implied, including, but not limited to, *
 * the implied warranties of merchantability and fitness for a particular   *
 * purpose.  Under no circumstances shall Texas Instruments be liable for   *
 * any incidental, special or consequential damages that result from the    *
 * use or inability to use the software or related documentation, even if   *
 * Texas Instruments has been advised of the liability.                     *
  ****************************************************************************
This example shows data transfer over AIF2 between two DSP on Dual C6670 EVM.
The examples can also be run on C6670 or TCI6614 EVM, but only internal loopback 
test can be run on those boards because there is only one DSP on them.
This file includes code for sending packets and processing RX packets,
TX/RX packets statistics information is collected and summarized
 ****************************************************************************
 * Created by :                                                             *
 *            Brighton Feng                                                 *
 *            Texas Instruments                                             * 
 *            August 10, 2011                                                 *
Updated on June 18, 2012 for TCI6614 by Brighton
***************************************************************************/
#include <stdio.h>
#include <string.h>
#include "KeyStone_common.h"
#include "AIF_PktDMA_init.h"
#include "aif_test_config.h"
#include "aif_Data.h"
#include "aif_intc.h"
#include "aif_debug.h"
#include "aif_setup.h"

//Aif state for 128 channels, one more stat buffer for unknown channel
AifChStat      aifChStat[129];

Uint32 uiAxcSymbolIndex=0; 	/*symbol index will be filled into PS field of packet*/
Uint32 uiAxcSequenceNum=0; 	/*sequence number be transfered for error checking*/
Uint32 uiGenericSequenceNum=0; /*sequence number be transfered for error checking*/

#pragma DATA_SECTION(errLog,".far:AifDebug")//use DDR3 memory for test/debug
#define ERROR_LOG_SIZE 	16 	/*must be power of 2*/
ErrorLog errLog[ERROR_LOG_SIZE]; 	//for debug
Uint32 uiErrLogIndex= 0;

unsigned char * ucpBufPointer;

void Data_Init()
{
	memset(aifChStat, 0, sizeof(aifChStat));

	uiAxcSymbolIndex= 0;
	uiAxcSequenceNum= 0;
	uiGenericSequenceNum=0;

	uiErrLogIndex= 0;
}

/*Send AxC packets for all enabled channels*/
void SendAxcPackets()
{
	int i, j;
	Uint32 uiPacketSize;
	AifTestInfo testInfo;
	
	testInfo.bGenericPacket= FALSE;
	testInfo.uiMagicNumber= AIF2_MAGIC_NUMBER;

	/*get packet size from look up table according to symbol index*/
	uiPacketSize= packetSizeLut[uiAxcSymbolIndex];

	for(i=0; i<NUM_AIF2_LINK; i++)	//for all links
	{
		if(0==aifLinkCfg[i].linkEnable)
			continue;

		/*for redirection test, DSP1 just redirect the data from DSP 0,
		DSP1 does not generate data*/
		if((0!=uiDSPNum)&&(TEST_PATH_EXTERNAL_REDIRECTION==aifLinkCfg[i].testDataPath))
			continue;

		if(AIF2_GENERIC_DATA_ONLY==aifLinkCfg[i].testDataType)
			continue;

		testInfo.testDataPath= aifLinkCfg[i].testDataPath;
		for(j=aifLinkCfg[i].firstAxcNum; 
			j<aifLinkCfg[i].firstAxcNum+ aifLinkCfg[i].numberAxC; j++)
		{
			testInfo.uiChannelNum= j;
			testInfo.uiSymNum= uiAxcSymbolIndex;
			/*scale packet size according to LTE bandwidth*/
			testInfo.uiPacketSize= uiPacketSize >> aifLinkCfg[i].lteBandwidth;
			testInfo.uiSequenceNum= uiAxcSequenceNum&0x3FFF;
			aifChStat[j].uiNumTxPackets= uiAxcSequenceNum;
			Init_Send_AxC_Packet(&testInfo);
		}
	}
	
	uiAxcSequenceNum++;
	uiAxcSymbolIndex++;
	if(uiAxcSymbolIndex >= FRMAE_NUM_PACKET)
		uiAxcSymbolIndex= 0;
}

/*Send generic packets for all enabled channels*/
void SendGenericPackets()
{
	int i, j;
	AifTestInfo testInfo;

	testInfo.bGenericPacket= TRUE;
	testInfo.uiMagicNumber= AIF2_MAGIC_NUMBER;
	testInfo.uiSymNum= 0; 	/*Symbol number is not used for generic packet*/

	for(i=0; i<NUM_AIF2_LINK; i++)	//for all links
	{
		if(0==aifLinkCfg[i].linkEnable)
			continue;

		/*for redirection test, DSP1 just redirect the data from DSP 0,
		DSP1 does not generate data*/
		if((0!=uiDSPNum)&&(TEST_PATH_EXTERNAL_REDIRECTION==aifLinkCfg[i].testDataPath))
			continue;

		if(AIF2_AXC_DATA_ONLY==aifLinkCfg[i].testDataType)
			continue;

		testInfo.testDataPath= aifLinkCfg[i].testDataPath;
		testInfo.uiPacketSize= aifLinkCfg[i].genericPacketSize;
		for(j=aifLinkCfg[i].firstGenericPktChNum; 
			j<aifLinkCfg[i].firstGenericPktChNum+ aifLinkCfg[i].numberGenericPktCh; j++)
		{
			testInfo.uiChannelNum= j;
			testInfo.uiSequenceNum= uiGenericSequenceNum;
			Init_Send_Generic_Packet(&testInfo);
		}
	}
	uiGenericSequenceNum++;
}

//process data in Rx Queue, recycle the descriptor, and thus get one descriptor 
Uint32 RecycleDescFromRxQueue(Uint32 flowIndex)
{
	int i;
	Uint32 uiDescriptor;
	Uint32 uiFDQ, uiRxQ;

	uiRxQ= AIF_RX_FLOW0_DST_Q+ flowIndex;
	
	for(i=0; i<4; i++)
	{
		uiDescriptor= KeyStone_queuePop(uiRxQ);
		if(NULL==uiDescriptor)
			break;

		Process_Rx_Packet(uiDescriptor);
	}

	uiFDQ= AIF_FDQ_BASE+ flowIndex;
	uiDescriptor =KeyStone_queuePop(uiFDQ);

	if(0==uiDescriptor)
	{
		AIF_Check_Free_Queues(); 	//for debug
		AIF_Check_TxRx_Queues(); 	//for debug
	}

	return uiDescriptor;
}

/*initialize packets and push them to TX queue*/
void Init_Send_AxC_Packet(AifTestInfo * testInfo)
{
	Uint32 uiDescriptor;
	Uint32 uiFDQ, uiTxQ;
	unsigned char * ucpBufPointer;
	AIF_Protocol_Descriptor *aifDescriptor;
	MonolithicPacketDescriptor * Descriptor;

	uiFDQ = AIF_FDQ_BASE+ flowTable[testInfo->uiChannelNum];
	uiDescriptor =KeyStone_queuePop(uiFDQ);
	if(0==uiDescriptor)
	{//process data in Rx Queue, recycle the descriptor, and thus get one descriptor 
		uiDescriptor =RecycleDescFromRxQueue(flowTable[testInfo->uiChannelNum]);

		if(0==uiDescriptor)
		{
			printf("Source queue %s descriptor is NULL for channel %d\n", 
				fdqName[uiFDQ-AIF_FDQ_BASE], testInfo->uiChannelNum);
			return;
		}
	}

	/*invalid cache before read descriptor RAM*/
	InvalidCache((void *)uiDescriptor, 64);

	Descriptor= (MonolithicPacketDescriptor *) uiDescriptor;
	Descriptor->psv_word_count= 1;
	aifDescriptor = (AIF_Protocol_Descriptor *)(Descriptor+1);
	aifDescriptor->egress= 1;
	aifDescriptor->axcNum= testInfo->uiChannelNum;
	aifDescriptor->symNum= testInfo->uiSymNum;	
	ucpBufPointer = ((unsigned char *)Descriptor) + 16;
	Descriptor->data_offset= 16;

	Descriptor->packet_length= testInfo->uiPacketSize;
	
	/*the SRC_TAG_LO field in the Tx descriptor is used as RX flow ID
	for loopback/infrastructure mode transfer*/
	Descriptor->src_tag_lo= 
		(testInfo->uiChannelNum);//for buffer DDR data via SL2
	
	/*put the testInfo int the packet,
	it will be verified at the RX side*/
	memcpy(ucpBufPointer, testInfo, sizeof(AifTestInfo));

	/*write back data from cache to buffer*/
	WritebackCache((void *)ucpBufPointer, 64);

#if 0 	/*for debug only*/
	int i;
	for(i=64; i< testInfo->uiPacketSize; i+=16)
	{
		_amem8(&ucpBufPointer[i])= i; 
		_amem8(&ucpBufPointer[i+8])= 0x1111111111111111; 
	}
	WritebackCache((void *)(ucpBufPointer+64), testInfo->uiPacketSize-64);
#endif

	/*write back data from cache to descriptor RAM*/
	WritebackCache((void *)uiDescriptor, 64);

	uiTxQ = QMSS_AIF_QUEUE_BASE+testInfo->uiChannelNum;
	
	_mfence(); 	//force all memory operation complete
	_mfence(); 	//force all memory operation complete
	KeyStone_queuePush(uiTxQ, uiDescriptor|FETCH_SIZE_64);
}

/*initialize packets and push them to TX queue*/
void Init_Send_Generic_Packet(AifTestInfo * testInfo)
{
	Uint32 uiDescriptor;
	Uint32 uiFDQ, uiTxQ;
	unsigned char * ucpBufPointer;
	MonolithicPacketDescriptor * Descriptor;

	uiFDQ = AIF_FDQ_BASE+ flowTable[testInfo->uiChannelNum];
	uiDescriptor =KeyStone_queuePop(uiFDQ);
	if(0==uiDescriptor)
	{//process data in Rx Queue, recycle the descriptor, and thus get one descriptor 
		uiDescriptor =RecycleDescFromRxQueue(flowTable[testInfo->uiChannelNum]);

		if(0==uiDescriptor)
		{
			printf("Source queue %s descriptor is NULL for channel %d\n", 
				fdqName[uiFDQ-AIF_FDQ_BASE], testInfo->uiChannelNum);
			return;
		}
	}

	/*invalid cache before read descriptor RAM*/
	InvalidCache((void *)uiDescriptor, 64);

	Descriptor= (MonolithicPacketDescriptor *) uiDescriptor;
	ucpBufPointer = (unsigned char *)(Descriptor+1);
	Descriptor->data_offset= 
		sizeof(MonolithicPacketDescriptor);

	Descriptor->packet_length= testInfo->uiPacketSize;
	Descriptor->psv_word_count= 0;
	
	/*the SRC_TAG_LO field in the Tx descriptor is used as RX flow ID
	for loopback/infrastructure mode transfer*/
	Descriptor->src_tag_lo= 
		(testInfo->uiChannelNum);//for buffer DDR data via SL2
	
	/*put the testInfo int the packet,
	it will be verified at the RX side*/
	memcpy(ucpBufPointer, testInfo, sizeof(AifTestInfo));

	/*write back data from cache to buffer*/
	WritebackCache((void *)ucpBufPointer, 64);

	/*write back data from cache to descriptor RAM*/
	WritebackCache((void *)uiDescriptor, 64);

	uiTxQ = QMSS_AIF_QUEUE_BASE+testInfo->uiChannelNum;
	
	_mfence(); 	//force all memory operation complete
	_mfence(); 	//force all memory operation complete
	KeyStone_queuePush(uiTxQ, uiDescriptor|FETCH_SIZE_64);

	aifChStat[testInfo->uiChannelNum].uiNumTxPackets++;
}

/*process received packet*/
void Process_Rx_Packet(Uint32 uiDescriptor)
{
	Uint32 uiPacketLength, uiChannelNum, uiErrorFlag;
	AifTestInfo * testInfo; 
	Uint32 uiTxQ;
	Bool bBadPacket= FALSE;
	AIF_Protocol_Descriptor *aifDescriptor;
	MonolithicPacketDescriptor * Descriptor;

	/*invalid cache before read descriptor RAM*/
	InvalidCache((void *)uiDescriptor, 64);

	Descriptor= (MonolithicPacketDescriptor *) uiDescriptor;
	ucpBufPointer = (unsigned char *)(uiDescriptor+
		Descriptor->data_offset);

	uiPacketLength= Descriptor->packet_length;
	uiErrorFlag= Descriptor->err_flags;

	/*test information received, need be verified*/
	testInfo = (AifTestInfo *)ucpBufPointer;

	/*invalid cache before read test data*/
	InvalidCache((void *)testInfo, 64);

	/*if there is PS information, get channel number in the PS,
	otherwise get channel number from test information*/
	if(Descriptor->psv_word_count)
	{
		aifDescriptor= (AIF_Protocol_Descriptor *)(Descriptor+1);
		uiChannelNum= aifDescriptor->axcNum;
	}
	else
		uiChannelNum= testInfo->uiChannelNum;

	/*channel number must less then number of channel used*/
	if(uiChannelNum >= uiNumAifChannel)
		uiChannelNum= uiNumAifChannel; /*force channel number to be the first unused channel number*/

	if(0==aifChStat[uiChannelNum].uiSequenceNum)
		aifChStat[uiChannelNum].uiSequenceNum = testInfo->uiSequenceNum&0x3FFF;
	else
		aifChStat[uiChannelNum].uiSequenceNum++;

	/*verify received information*/
	if((testInfo->uiSequenceNum&0x3FFF) != (aifChStat[uiChannelNum].uiSequenceNum&0x3FFF))
	{
		bBadPacket= TRUE;
		aifChStat[uiChannelNum].dataErrCnt.uiSequenceNumErrCnt++;
		aifChStat[uiChannelNum].uiSequenceNum= testInfo->uiSequenceNum&0x3FFF;
	}
	
	if(uiErrorFlag)
	{
		bBadPacket = TRUE;
		aifChStat[uiChannelNum].dataErrCnt.uiErrFlagCnt++;
	}
	
	if(testInfo->uiPacketSize != uiPacketLength)
	{
		bBadPacket = TRUE;
		aifChStat[uiChannelNum].dataErrCnt.uiPacketSizeErrCnt++;
	}
	
	if(testInfo->uiMagicNumber != AIF2_MAGIC_NUMBER)
	{
		bBadPacket = TRUE;
		aifChStat[uiChannelNum].dataErrCnt.uiMagicNumberErrCnt++;
	}
	
	if(bBadPacket)
	{
		aifChStat[uiChannelNum].uiNumBadPackets++;

		//log error information for debug
		if(uiErrLogIndex<ERROR_LOG_SIZE&&uiFrameCount>10)
		{
			errLog[uiErrLogIndex].uiChannelNum= uiChannelNum;
			errLog[uiErrLogIndex].uiSymbolCount= uiSymbolCount;
			errLog[uiErrLogIndex].uiErrorFlag= uiErrorFlag;
			errLog[uiErrLogIndex].uiPacketSizeInDesc= uiPacketLength;
			errLog[uiErrLogIndex].uiNumRxPackets= aifChStat[uiChannelNum].uiNumRxPackets;

			memcpy(&errLog[uiErrLogIndex].testInfo, testInfo, sizeof(AifTestInfo));
			uiErrLogIndex++;
		}
	}
	else
	{
		aifChStat[uiChannelNum].uiNumGoodPackets++;
	}
	aifChStat[uiChannelNum].ullNumBytes += uiPacketLength;
	aifChStat[uiChannelNum].uiNumRxPackets++;

	/* on DSP0, Generic packet will always be retransmited when it is received, 
	on DSP1, it is only retransmitted if it is for internal loopback test*/
	if((TRUE == testInfo->bGenericPacket) && ((0 == uiDSPNum)||
		((0 != uiDSPNum)&&(TEST_PATH_INTERNAL_LOOPBACK==testInfo->testDataPath))))
	{
		/*resend the packet*/
		testInfo->uiChannelNum = uiChannelNum;
		testInfo->uiSequenceNum = aifChStat[uiChannelNum].uiNumTxPackets;
		aifChStat[uiChannelNum].uiNumTxPackets++;
		
		Descriptor->err_flags= 0;

		/*the SRC_TAG_LO field in the Tx descriptor is used as RX flow ID
		for loopback/infrastructure mode transfer*/
		Descriptor->src_tag_lo= 
			(uiChannelNum);//for buffer DDR data via SL2

		/*write back data from cache to buffer*/
		WritebackCache((void *)testInfo, 64);

		/*write back data from cache to descriptor RAM*/
		WritebackCache((void *)uiDescriptor, 64);

		uiTxQ = QMSS_AIF_QUEUE_BASE+uiChannelNum;

		_mfence(); 	//force all memory operation complete
		_mfence(); 	//force all memory operation complete
		KeyStone_queuePush(uiTxQ, uiDescriptor|FETCH_SIZE_64);
	}
	else
	{
		/*recycle RX packet buffer*/
		KeyStone_ReturnQueue(uiDescriptor);
	}
}

void PrintChannelResult(Uint32 uiChNum)
{
	aifChStat[uiChNum].uiBandwidth= aifChStat[uiChNum].ullNumBytes/((TEST_NUM_FRAME-1)*10000);
	
	printf("Channel %d transfer %d packets, receive %d packets (%d good, %d bad), %lld bytes in %s, achieve %d MB/s\n",
		uiChNum, aifChStat[uiChNum].uiNumTxPackets, aifChStat[uiChNum].uiNumRxPackets, 
		aifChStat[uiChNum].uiNumGoodPackets, aifChStat[uiChNum].uiNumBadPackets, 
		aifChStat[uiChNum].ullNumBytes,	flow_str[flowTable[uiChNum]],
		aifChStat[uiChNum].uiBandwidth);
}

void PrintTestResult()
{
	Int32 i, j;
	Uint32 uiTotalBandwidth=0, uiLinkBandwidth;
	Uint32 uiTotalGoodPackets=0, uiLinkGoodPackets;
	Uint32 uiTotalBadPackets=0, uiLinkBadPackets;

	print_AIF_status(); 	//for debug
	AIF_Check_Free_Queues();	//for debug
	AIF_Check_TxRx_Queues(); 	//for debug

	for(i=0; i<NUM_AIF2_LINK; i++)	//for all links
	{
		if(0==aifLinkCfg[i].linkEnable)
			continue;

		uiLinkBandwidth= 0;
		uiLinkGoodPackets= 0;
		uiLinkBadPackets= 0;

		if(AIF2_GENERIC_DATA_ONLY != aifLinkCfg[i].testDataType)
		{
			for(j=aifLinkCfg[i].firstAxcNum; 
				j<aifLinkCfg[i].firstAxcNum+ aifLinkCfg[i].numberAxC; j++)
			{
				if(aifChStat[j].uiNumRxPackets)
				{
					printf("  AxC ");
					PrintChannelResult(j);
					uiLinkBandwidth+= aifChStat[j].uiBandwidth;
					uiLinkGoodPackets+= aifChStat[j].uiNumGoodPackets;
					uiLinkBadPackets+= aifChStat[j].uiNumBadPackets;
				}
			}
		}

		if(AIF2_AXC_DATA_ONLY != aifLinkCfg[i].testDataType)
		{
			for(j=aifLinkCfg[i].firstGenericPktChNum; 
				j<aifLinkCfg[i].firstGenericPktChNum+ aifLinkCfg[i].numberGenericPktCh; j++)
			{
				if(aifChStat[j].uiNumRxPackets)
				{
					printf("  generic ");
					PrintChannelResult(j);
					uiLinkBandwidth+= aifChStat[j].uiBandwidth;
					uiLinkGoodPackets+= aifChStat[j].uiNumGoodPackets;
					uiLinkBadPackets+= aifChStat[j].uiNumBadPackets;
				}
			}
		}

		printf("Throughput of link %d = %d MB/s (%d good packets, %d bad packets)\n", 
			i, uiLinkBandwidth, uiLinkGoodPackets, uiLinkBadPackets);
		uiTotalBandwidth+= uiLinkBandwidth;
		uiTotalGoodPackets+= uiLinkGoodPackets;
		uiTotalBadPackets+= uiLinkBadPackets;
	}
	printf("Total throughput of AIF = %d MB/s (%d good packets, %d bad packets)\n", 
		uiTotalBandwidth, uiTotalGoodPackets, uiTotalBadPackets);

}


