/****************************************************************************\
 *           Copyright (C) 2011, 2012 Texas Instruments Incorporated.             *
 *                           All Rights Reserved                            *
 *                                                                          *
 * GENERAL DISCLAIMER                                                       *
 * ------------------                                                       *
 * All software and related documentation is provided "AS IS" and without   *
 * warranty or support of any kind and Texas Instruments expressly disclaims*
 * all other warranties, express or implied, including, but not limited to, *
 * the implied warranties of merchantability and fitness for a particular   *
 * purpose.  Under no circumstances shall Texas Instruments be liable for   *
 * any incidental, special or consequential damages that result from the    *
 * use or inability to use the software or related documentation, even if   *
 * Texas Instruments has been advised of the liability.                     *
 ****************************************************************************
 * Created by :                                                             *
 *            Brighton Feng                                                 *
 *            Texas Instruments                                             * 
 *            August 10, 2011                                                 *
Updated on June 18, 2012 for TCI6614 by Brighton
\***************************************************************************/
#ifndef _AIF_DATA_H
#define _AIF_DATA_H
#include <tistdtypes.h>

/*magic number indicate the start of a symbol*/
#define AIF2_MAGIC_NUMBER 	0x0F1F2F3F

/*Number of generic packets pushed into TX queue,
those packets will circulate in the system,
TX queue -> AIF -> RX queue -> TX queue -> AIF -> ......*/
#define NUM_GENERIC_PACKETS_IN_TX_QUUE 	5

typedef struct
{
	Uint16 uiMagicNumberErrCnt;
	Uint16 uiSequenceNumErrCnt;
	Uint16 uiPacketSizeErrCnt;
	Uint16 uiErrFlagCnt;
}AifDataErrCnt;

typedef struct
{
	Uint32 uiNumTxPackets;
	Uint32 uiSequenceNum;
	Uint32 uiNumRxPackets;
	Uint32 uiNumGoodPackets;
	Uint32 uiNumBadPackets;
	Uint32 uiBandwidth;
	unsigned long long ullNumBytes;
	AifDataErrCnt dataErrCnt;
}AifChStat;

/*test information fill at TX side, transfered as payload, verify at RX side*/
typedef struct
{
	Uint32 uiMagicNumber; 	/*indication of payload head*/
	Uint32 uiSequenceNum;
	Uint16 uiChannelNum;
	Uint16 uiSymNum;
	Uint32 uiPacketSize;
	Bool bGenericPacket;
	TestDataPath testDataPath; 	
}AifTestInfo;

/*error information log, for debug*/
typedef struct
{
	Uint32 uiSymbolCount;
	Uint32 uiNumRxPackets;
	Uint16 uiErrorFlag;
	Uint16 uiChannelNum;
	Uint16 uiPacketSizeInDesc;
	AifTestInfo testInfo;
}ErrorLog;

//Aif state for 128 channels
extern AifChStat      aifChStat[];

extern void Data_Init();

extern void SendAxcPackets();
extern void SendGenericPackets();

/*initialize packets and push them to TX queue*/
extern void Init_Send_AxC_Packet(AifTestInfo * testInfo);
extern void Init_Send_Generic_Packet(AifTestInfo * testInfo);

/*process received packet*/
extern void Process_Rx_Packet(Uint32 uiDescriptor);

extern void PrintTestResult();

#endif
