/****************************************************************************\
 *           Copyright (C) 2011, 2012 Texas Instruments Incorporated.             *
 *                           All Rights Reserved                            *
 *                                                                          *
 * GENERAL DISCLAIMER                                                       *
 * ------------------                                                       *
 * All software and related documentation is provided "AS IS" and without   *
 * warranty or support of any kind and Texas Instruments expressly disclaims*
 * all other warranties, express or implied, including, but not limited to, *
 * the implied warranties of merchantability and fitness for a particular   *
 * purpose.  Under no circumstances shall Texas Instruments be liable for   *
 * any incidental, special or consequential damages that result from the    *
 * use or inability to use the software or related documentation, even if   *
 * Texas Instruments has been advised of the liability.                     *
*****************************************************************************
This file setup interrupt controller, and includes ISR for Frame/Slot/Symbol,
data transfer are triggered by them. ISR of QMSS accumulation triggers RX 
packet processing.
****************************************************************************
 * Created by :                                                             *
 *            Brighton Feng                                                 *
 *            Texas Instruments                                             * 
 *            August 10, 2011                                                 *
Updated on June 18, 2012 for TCI6614 by Brighton
***************************************************************************/
#include <stdio.h>
#include <c6x.h>
#include <tistdtypes.h>
#include <csl_aif2HwControlAux.h>
#include <csl_cpintc.h>
#include "KeyStone_common.h"
#include "AIF_PktDMA_init.h"
#include "aif_setup.h"
#include "aif_test_config.h"
#include "aif_debug.h"
#include "aif_data.h"
#include "aif_intc.h"

#define NUM_AIF_ERR_STS_LOG  	128
#pragma DATA_SECTION(err_sts_log,".far:AifDebug")
Aif_Error_Status_Log err_sts_log[NUM_AIF_ERR_STS_LOG];
Uint32 err_sts_log_cnt=0;

/*minimal AIF2 error/status interrupt processing interval in cycles,
for interrupt pacing*/
#define MIN_AIF2_ERR_STS_INT_PROC_INTERVAL 1000000 
/*maximum temprary error/status interrupts bufferred*/
#define MAX_TEMP_AIF2_ERR_STS_INT 	64 

/*to avoid too many error interrupts processing overloads the DSP, 
an interrupt pacing mechanism is implemented. 
The error status flag from interrupt status registers are masked into 
following temprary flag variables if the last intterrupt is very close 
to this interrupt.*/
Uint32 AifErrSts[NUM_ERR_STS_REGS]; 	//error status flag temprary buffer
Uint32 AifErrOrgn= 0;  	//temprary flag for origin of error status

/*Time Stamp Count of last AIF2 error/status interrupt*/
Uint32 uiLastAifErrIntTSC= 0;
/*Aif2 error/status interrupt count*/
Uint32 uiAifErrIntCnt=0;

void AIF_Interrupts_Init(void)
{
	/*-----------------------CP_INTC configuration---------------------*/
	/* Disable Global host interrupts. */
	gpCIC0_regs->GLOBAL_ENABLE_HINT_REG= 0;

	/* Configure no nesting support in the CPINTC Module. */
	gpCIC0_regs->CONTROL_REG= ((gpCIC0_regs->CONTROL_REG&
		~CSL_CPINTC_CONTROL_REG_NEST_MODE_MASK)|
		(CPINTC_NO_NESTING<<CSL_CPINTC_CONTROL_REG_NEST_MODE_SHIFT));

	/*route AIF2 error/alarm interrupt, CIC in 133, to CIC out 64*/
	KeyStone_CIC_event_map(gpCIC0_regs, 133, 64);

	/* Enable Global host interrupts. */
	gpCIC0_regs->GLOBAL_ENABLE_HINT_REG= 1;

	/*-----------------------Core INTC configuration---------------------*/
	/*map AIF AT event 2,3,4,5 to INT4,5,6,7*/
	gpCGEM_regs->INTMUX1 =
		(CSL_GEM_AIF_EVENT2<<CSL_CGEM_INTMUX1_INTSEL4_SHIFT)|
		(CSL_GEM_AIF_EVENT3<<CSL_CGEM_INTMUX1_INTSEL5_SHIFT)|
		(CSL_GEM_AIF_EVENT4<<CSL_CGEM_INTMUX1_INTSEL6_SHIFT)|
		(CSL_GEM_AIF_EVENT5<<CSL_CGEM_INTMUX1_INTSEL7_SHIFT);


	/*map descriptor accumulation low priority channel 0 interrupt to INT8
	map AIF2 error/alarm interruptm CIC out 64, to DSP core interrupt 9*/
	gpCGEM_regs->INTMUX2 =
		(CSL_GEM_QM_INT_LOW_0<<CSL_CGEM_INTMUX2_INTSEL8_SHIFT)
		|(22<<CSL_CGEM_INTMUX2_INTSEL9_SHIFT);

	//map local timer interrupt to INT14
	gpCGEM_regs->INTMUX3 = CSL_GEM_TINTLN<<CSL_CGEM_INTMUX3_INTSEL14_SHIFT;

	//enable interrupts
	CPU_interrupt_enable((1<<4)|(1<<5)|(1<<6)|(1<<7)|(1<<8)|(1<<9)|(1<<14));
}

Uint32 uiSymbolCount=0;
interrupt void AIF_Symbol_ISR()
{
	uiSymbolCount++;


	/*AIF2 takes at least one frame for synchronization, so lets begin
	real data transfer at frame boundary 2,	push packet one symbol ahead*/
	if((FRMAE_NUM_PACKET*2-1)<=uiSymbolCount)
	{
		SendAxcPackets();
	}
}

Uint32 uiUlSymbolSlotCount=0;
/*uplink radio timer symbol or slot interrupt*/
interrupt void AIF_Uplink_Symbol_Slot_ISR()
{
	uiUlSymbolSlotCount++;
}

/*first slot interrupt happens on frame 1 boundary, it is actually for 
slot with index "LTE_FRAME_NUM_SLOT"*/
Uint32 uiSlotCount=LTE_FRAME_NUM_SLOT-1;
interrupt void AIF_Slot_ISR()
{
	int i;
	
	uiSlotCount++;

	/*Generic packet data must be pushed after frame synchronized,
	here skip 5 frames for it*/
	if(LTE_FRAME_NUM_SLOT*5+1==uiSlotCount)
	{
		for(i=0; i<NUM_GENERIC_PACKETS_IN_TX_QUUE; i++)
			SendGenericPackets();	
	}	


}

volatile Bool bTerminating= FALSE; 	//Is AIF2 in the progress of terminating?
Uint32 uiFrameCount=0;
Uint32 uiTestNumFrame=TEST_NUM_FRAME;
interrupt void AIF_Frame_ISR()
{
	int i;
	
	uiFrameCount++;

 	for(i=0; i<NUM_AIF2_LINK; i++)	//for all links
	{
		if(0==aifLinkCfg[i].linkEnable)
			continue;

		if(FALSE==aifLinkCfg[i].bPDEnabled)
			Enable_PD_After_Sync(i);
	}

	/*test terminate after run for number of frames*/
	if(uiFrameCount>=uiTestNumFrame)
	{
		/*copy registers to memory to keep the values when the transfer
		complete, avoid registers value change during print*/
		copyAif2ErrStsRegs(); 	//for debug

		bTerminating= TRUE; 	//set up flag to terminate AIF2

		/*disable all AIF2 related interrupts execpt for the RX interrupt,
		let Rx interrupt to recycle all packet descriptors to FDQ*/
		IER = 3|(1<<8);
	}
}

/*ping/pong switch of descriptor accumulation buffer*/
Uint32 uiAccPingPong=0; 	
/*Accumulation interrupt for AIF RX packets*/
interrupt void AIF_Data_ISR()
{
	int i;
	Uint32 qmIntStatus1;
	Uint32 uiDescriptor, uiDescNum;

	/*read interrupt status*/
	qmIntStatus1= gpQM_INTD_regs->STATUS_REG1;

	/*clear interrupt status*/
	gpQM_INTD_regs->STATUS_CLR_REG1= qmIntStatus1;

	/*low priority channel 0 interrupt*/
	if(1&qmIntStatus1)
	{
		//clear interrupt		
		gpQM_INTD_regs->INTCNT_REG[32]= 1 ;
		gpQM_INTD_regs->EOI_REG= 34;

		/*The first list entry is used to store the total list entry count*/
		uiDescNum= uiaAifDescAccList[(uiAccPingPong&1)*(uiQueueAccThresh+1)];
		for(i=1; i<uiDescNum+1; i++)
		{
			uiDescriptor= uiaAifDescAccList[(uiAccPingPong&1)*
				(uiQueueAccThresh+1)+i];

			if(bTerminating||((FRMAE_NUM_PACKET*2-1)>uiSymbolCount))
				/*return the descriptor to free queue when AIF is terminating
				or TX is not start*/
				KeyStone_ReturnQueue(uiDescriptor&0xFFFFFFF0);				
			else
				Process_Rx_Packet(uiDescriptor&0xFFFFFFF0);
		}
		_mfence(); 	//force all memory operation complete
		_mfence(); 	//force all memory operation complete
		uiAccPingPong++;
	} 
}

void clear_AIF_error_status()
{
	int i;
	Uint32 uiReg;
	
    aif2Regs->EE_DB_IRS_CLR= 0xFFFFFFFF;
    aif2Regs->EE_AD_IRS_CLR= 0xFFFFFFFF;
    aif2Regs->EE_CD_IRS_CLR= 0xFFFFFFFF;
    aif2Regs->EE_SD_IRS_CLR= 0xFFFFFFFF;
    aif2Regs->EE_VC_IRS_CLR= 0xFFFFFFFF;
    aif2Regs->EE_AT_IRS_CLR= 0xFFFFFFFF;

	for(i=0; i<NUM_AIF2_LINK; i++) 	//for all links
	{
		aif2Regs->EE_LK[i].EE_LK_IRS_CLR_A= 0xFFFFFFFF;
		aif2Regs->EE_LK[i].EE_LK_IRS_CLR_B= 0xFFFFFFFF;

		/*reset line code violation counter*/
		uiReg= aif2Regs->G_RM_LKS[i].RM_LK_CFG0;
		aif2Regs->G_RM_LKS[i].RM_LK_CFG0 = 
			uiReg&(~(CSL_AIF2_RM_LK_CFG0_LCV_CNTR_EN_MASK));
		aif2Regs->G_RM_LKS[i].RM_LK_CFG0 = 
			uiReg|CSL_AIF2_RM_LK_CFG0_LCV_CNTR_EN_MASK;
	}

	err_sts_log_cnt= 0;

	AifErrOrgn= 0;
	for(i=0; i<NUM_ERR_STS_REGS; i++)
	{
		AifErrSts[i]= 0;
	}
}

void Enable_AIF_error_status_interrupt()
{
	int i;
	
    aif2Regs->EE_DB_EN_SET_EV0=
		 (1<<CSL_AIF2_EE_DB_EN_SET_EV0_DB_EE_I_TRC_RAM_OVFL_ERR_SHIFT)
		|(1<<CSL_AIF2_EE_DB_EN_SET_EV0_DB_EE_I_TOKEN_OVFL_ERR_SHIFT  )
		|(1<<CSL_AIF2_EE_DB_EN_SET_EV0_DB_EE_I_FIFO_OVFL_ERR_SHIFT   )
		|(1<<CSL_AIF2_EE_DB_EN_SET_EV0_DB_EE_I_PD2DB_FULL_ERR_SHIFT  )
		|(1<<CSL_AIF2_EE_DB_EN_SET_EV0_DB_EE_E_PS_AXC_ERR_SHIFT      )
		|(0<<CSL_AIF2_EE_DB_EN_SET_EV0_DB_EE_E_CD_DATA_ERR_SHIFT     )
		|(1<<CSL_AIF2_EE_DB_EN_SET_EV0_DB_EE_E_CD_DATA_TYPE_ERR_SHIFT);
    aif2Regs->EE_AD_EN_SET_EV0= 
		 (0<<CSL_AIF2_EE_AD_EN_SET_EV0_AD_EE_I_CD_DATA_ERR_SHIFT           )
		|(0<<CSL_AIF2_EE_AD_EN_SET_EV0_AD_EE_E_CD_SCH_ERR_SHIFT            )
		|(1<<CSL_AIF2_EE_AD_EN_SET_EV0_AD_EE_I_DMA_0_ERR_SHIFT             )
		|(1<<CSL_AIF2_EE_AD_EN_SET_EV0_AD_EE_I_DMA_1_ERR_SHIFT             )
		|(1<<CSL_AIF2_EE_AD_EN_SET_EV0_AD_EE_I_DMA_2_ERR_SHIFT             )
		|(1<<CSL_AIF2_EE_AD_EN_SET_EV0_AD_EE_E_DMA_0_ERR_SHIFT             )
		|(1<<CSL_AIF2_EE_AD_EN_SET_EV0_AD_EE_E_DMA_1_ERR_SHIFT             )
		|(1<<CSL_AIF2_EE_AD_EN_SET_EV0_AD_EE_E_DMA_2_ERR_SHIFT             );
    aif2Regs->EE_CD_EN_SET_EV= 
		 (1<<CSL_AIF2_EE_CD_EN_SET_EV_CD_EE_SOP_DESC_STARVE_ERR_SHIFT      )
		|(1<<CSL_AIF2_EE_CD_EN_SET_EV_CD_EE_MOP_DESC_STARVE_ERR_SHIFT      );
    aif2Regs->EE_SD_EN_SET_EV0= 
		 (0<<CSL_AIF2_EE_SD_EN_SET_EV0_SD_EE_STSPLL_B4_ERR_SHIFT           )
		|(0<<CSL_AIF2_EE_SD_EN_SET_EV0_SD_EE_STSPLL_B8_ERR_SHIFT           );
    aif2Regs->EE_VC_EN_SET_EV0= 
		 (1<<CSL_AIF2_EE_VC_EN_SET_EV0_VC_EE_VBUS_ERR_SHIFT                );
    aif2Regs->EE_AT_EN_SET_EV0= 
		 (1<<CSL_AIF2_EE_AT_EN_SET_EV0_AT_EE_RP1_TYPE_SYS_RCVD_ERR_SHIFT   )
		|(1<<CSL_AIF2_EE_AT_EN_SET_EV0_AT_EE_RP1_TYPE_RP3_RCVD_ERR_SHIFT   )
		|(1<<CSL_AIF2_EE_AT_EN_SET_EV0_AT_EE_RP1_TYPE_TOD_RCVD_ERR_SHIFT   )
		|(1<<CSL_AIF2_EE_AT_EN_SET_EV0_AT_EE_RP1_TYPE_UNSEL_ERR_SHIFT      )
		|(1<<CSL_AIF2_EE_AT_EN_SET_EV0_AT_EE_RP1_TYPE_SPARE_ERR_SHIFT      )
		|(1<<CSL_AIF2_EE_AT_EN_SET_EV0_AT_EE_RP1_TYPE_RSVD_ERR_SHIFT       )
		|(1<<CSL_AIF2_EE_AT_EN_SET_EV0_AT_EE_RP1_BIT_WIDTH_ERR_SHIFT       )
		|(1<<CSL_AIF2_EE_AT_EN_SET_EV0_AT_EE_RP1_CRC_ERR_SHIFT             )
		|(1<<CSL_AIF2_EE_AT_EN_SET_EV0_AT_EE_RP1_RP3_ERR_SHIFT             )
		|(1<<CSL_AIF2_EE_AT_EN_SET_EV0_AT_EE_RP1_SYS_ERR_SHIFT             )
		|(1<<CSL_AIF2_EE_AT_EN_SET_EV0_AT_EE_PI0_ERR_SHIFT                 )
		|(1<<CSL_AIF2_EE_AT_EN_SET_EV0_AT_EE_PI1_ERR_SHIFT                 )
		|(1<<CSL_AIF2_EE_AT_EN_SET_EV0_AT_EE_PI2_ERR_SHIFT                 )
		|(1<<CSL_AIF2_EE_AT_EN_SET_EV0_AT_EE_PI3_ERR_SHIFT                 )
		|(1<<CSL_AIF2_EE_AT_EN_SET_EV0_AT_EE_PI4_ERR_SHIFT                 )
		|(1<<CSL_AIF2_EE_AT_EN_SET_EV0_AT_EE_PI5_ERR_SHIFT                 )
		|(1<<CSL_AIF2_EE_AT_EN_SET_EV0_AT_EE_PHYT_SYNC_ERR_SHIFT           )
		|(1<<CSL_AIF2_EE_AT_EN_SET_EV0_AT_EE_RADT_SYNC_ERR_SHIFT           );

	for(i=0; i<NUM_AIF2_LINK; i++)	//for all links
	{
		if(!aifLinkCfg[i].linkEnable)
			continue;

	    aif2Regs->EE_LK[i].EE_LK_EN_A_SET_EV0= 
			 (1<<CSL_AIF2_EE_LK_EN_A_SET_EV0_RM_EE_SYNC_STATUS_CHANGE_ERR_SHIFT)
			|(1<<CSL_AIF2_EE_LK_EN_A_SET_EV0_RM_EE_NUM_LOS_DET_ERR_SHIFT       )
			|(0<<CSL_AIF2_EE_LK_EN_A_SET_EV0_RM_EE_LCV_DET_ERR_SHIFT           )
			|(0<<CSL_AIF2_EE_LK_EN_A_SET_EV0_RM_EE_FRAME_BNDRY_DET_ERR_SHIFT   )
			|(0<<CSL_AIF2_EE_LK_EN_A_SET_EV0_RM_EE_BLOCK_BNDRY_DET_ERR_SHIFT   )
			|(1<<CSL_AIF2_EE_LK_EN_A_SET_EV0_RM_EE_MISSING_K28P5_ERR_SHIFT     )
			|(1<<CSL_AIF2_EE_LK_EN_A_SET_EV0_RM_EE_MISSING_K28P7_ERR_SHIFT     )
			|(0<<CSL_AIF2_EE_LK_EN_A_SET_EV0_RM_EE_K30P7_DET_ERR_SHIFT         )
			|(1<<CSL_AIF2_EE_LK_EN_A_SET_EV0_RM_EE_LOC_DET_ERR_SHIFT           )
			|(1<<CSL_AIF2_EE_LK_EN_A_SET_EV0_RM_EE_RX_FIFO_OVF_ERR_SHIFT       )
			|(0<<CSL_AIF2_EE_LK_EN_A_SET_EV0_RM_EE_RCVD_LOS_ERR_SHIFT          )
			|(0<<CSL_AIF2_EE_LK_EN_A_SET_EV0_RM_EE_RCVD_LOF_ERR_SHIFT          )
			|(0<<CSL_AIF2_EE_LK_EN_A_SET_EV0_RM_EE_RCVD_RAI_ERR_SHIFT          )
			|(0<<CSL_AIF2_EE_LK_EN_A_SET_EV0_RM_EE_RCVD_SDI_ERR_SHIFT          )
			|(0<<CSL_AIF2_EE_LK_EN_A_SET_EV0_RM_EE_LOS_ERR_SHIFT               )
			|(0<<CSL_AIF2_EE_LK_EN_A_SET_EV0_RM_EE_LOF_ERR_SHIFT               )
			|(1<<CSL_AIF2_EE_LK_EN_A_SET_EV0_RM_EE_HFNSYNC_STATE_ERR_SHIFT     )
			|(1<<CSL_AIF2_EE_LK_EN_A_SET_EV0_RM_EE_LOF_STATE_ERR_SHIFT         )
			#ifdef AIF2_LINK_PROTOCOL_OBSAI
			|(0<<CSL_AIF2_EE_LK_EN_A_SET_EV0_TM_EE_FRM_MISALIGN_ERR_SHIFT      )
			#else
			|(1<<CSL_AIF2_EE_LK_EN_A_SET_EV0_TM_EE_FRM_MISALIGN_ERR_SHIFT      )
			#endif
			|(1<<CSL_AIF2_EE_LK_EN_A_SET_EV0_TM_EE_FIFO_STARVE_ERR_SHIFT       );
	    aif2Regs->EE_LK[i].EE_LK_EN_B_SET_EV0= 
			 (1<<CSL_AIF2_EE_LK_EN_B_SET_EV0_PD_EE_EOP_ERR_SHIFT               )
			|(1<<CSL_AIF2_EE_LK_EN_B_SET_EV0_PD_EE_CRC_ERR_SHIFT               )
			|(1<<CSL_AIF2_EE_LK_EN_B_SET_EV0_PD_EE_CPRI_FRAME_ERR_SHIFT        )
			|(1<<CSL_AIF2_EE_LK_EN_B_SET_EV0_PD_EE_AXC_FAIL_ERR_SHIFT          )
			|(1<<CSL_AIF2_EE_LK_EN_B_SET_EV0_PD_EE_SOP_ERR_SHIFT               )
			|(1<<CSL_AIF2_EE_LK_EN_B_SET_EV0_PD_EE_OBSAI_FRM_ERR_SHIFT         )
			|(0<<CSL_AIF2_EE_LK_EN_B_SET_EV0_PD_EE_WR2DB_ERR_SHIFT             )
			|(1<<CSL_AIF2_EE_LK_EN_B_SET_EV0_PE_EE_MODRULE_ERR_SHIFT           )
			|(1<<CSL_AIF2_EE_LK_EN_B_SET_EV0_PE_EE_SYM_ERR_SHIFT               )
			|(1<<CSL_AIF2_EE_LK_EN_B_SET_EV0_PE_EE_MF_FIFO_OVERFLOW_ERR_SHIFT  )
			|(1<<CSL_AIF2_EE_LK_EN_B_SET_EV0_PE_EE_MF_FIFO_UNDERFLOW_ERR_SHIFT )
			|(1<<CSL_AIF2_EE_LK_EN_B_SET_EV0_PE_EE_DB_STARVE_ERR_SHIFT         )
			|(1<<CSL_AIF2_EE_LK_EN_B_SET_EV0_PE_EE_RT_IF_ERR_SHIFT             )
			|(1<<CSL_AIF2_EE_LK_EN_B_SET_EV0_PE_EE_PKT_STARVE_ERR_SHIFT        )
			|(1<<CSL_AIF2_EE_LK_EN_B_SET_EV0_RT_EE_FRM_ERR_SHIFT               )
			|(1<<CSL_AIF2_EE_LK_EN_B_SET_EV0_RT_EE_OVFL_ERR_SHIFT              )
			|(1<<CSL_AIF2_EE_LK_EN_B_SET_EV0_RT_EE_UNFL_ERR_SHIFT              )
			|(1<<CSL_AIF2_EE_LK_EN_B_SET_EV0_RT_EE_EM_ERR_SHIFT                )
			|(1<<CSL_AIF2_EE_LK_EN_B_SET_EV0_RT_EE_HDR_ERR_SHIFT               );
	}

	aif2Regs->EE_VB_EOI= 0;	
}

/*save AIF2 error/status into log buffer*/
void AIF_error_record()
{
	int i;
	Uint32 frame_cnt;
	Uint32 symbol_clock_cnt;

	frame_cnt= (aif2Regs->AT_RADT_VALUE_MSBS<<12)
				|(aif2Regs->AT_RADT_VALUE_MID&0xFFF);
	symbol_clock_cnt= aif2Regs->AT_RADT_VALUE_LSBS;

	/*ignore the error at the very begining*/
	if((frame_cnt==0)&&(symbol_clock_cnt<100)) 	return;

	for(i=0; i<NUM_ERR_STS_REGS; i++)
	{
		if(((AifErrOrgn>>i)&1)&&(err_sts_log_cnt<NUM_AIF_ERR_STS_LOG))
		{
			err_sts_log[err_sts_log_cnt].IRS_index= i;
			err_sts_log[err_sts_log_cnt].IRS_value= AifErrSts[i];
			err_sts_log[err_sts_log_cnt].frame_cnt= frame_cnt;
			err_sts_log[err_sts_log_cnt].AT_RADT_VALUE_LSBS= symbol_clock_cnt;

			AifErrSts[i]= 0;

			err_sts_log_cnt++;
		}
	}
	AifErrOrgn= 0;
}

/*lookup table of IRS registers address*/
Aif_INT_STS * INT_STS_REGS[NUM_ERR_STS_REGS]=
{
	(Aif_INT_STS *)(CSL_AIF2_CONTROL_REGS+0x40000),	/*EE_LK_STS_A0*/
	(Aif_INT_STS *)(CSL_AIF2_CONTROL_REGS+0x4002C),	/*EE_LK_STS_B0*/
	(Aif_INT_STS *)(CSL_AIF2_CONTROL_REGS+0x40080),	/*EE_LK_STS_A1*/
	(Aif_INT_STS *)(CSL_AIF2_CONTROL_REGS+0x400AC),	/*EE_LK_STS_B1*/
	(Aif_INT_STS *)(CSL_AIF2_CONTROL_REGS+0x40100),	/*EE_LK_STS_A2*/
	(Aif_INT_STS *)(CSL_AIF2_CONTROL_REGS+0x4012C),	/*EE_LK_STS_B2*/
	(Aif_INT_STS *)(CSL_AIF2_CONTROL_REGS+0x40180),	/*EE_LK_STS_A3*/
	(Aif_INT_STS *)(CSL_AIF2_CONTROL_REGS+0x401AC),	/*EE_LK_STS_B3*/
	(Aif_INT_STS *)(CSL_AIF2_CONTROL_REGS+0x40200),	/*EE_LK_STS_A4*/
	(Aif_INT_STS *)(CSL_AIF2_CONTROL_REGS+0x4022C),	/*EE_LK_STS_B4*/
	(Aif_INT_STS *)(CSL_AIF2_CONTROL_REGS+0x40280),	/*EE_LK_STS_A5*/
	(Aif_INT_STS *)(CSL_AIF2_CONTROL_REGS+0x402AC),	/*EE_LK_STS_B5*/
	(Aif_INT_STS *)(CSL_AIF2_CONTROL_REGS+0x40300),	/*EE_AT_STS   */
	(Aif_INT_STS *)(CSL_AIF2_CONTROL_REGS+0x4400),	/*EE_SD_STS   */
	(Aif_INT_STS *)(CSL_AIF2_CONTROL_REGS+0x4100),	/*EE_DB_STS   */
	(Aif_INT_STS *)(CSL_AIF2_CONTROL_REGS+0x4200),	/*EE_AD_STS   */
	(Aif_INT_STS *)(CSL_AIF2_CONTROL_REGS+0x4300),	/*EE_CD_STS   */
	(Aif_INT_STS *)(CSL_AIF2_CONTROL_REGS+0x4500)	/*EE_VC_STS   */
};

interrupt void AIF2_Error_Alarm_ISR()
{
	int i;
	Uint32 err_orgn_flag;
	Uint32 EN_INT_STS; 	//enabled interrupt status

	if(err_sts_log_cnt>=NUM_AIF_ERR_STS_LOG)
		return;
		
	/* Disable the host interrupt in CIC*/
	gpCIC0_regs->HINT_ENABLE_CLR_INDEX_REG = 64;

	/*check error/status*/
	err_orgn_flag= aif2Regs->EE_ERR_ALRM_ORGN;
	AifErrOrgn |= err_orgn_flag;
	for(i=0; i<NUM_ERR_STS_REGS; i++)
	{
		if((err_orgn_flag>>i)&1)
		{
			/*CD interrupt registers are different, need special handle*/
			if(EE_CD_STS==i) 	
				EN_INT_STS= ((Aif_CD_INT_STS *)(INT_STS_REGS[i]))->EN_STS_EV;
			else
				EN_INT_STS= INT_STS_REGS[i]->EN_STS_EV0;

			/*clear the flag*/
			INT_STS_REGS[i]->IRS_CLR= EN_INT_STS;

			/*OR flag from register into temprary variable*/
			AifErrSts[i]|= EN_INT_STS;
		}
	}

	/* Clear the status of system event in CIC*/
	gpCIC0_regs->STATUS_CLR_INDEX_REG= 133;
	
	/* Enable the host interrupt in CIC*/
	gpCIC0_regs->HINT_ENABLE_SET_INDEX_REG = 64;

	aif2Regs->EE_VB_EOI= 0;	

	/*save the error/status into log if the time is long them interrupt
	process interval or interrupts happened more then MAX times*/
	uiAifErrIntCnt++;
	if((TSC_getDelay(uiLastAifErrIntTSC)>MIN_AIF2_ERR_STS_INT_PROC_INTERVAL)
		||(uiAifErrIntCnt>MAX_TEMP_AIF2_ERR_STS_INT))
	{
		AIF_error_record();
		uiLastAifErrIntTSC= TSCL;
		uiAifErrIntCnt=0;
	}
}

void Clear_AIF2_INTC()
{
	int i;
	
	/* Disable the host interrupt in CIC*/
	gpCIC0_regs->HINT_ENABLE_CLR_INDEX_REG = 64;

	/*clear AIF2 raw error/alarm flags*/
	clear_AIF_error_status();

	/* Clear the status of system event in CIC*/
	gpCIC0_regs->STATUS_CLR_INDEX_REG= 133;
	
	/* Enable the host interrupt in CIC*/
	gpCIC0_regs->HINT_ENABLE_SET_INDEX_REG = 64;

	for(i=0; i<NUM_ERR_STS_REGS; i++)
		AifErrSts[i]= 0;
	AifErrOrgn= 0;		
	uiAifErrIntCnt=0;
}

//Interrupt service routine for timer
Uint32 uiTimerCount=0;
void interrupt Timer_ISR(void)
{
	uiTimerCount++;
}

void Counter_Init()
{
	uiSymbolCount= 0;
	uiUlSymbolSlotCount=0;
	/*first slot interrupt happens on frame 1 boundary, it is actually for 
	slot with index "LTE_FRAME_NUM_SLOT"*/
	uiSlotCount= LTE_FRAME_NUM_SLOT-1;
	uiFrameCount= 0;
}

