/****************************************************************************\
 *           Copyright (C) 2011, 2012 Texas Instruments Incorporated.             *
 *                           All Rights Reserved                            *
 *                                                                          *
 * GENERAL DISCLAIMER                                                       *
 * ------------------                                                       *
 * All software and related documentation is provided "AS IS" and without   *
 * warranty or support of any kind and Texas Instruments expressly disclaims*
 * all other warranties, express or implied, including, but not limited to, *
 * the implied warranties of merchantability and fitness for a particular   *
 * purpose.  Under no circumstances shall Texas Instruments be liable for   *
 * any incidental, special or consequential damages that result from the    *
 * use or inability to use the software or related documentation, even if   *
 * Texas Instruments has been advised of the liability.                     *
 ****************************************************************************
 * Created by :                                                             *
 *            Brighton Feng                                                 *
 *            Texas Instruments                                             * 
 *            August 4, 2011                                                *
Updated on June 18, 2012 for TCI6614 by Brighton
\***************************************************************************/
#ifndef AIF_TEST_CONFIG_H
#define AIF_TEST_CONFIG_H
#include <tistdtypes.h>
#include <csl_aif2.h>

/*number of 10ms frame for test*/
#define TEST_NUM_FRAME 		200

#define AIF2_TRIGGER_MANUALLY

/*Number of AIF links be used <= 6*/
#define NUM_AIF2_LINK       6

/*--------select between LTE normal symbol and extended symbol--------*/
#if 1
#define LTE_NORMAL_SYMBOL_FRAME
#else
#define LTE_EXTENDED_SYMBOL_FRAME
#endif

#define LTE_20MHZ_EXTENDED_SYMBOL_NUM_MSG      (2560/4)
#define LTE_20MHZ_FIRST_NORMAL_SYMBOL_NUM_MSG  (2208/4)
#define LTE_20MHZ_OTHER_NORMAL_SYMBOL_NUM_MSG  (2192/4)
#define LTE_SLOT_NUM_EXTENDED_SYMBOL     (6)
#define LTE_SLOT_NUM_NORMAL_SYMBOL       (7)
#define LTE_FRAME_NUM_SUBFRAME           (10)
#define LTE_FRAME_NUM_SLOT               (20)
#define LTE_FRAME_NUM_EXTENDED_SYMBOL    (LTE_SLOT_NUM_EXTENDED_SYMBOL*LTE_FRAME_NUM_SLOT)
#define LTE_FRAME_NUM_NORMAL_SYMBOL      (LTE_SLOT_NUM_NORMAL_SYMBOL*LTE_FRAME_NUM_SLOT)

#define LTE_SUBFRAME_NUM_CLOCK           (FRAME_CLOCK_COUNT/LTE_FRAME_NUM_SUBFRAME)
#define LTE_SLOT_NUM_CLOCK               (FRAME_CLOCK_COUNT/LTE_FRAME_NUM_SLOT)

#define LTE_SAMPLE_NUM_CLK               (FRAME_CLOCK_COUNT*100/30720000)
#define LTE_EXTENDED_SYMBOL_NUM_CLK      (2560*LTE_SAMPLE_NUM_CLK)
#define LTE_FIRST_NORMAL_SYMBOL_NUM_CLK  (2208*LTE_SAMPLE_NUM_CLK)
#define LTE_OTHER_NORMAL_SYMBOL_NUM_CLK  (2192*LTE_SAMPLE_NUM_CLK)
/*11110011111111001111*/
#define LTE_TDD_DL_SLOT_BITMAP           (0xF3FCF) 

#ifdef LTE_EXTENDED_SYMBOL_FRAME
#define SLOT_NUM_PACKET  				LTE_SLOT_NUM_EXTENDED_SYMBOL
#define FRMAE_NUM_PACKET 				LTE_FRAME_NUM_EXTENDED_SYMBOL
#else
#define SLOT_NUM_PACKET 				LTE_SLOT_NUM_NORMAL_SYMBOL
#define FRMAE_NUM_PACKET 				LTE_FRAME_NUM_NORMAL_SYMBOL
#endif

#define PKTDMA_OVERHEAD             150
#define L2_DELAY               		30
#define DDR_DELAY               	50

#define PI_WINDOW                   200


//AIF timing const
#define PE1_PE2_DELAY               10
#define PE2_TM_DELAY                60
#define SERDES_2X_DELAY             24
#define SERDES_4X_DELAY             15
#define SERDES_8X_DELAY             10
#define PROPAGATION_DELAY 			1 		/*delay on board or fiber*/
#define RM_PE1_DELAY                100 	/*15~500*/

#ifdef AIF2_LINK_PROTOCOL_OBSAI
#define AXC_OFFSET_WIN              1000
#define DB_PE2_DELAY                28
#define FRAME_CLOCK_COUNT           3072000
#define OBSAI_TYPE_LTE                   0x0E
#define OBSAI_TYPE_GENERIC               0x0F
#else
#define AXC_OFFSET_WIN              300
#define DB_PE2_DELAY                0
#define FRAME_CLOCK_COUNT           2457600
#endif

typedef enum
{
	/* normal operation */
	TEST_PATH_NORMAL_OPERATION = 0,
	/* internal loopback test */
	TEST_PATH_INTERNAL_LOOPBACK,
	/*internal redirection test, the data path is: 
	Rx Serdes -> DSP RM -> DSP TM -> TX Serdes */
	TEST_PATH_INTERNAL_REDIRECTION,
	/*external redirection test, the data path is: 
	DSP0 PE -> DSP0 TM -> DSP1 RM -> DSP1 TM -> DSP0 RM -> DSP0 PD */
	TEST_PATH_EXTERNAL_REDIRECTION,
	/* external line loopback test */
	TEST_PATH_EXTERLNAL_LINE_LOOPBACK
} TestDataPath;

typedef enum
{
	LTE_20MHZ = 0,
	LTE_10MHZ,
	LTE_5MHZ
} LTE_Bandwidth;

typedef enum
{
	/*antenna data*/
	AIF2_AXC_PACKET = 0,
	/*generic data*/
	AIF2_GENERIC_PACKET
} Aif2PacketType;

typedef enum
{
	AIF2_AXC_DATA_ONLY = 0,
	AIF2_GENERIC_DATA_ONLY,
	/*antenna data on AxC slot, generic data on control slot*/
	AIF2_AXC_AND_GENERIC_DATA
} Aif2TestDataType;

/*define bandwidth grade of a channel*/
typedef enum
{
	CHANNEL_8X_LINK_BANDWIDTH = 0,
	CHANNEL_4X_LINK_BANDWIDTH,
	CHANNEL_2X_LINK_BANDWIDTH,
	CHANNEL_1X_LINK_BANDWIDTH,
	CHANNEL_SMALL_BANDWIDTH
} Aif2ChannelBandWidth;

typedef struct _AifConfig
{
	Uint16 				linkEnable; 	/*1: enable, 0 disable*/
	CSL_Aif2LinkRate	linkRate; 		

	TestDataPath 		testDataPath; 	
	Aif2TestDataType 	testDataType;
	Uint32              genericPacketSize;

	LTE_Bandwidth       lteBandwidth;
	Uint32 				numberAxC; 	/*number of AxC's in this link*/

	Uint32 				piOffset; 		
	Uint32 				PeEvent1Offset;
	Uint32 				PeEvent2Offset;
	Uint32 				deltaOffset; 	

	Uint32 				DBM_X; 	/*X value for Dual Bit Map configuration*/
	Uint32 				firstAxcNum; /*first channel number for AxC in this link*/
	Aif2ChannelBandWidth axcChBw; 	/*AxC channel bandwidth, used to determin PE IN FIFO levle*/

	Uint32 				numberGenericPktCh; /*number of packet channels in this link*/
	Uint32 				firstGenericPktChNum; /*first channel number packet in this link*/
	Aif2ChannelBandWidth genericChBw; 	/*generic channel bandwidth, used to determin PE IN FIFO levle*/

	CSL_Aif2RtConfig 	RT_cfg;
	CSL_Aif2PeRtContol 	PE_RT_ctl;

	Bool bEnableTxSyncMater; 	/*first used link should be set as Sync master*/

	Bool bPDEnabled; 	/*enable PD after frame Sync*/
	Uint32 uiSyncFrameCnt; 	/*number of synchronized frames*/
}AifLinkConfig;

//Aif configuration for 6 links
extern AifLinkConfig      aifLinkCfg[6];

extern Uint32 uiNumAifChannel; //Total number of channels used

/*look up table of different link rate*/
extern unsigned char linkRate[4];

extern Uint32 uiDSPNum;

extern Uint16 packetSizeLut[];

//Calculate some other AIF Parameters according above pre-set paremeters
//this function should called before EDMA, FSync and AIF configuration
void InitAifTestParameters();

#endif
