/****************************************************************************\
 *           Copyright (C) 2011, 2012 Texas Instruments Incorporated.             *
 *                           All Rights Reserved                            *
 *                                                                          *
 * GENERAL DISCLAIMER                                                       *
 * ------------------                                                       *
 * All software and related documentation is provided "AS IS" and without   *
 * warranty or support of any kind and Texas Instruments expressly disclaims*
 * all other warranties, express or implied, including, but not limited to, *
 * the implied warranties of merchantability and fitness for a particular   *
 * purpose.  Under no circumstances shall Texas Instruments be liable for   *
 * any incidental, special or consequential damages that result from the    *
 * use or inability to use the software or related documentation, even if   *
 * Texas Instruments has been advised of the liability.                     *
 ****************************************************************************
This example shows data transfer over AIF2 between two DSP on dual Nyquist EVM
The examples can also be run on C6670 or TCI6614 EVM, but only internal loopback 
test can be run on those boards because there is only one DSP on them.
This file includes basic configuration structure for test, user can modify 
the basic parameters to change the test mode to verify most functionality 
of AIF. The codes in this file calculate many other parameters based on the 
basic configuration.
****************************************************************************
 * Created by :                                                             *
 *            Brighton Feng                                                 *
 *            Texas Instruments                                             * 
 *            August 4, 2011                                                *
Updated on June 18, 2012 for TCI6614 by Brighton
***************************************************************************/

#include <stdio.h>
#include <string.h>
#include "aif_test_config.h"
#include "aif_data.h"
#include "AIF_PktDMA_init.h"

/*Change following basic parameters to switch between different test modes
Please note, AIF link 0~1 is connected between the two DSP on dual Nyquist EVM, 
AIF link 2~5 on the dual Nyquist EVM is connected to AMC interface.
Without external connection, a link can only be used for internal loopback test*/
AifLinkConfig aifLinkCfg[6]=
{
	/*Configuration for link 0*/ 
	{ 	
		1, 			/*Link Enable: 1=enable, 0=disable*/
		CSL_AIF2_LINK_RATE_8x,  
		TEST_PATH_INTERNAL_LOOPBACK, 	/*test data path*/
		AIF2_AXC_DATA_ONLY, 			/*testDataType*/
		AIF_MONO_PACKET_SIZE-16,		/*genericPacketSize*/
		LTE_20MHZ,      	/*lteBandwidth*/
		3 					/*numberAxC: 0 means maximum number*/
	},
	/*Configuration for link 1*/
	{ 	
		1, 			/*Link Enable: 1=enable, 0=disable*/
		CSL_AIF2_LINK_RATE_4x,
		TEST_PATH_INTERNAL_LOOPBACK, 	/*test data path*/
		AIF2_AXC_DATA_ONLY, 			/*testDataType*/
		AIF_MONO_PACKET_SIZE-16,		/*genericPacketSize*/
		LTE_10MHZ,      	/*lteBandwidth*/
		0 					/*numberAxC: 0 means maximum number*/
	},
	/*Configuration for link 2*/
	{ 	
		1, 			/*Link Enable: 1=enable, 0=disable*/
		CSL_AIF2_LINK_RATE_2x,
		TEST_PATH_INTERNAL_LOOPBACK, 	/*test data path*/
		AIF2_AXC_AND_GENERIC_DATA, 			/*testDataType*/
		AIF_MONO_PACKET_SIZE-16,		/*genericPacketSize*/
		LTE_5MHZ,      	/*lteBandwidth*/
		0 					/*numberAxC: 0 means maximum number*/
	},
	/*Configuration for link 3*/
	{ 	
		1, 			/*Link Enable: 1=enable, 0=disable*/
		CSL_AIF2_LINK_RATE_4x,
		TEST_PATH_INTERNAL_LOOPBACK,  	/*test data path*/
		AIF2_GENERIC_DATA_ONLY, 			/*testDataType*/
		AIF_MONO_PACKET_SIZE-16,		/*genericPacketSize*/
		LTE_20MHZ,      	/*lteBandwidth*/
		0 					/*numberAxC: 0 means maximum number*/
	},
	/*Configuration for link 4*/
	{ 	
		1, 			/*Link Enable: 1=enable, 0=disable*/
		CSL_AIF2_LINK_RATE_4x,
		TEST_PATH_INTERNAL_LOOPBACK, 	/*test data path*/
		AIF2_AXC_DATA_ONLY, 			/*testDataType*/
		AIF_MONO_PACKET_SIZE-16,		/*genericPacketSize*/
		LTE_20MHZ,      	/*lteBandwidth*/
		0 					/*numberAxC: 0 means maximum number*/
	},
	/*Configuration for link 5*/
	{ 	
		1, 			/*Link Enable: 1=enable, 0=disable*/
		CSL_AIF2_LINK_RATE_4x,
		TEST_PATH_INTERNAL_LOOPBACK, 	/*test data path*/
		AIF2_AXC_DATA_ONLY, 			/*testDataType*/
		AIF_MONO_PACKET_SIZE-16,		/*genericPacketSize*/
		LTE_20MHZ,      	/*lteBandwidth*/
		0 					/*numberAxC: 0 means maximum number*/
	}
};

/*packet size look up table*/
#ifdef LTE_EXTENDED_SYMBOL_FRAME
char radioStdStr[]= "LTE TDD extended cyclic prefix";
Uint16 packetSizeLut[FRMAE_NUM_PACKET]=
{
	LTE_20MHZ_EXTENDED_SYMBOL_NUM_MSG*16,
	LTE_20MHZ_EXTENDED_SYMBOL_NUM_MSG*16,
	LTE_20MHZ_EXTENDED_SYMBOL_NUM_MSG*16,
	LTE_20MHZ_EXTENDED_SYMBOL_NUM_MSG*16,
	LTE_20MHZ_EXTENDED_SYMBOL_NUM_MSG*16,
	LTE_20MHZ_EXTENDED_SYMBOL_NUM_MSG*16
};
#else
char radioStdStr[]= "LTE TDD normal cyclic prefix";
Uint16 packetSizeLut[FRMAE_NUM_PACKET]=
{
	LTE_20MHZ_FIRST_NORMAL_SYMBOL_NUM_MSG*16,
	LTE_20MHZ_OTHER_NORMAL_SYMBOL_NUM_MSG*16,
	LTE_20MHZ_OTHER_NORMAL_SYMBOL_NUM_MSG*16,
	LTE_20MHZ_OTHER_NORMAL_SYMBOL_NUM_MSG*16,
	LTE_20MHZ_OTHER_NORMAL_SYMBOL_NUM_MSG*16,
	LTE_20MHZ_OTHER_NORMAL_SYMBOL_NUM_MSG*16,
	LTE_20MHZ_OTHER_NORMAL_SYMBOL_NUM_MSG*16
};
#endif

/*look up table of different link rate*/
unsigned char linkRate[4]= 
{
	8, /*CSL_AIF2_LINK_RATE_8x*/
	4, /*CSL_AIF2_LINK_RATE_4x*/
	2  /*CSL_AIF2_LINK_RATE_2x*/
};

/*look up table of Serdes delay for different link rate*/
unsigned char SerdesDelay[4]= 
{
	SERDES_8X_DELAY,
	SERDES_4X_DELAY,
	SERDES_2X_DELAY
};

Uint32 uiNumAifChannel; //Total number of channels used

//calculate AIF link timing parameters
void calcAifLinkTimingParameters(AifLinkConfig * aifLinkCfg, Uint32 uiLinkNum)
{
	Uint32 dmaDelay;
	Uint32 peOffset;
	Uint32 tmOffset;
	Uint32 rmOffset;
	
	aifLinkCfg->RT_cfg= CSL_AIF2_RT_MODE_TRANSMIT;
	aifLinkCfg->PE_RT_ctl= CSL_AIF2_PE_RT_INSERT;

	//MAX delay for DIO transfer
	dmaDelay = PKTDMA_OVERHEAD + DDR_DELAY*uiNumAifChannel;

	if(TEST_PATH_NORMAL_OPERATION == aifLinkCfg->testDataPath)
	{
		/*for normal operation, PeEvent1Offset and piOffset
		should be set manually or calculated based on real test scenario*/
		peOffset= dmaDelay;
		aifLinkCfg->PeEvent1Offset= peOffset;

		peOffset = aifLinkCfg->PeEvent1Offset+ PE1_PE2_DELAY;
		aifLinkCfg->PeEvent2Offset= peOffset;

		tmOffset= peOffset + PE2_TM_DELAY;
		aifLinkCfg->deltaOffset= tmOffset;

		/*------------timing of RX node---------------*/
		/*Rx offset= Tx offset on previous node + SerDes delay*/
		rmOffset= tmOffset + SerdesDelay[aifLinkCfg->linkRate];
		aifLinkCfg->piOffset= rmOffset;

	}
	else if((TEST_PATH_INTERNAL_LOOPBACK == aifLinkCfg->testDataPath)
		||(TEST_PATH_EXTERLNAL_LINE_LOOPBACK == aifLinkCfg->testDataPath))
	{
		/*------------timing of TX node---------------*/
		//PePrepSync should start at or after Packet DMA done
		peOffset= dmaDelay;
		aifLinkCfg->PeEvent1Offset= peOffset;

		peOffset += PE1_PE2_DELAY;
		aifLinkCfg->PeEvent2Offset= peOffset;

		tmOffset= peOffset + PE2_TM_DELAY;
		aifLinkCfg->deltaOffset= tmOffset;

		/*------------timing of RX node---------------*/
		/*Rx offset= Tx offset on previous node + SerDes delay*/
		rmOffset= tmOffset + SerdesDelay[aifLinkCfg->linkRate];
		aifLinkCfg->piOffset= rmOffset;
	}
	else if(TEST_PATH_INTERNAL_REDIRECTION == aifLinkCfg->testDataPath)
	{
		aifLinkCfg->RT_cfg= CSL_AIF2_RT_MODE_RETRANSMIT;
		aifLinkCfg->PE_RT_ctl= CSL_AIF2_PE_RT_RETRANS;

		peOffset= aifLinkCfg->piOffset + RM_PE1_DELAY;
		aifLinkCfg->PeEvent1Offset= peOffset;

		peOffset += PE1_PE2_DELAY;
		aifLinkCfg->PeEvent2Offset= peOffset;

		tmOffset= peOffset + PE2_TM_DELAY;
		aifLinkCfg->deltaOffset= tmOffset;
	}
	else	/*TEST_PATH_EXTERNAL_REDIRECTION*/
	{
		if(0!=uiDSPNum)
		{
			aifLinkCfg->RT_cfg= CSL_AIF2_RT_MODE_RETRANSMIT;
			aifLinkCfg->PE_RT_ctl= CSL_AIF2_PE_RT_RETRANS;
		}

		/*------------timing of first TX node---------------*/
		//PePrepSync should start at or after Packet DMA done
		peOffset= dmaDelay;
		if(0==uiDSPNum)
			aifLinkCfg->PeEvent1Offset= peOffset;

		peOffset += PE1_PE2_DELAY;
		if(0==uiDSPNum)
			aifLinkCfg->PeEvent2Offset= peOffset;

		tmOffset= peOffset + PE2_TM_DELAY;
		if(0==uiDSPNum)
			aifLinkCfg->deltaOffset= tmOffset;

		/*------------timing of first RX node---------------*/
		/*Rx offset= Tx offset on previous node + SerDes delay*/
		rmOffset= tmOffset + SerdesDelay[aifLinkCfg->linkRate]
			+ PROPAGATION_DELAY;
		if(0!=uiDSPNum)
			aifLinkCfg->piOffset= rmOffset;

		/*------------timing of second TX node---------------*/
		/*For daisy chain redirection, the TM offset is calculated according 
		to RX offset*/
		peOffset= rmOffset + RM_PE1_DELAY;
		if(0!=uiDSPNum)
			aifLinkCfg->PeEvent1Offset= peOffset;

		peOffset += PE1_PE2_DELAY;
		if(0!=uiDSPNum)
			aifLinkCfg->PeEvent2Offset= peOffset;

		tmOffset= peOffset + PE2_TM_DELAY;
		if(0!=uiDSPNum)
			aifLinkCfg->deltaOffset= tmOffset;

		/*------------timing of last RX node---------------*/
		/*Rx offset= Tx offset on previous node + SerDes delay*/
		rmOffset= tmOffset + SerdesDelay[aifLinkCfg->linkRate]
			+ PROPAGATION_DELAY;
		if(0==uiDSPNum)
			aifLinkCfg->piOffset= rmOffset;
	}
	
}

char * testPathStr[]=
{
	"normal operation",
	"internal loopback test",
	"internal redirection test",
	"external redirection test",
	"external line loopback test"
};

char * dataTypeStr[]=
{
	"antenna data in AxC slot only",
	"generic data in AxC slot only",
	"antenna data on AxC slot, generic data on control slot"
};

/*print test configuration*/
void PrintTestConfiguration()
{
	Int32 i;
#ifdef AIF2_LINK_PROTOCOL_OBSAI
	printf("=================AIF OBSAI mode test for %d ms (%d frames, %s)=================\n",
		TEST_NUM_FRAME*10, TEST_NUM_FRAME, radioStdStr);
#else
	printf("=================AIF CPRI mode test for %d ms (%d frames, %s)=================\n",
		TEST_NUM_FRAME*10, TEST_NUM_FRAME, radioStdStr);
#endif

	for(i=0; i<NUM_AIF2_LINK; i++)	//for all links
	{
		if(1==aifLinkCfg[i].linkEnable)
		{
			printf("link %d runs at %dx rate, %s", i, 
				linkRate[aifLinkCfg[i].linkRate], 
				testPathStr[aifLinkCfg[i].testDataPath]);

			if(AIF2_GENERIC_DATA_ONLY != aifLinkCfg[i].testDataType)
				printf(", LTE %d MHz AxC", 20>>aifLinkCfg[i].lteBandwidth);

			printf(", %s", dataTypeStr[aifLinkCfg[i].testDataType]);

			if(AIF2_AXC_DATA_ONLY != aifLinkCfg[i].testDataType)
				printf(", generic packet size = %d\n",
					aifLinkCfg[i].genericPacketSize);
			else
				printf("\n");
		}
		else
			printf("link %d is disabled\n", i);
	}
}

Uint8 TDD_enable[144]; 	//symbol enable look up table for TDD mode
/*To simplify the LTE TDD DL symbols selction, the DL bitmap can be defined in
slot, this function convert TDD DL slot enable bitmap to symbol enable bitmap*/
void setLteTddDlBitmap(Uint32 uiLteDlSlotBitmap, Uint32 uiSymbolPerSlot)
{
	int i, j;

	for(i=0;i<144;i++)
		TDD_enable[i]= 1; //enable all symbols by default
	
	for(i=0; i<LTE_FRAME_NUM_SLOT; i++)
	{
		/*extend bit map for a slot to all symbols in the slot*/
		for(j=0; j<uiSymbolPerSlot; j++)
			TDD_enable[i*uiSymbolPerSlot + j]= uiLteDlSlotBitmap&1;
	
		uiLteDlSlotBitmap >>= 1;
	}

}

//Calculate some other AIF Parameters according above pre-set paremeters
//this function should called before Packet DMA, and AIF configuration
void InitAifTestParameters()
{
	Int32 i, j;
	Uint32 uiDBM_X=0;
	Int32 iLowNeighbor=-1;

	uiNumAifChannel= 0;

	for(i=0; i<NUM_AIF2_LINK; i++)	//for all links
	{
		if(0==aifLinkCfg[i].linkEnable)
			continue;

		if(AIF2_GENERIC_DATA_ONLY != aifLinkCfg[i].testDataType)
		{
			uiDBM_X= (linkRate[aifLinkCfg[i].linkRate]/2)
				<<aifLinkCfg[i].lteBandwidth;
			if(LTE_20MHZ ==aifLinkCfg[i].lteBandwidth)
				aifLinkCfg[i].axcChBw= CHANNEL_2X_LINK_BANDWIDTH;
			else if(LTE_10MHZ ==aifLinkCfg[i].lteBandwidth)
				aifLinkCfg[i].axcChBw= CHANNEL_1X_LINK_BANDWIDTH;
			else
				aifLinkCfg[i].axcChBw= CHANNEL_SMALL_BANDWIDTH;
			aifLinkCfg[i].DBM_X= uiDBM_X;
			aifLinkCfg[i].firstAxcNum = uiNumAifChannel;

			if((0==aifLinkCfg[i].numberAxC)||(aifLinkCfg[i].numberAxC>uiDBM_X))
				aifLinkCfg[i].numberAxC= uiDBM_X;

			if(aifLinkCfg[i].numberAxC>64) 	/*max AxC per channel is 64*/
				aifLinkCfg[i].numberAxC= 64;

			uiNumAifChannel += aifLinkCfg[i].numberAxC;
		}
		else
		{
			//full link bandwidth is used as one generic data channel
			aifLinkCfg[i].DBM_X= 0;
			aifLinkCfg[i].numberAxC= 0;
			aifLinkCfg[i].firstAxcNum= 0;
		}

		if(AIF2_AXC_DATA_ONLY != aifLinkCfg[i].testDataType)
		{
			if(AIF2_GENERIC_DATA_ONLY == aifLinkCfg[i].testDataType)
			{
				/*In this example, if AxC slots are used for generic packet,
				the bandwidth is divided in to channels equivalent to 2x link,
				for example, a 8x link is divided into 4 channels*/
				uiDBM_X= linkRate[aifLinkCfg[i].linkRate]/2;
				aifLinkCfg[i].DBM_X= uiDBM_X;
				aifLinkCfg[i].numberGenericPktCh= uiDBM_X;
				aifLinkCfg[i].firstGenericPktChNum= uiNumAifChannel;
				uiNumAifChannel += linkRate[aifLinkCfg[i].linkRate]/2;
				aifLinkCfg[i].genericChBw= CHANNEL_2X_LINK_BANDWIDTH;
			}
			else
			{	
				/*if only control slot is used for generic packet,
				only 1 channel is used*/
				aifLinkCfg[i].numberGenericPktCh= 1;
				aifLinkCfg[i].firstGenericPktChNum= uiNumAifChannel++;
				aifLinkCfg[i].genericChBw= CHANNEL_SMALL_BANDWIDTH;
			}

		}
		else
		{
			aifLinkCfg[i].firstGenericPktChNum= 0;
			aifLinkCfg[i].numberGenericPktCh= 0;
		}

		if(uiNumAifChannel > 128)
		{/*maximum channel number = 128*/
			for(j=i; j<6; j++)
				aifLinkCfg[j].linkEnable= 0;

			puts("channel number larger than 128, the links with channel number larger than 128 is disabled!");
			break;
		}

		calcAifLinkTimingParameters(&aifLinkCfg[i], i);

		/*setup Serdes Sync Master according to following rules:
		1. Lanes being aggregated must have set this bit low.
		2. One macro must be nominated as the master.(B8 if it is used)
		3. The master macro must have one lane which tied this high. 
			(lower numbered lane)
		4. All other lanes on all other macros being aggregated should 
			be tied low.
		5. Transmit lanes that have a lower-numbered neighbor of a different 
			rate or width configuration must have set this bit high.*/
		if(-1==iLowNeighbor)/*first used lane*/
			aifLinkCfg[i].bEnableTxSyncMater= TRUE;
		else if(aifLinkCfg[i].linkRate!=
			aifLinkCfg[iLowNeighbor].linkRate)/*different link rate*/
			aifLinkCfg[i].bEnableTxSyncMater= TRUE;
		else/*lane same as previous lane*/
			aifLinkCfg[i].bEnableTxSyncMater= FALSE;
		
		iLowNeighbor= i;

	}

	/*initialize packet size LUT*/
	for(i=SLOT_NUM_PACKET; i<FRMAE_NUM_PACKET; i+=SLOT_NUM_PACKET)
	{
		/*the packet size of each slot in a frame is same. copy the 
		packet size values for first slot to other solts of the frame*/
		for(j=0; j<SLOT_NUM_PACKET; j++)
		{
			packetSizeLut[i+j]= packetSizeLut[j];
		}
	}
	
	/*To simplify the LTE TDD DL symbols selction, the DL bitmap can be defined in
	slot, this function convert TDD DL slot enable bitmap to symbol enable bitmap*/
	setLteTddDlBitmap(LTE_TDD_DL_SLOT_BITMAP, SLOT_NUM_PACKET);

	PrintTestConfiguration();
}


