/*  ============================================================================
 *   Copyright (c) Texas Instruments Inc 2012
 *
 *   Use of this software is controlled by the terms and conditions found in the
 *   license agreement under which this software has been supplied.
* ============================================================================
This file setup Packet DMA and QMSS for AIF2
* =============================================================================
 * Created by :                                                             *
 *            Brighton Feng                                                 *
 *            Texas Instruments                                             * 
 *            Jan 7, 2012                                                 *
 * ============================================================================
 */

#include <stdio.h>
#include <string.h>
#include "KeyStone_common.h"
#include "Aif_test_config.h"
#include "AIF_PktDMA_init.h"
#include "aif_data.h"
#ifdef _BIG_ENDIAN
#include <ti/drv/qmss/firmware/acc48_be_bin.h>
#else
#include <ti/drv/qmss/firmware/acc48_le_bin.h>
#endif

CSL_Cppidma_global_configRegs * aifDmaCfgRegs= 
	(CSL_Cppidma_global_configRegs * )CSL_AIF2_CFG_CPPI_DMA_GLOBAL_CFG_REGS;
CSL_Cppidma_tx_channel_configRegs * aifDmaTxChCfgRegs= 
	(CSL_Cppidma_tx_channel_configRegs * )CSL_AIF2_CFG_CPPI_DMA_TX_CFG_REGS;
CSL_Cppidma_rx_channel_configRegs * aifDmaRxChCfgRegs= 
	(CSL_Cppidma_rx_channel_configRegs * )CSL_AIF2_CFG_CPPI_DMA_RX_CFG_REGS;
volatile Uint32 * aifDmaTxChPriority = 
	(volatile Uint32 *)CSL_AIF2_CFG_CPPI_DMA_TX_SCHEDULER_CFG_REGS;
CSL_Cppidma_rx_flow_configRegs * aifDmaRxFlowCfgRegs= 
	(CSL_Cppidma_rx_flow_configRegs * )CSL_AIF2_CFG_CPPI_DMA_RX_FLOW_CFG_REGS;
PKT_DMA_Regs aifPktDmaRegs=
{
	(CSL_Cppidma_global_configRegs * )CSL_AIF2_CFG_CPPI_DMA_GLOBAL_CFG_REGS,
	(CSL_Cppidma_tx_channel_configRegs * )CSL_AIF2_CFG_CPPI_DMA_TX_CFG_REGS,
	(CSL_Cppidma_rx_channel_configRegs * )CSL_AIF2_CFG_CPPI_DMA_RX_CFG_REGS,
	(volatile Uint32 *)CSL_AIF2_CFG_CPPI_DMA_TX_SCHEDULER_CFG_REGS,
	(CSL_Cppidma_rx_flow_configRegs * )CSL_AIF2_CFG_CPPI_DMA_RX_FLOW_CFG_REGS
};

#pragma DATA_SECTION(monoCtlPacket,".far:ControlData")
#pragma DATA_ALIGN (monoCtlPacket, 128)
Uint8 monoCtlPacket[NUM_AIF_CTL_MONO_PACKET][AIF_MONO_PACKET_SIZE];

/* descriptor memory region configuration.
descriptor Base address must be specified in ascending order,
no overlap is allowed between regions*/
Qmss_DescMemRegionCfg descMemRegionsCfg=
{   /*The base address of descriptor region                 Size of each descriptor   Number of descriptors 
                                                            16*n,1<=n<=8192           2^(5+n), 0<=n<=15*/
    (Uint32)monoCtlPacket,                                 AIF_MONO_PACKET_SIZE,     NUM_AIF_CTL_MONO_PACKET
};

FreeMonoQueueCfg freeQueueCfgTable[]=
{
    /*free queue number*//*address of first descriptor*/                   /*size of descriptor*//*number of descriptors*/
    {AIF_RX_FDQ,     (Uint32)monoCtlPacket,                                AIF_MONO_PACKET_SIZE, NUM_AIF_CTL_MONO_PACKET/2},
    {AIF_TX_FDQ,     (Uint32)&monoCtlPacket[NUM_AIF_CTL_MONO_PACKET/2][0], AIF_MONO_PACKET_SIZE, NUM_AIF_CTL_MONO_PACKET/2}
};

/*flow configuration*/
PktDma_RxFlowCfg flowCfgTable[]=
{
    /*dest_qnum*/    /*sop_offset*//*ps_loc*//*desc_type*//*error retry*//*psinfo*/ /*einfo*//*size_thresh0(1,2)_en*//*size_thresh0,1,2*//*fdq0_sz0_qnum, sz1_qnum, sz2_qnum, sz3_qnum*//*fdq1_qnum*//*fdq2_qnum*//*fdq3_qnum*//*dest_tag_lo*//*dest_tag_hi*//*src_tag_lo*//*src_tag_hi*//*dest_tag_lo_sel*//*dest_tag_hi_sel*//*src_tag_lo_sel*//*src_tag_hi_sel*/
    {AIF_RX_FLOW_DST_Q,  16,          0,  Cppi_DescType_MONOLITHIC,     0,   0     ,     0,      0, 0, 0,                0, 0, 0,           AIF_RX_FDQ,       0, 0, 0,                  AIF_RX_FDQ,   AIF_RX_FDQ,  AIF_RX_FDQ,      0,          0,              0,              0,              0,                  0,                  0,                  0},
    {AIF_RX_FLOW_DST_Q+1,16,          0,  Cppi_DescType_MONOLITHIC,     0,   0     ,     0,      0, 0, 0,                0, 0, 0,           AIF_RX_FDQ,       0, 0, 0,                  AIF_RX_FDQ,   AIF_RX_FDQ,  AIF_RX_FDQ,      0,          0,              0,              0,              0,                  0,                  0,                  0}
};

/*Descriptor accumulation buffer*/
Uint32 uiaAifDescAccList[(QUEUE_ACC_THRESH+1)*2];
Uint32 uiQueueAccThresh = QUEUE_ACC_THRESH;
Uint32 uiNumInitialFdq;
Uint32 uiNumUsedRxFlow;
void QMSS_Accumulation_config()
{
	Uint32 uiReturnCode;
	Qmss_AccCmdCfg loQuAccCfg;

	/*set PDSP tick interval 25us*/
	uiReturnCode= KeyStone_Qmss_config_Acc_Timer(Qmss_PdspId_PDSP1, 4167);
	if(1!=uiReturnCode)
		printf("ACC timer configure return code = %d\n", uiReturnCode);

	loQuAccCfg.channel = 32;
	loQuAccCfg.command = Qmss_AccCmd_ENABLE_CHANNEL;
	loQuAccCfg.queueEnMask = 0x3F;
	loQuAccCfg.listAddress = GLOBAL_ADDR(uiaAifDescAccList);

	/*this value must be multiple of 32 for multiple queue mode*/
	loQuAccCfg.queMgrIndex = AIF_RX_FLOW_DST_Q;
#if 1
	/*Accumulation threshold match to number of channel,
	accumulation interrupt will be generated when all channel receive
	one or two packets*/
	if(uiNumCtlChannel*2 < QUEUE_ACC_THRESH)
		uiQueueAccThresh = uiNumCtlChannel*2;
#endif
	loQuAccCfg.maxPageEntries = uiQueueAccThresh+1;

	loQuAccCfg.timerLoadCount = 2;
	loQuAccCfg.interruptPacingMode = Qmss_AccPacingMode_FIRST_NEW_PACKET;
	loQuAccCfg.listEntrySize = Qmss_AccEntrySize_REG_D;
	loQuAccCfg.listCountMode = Qmss_AccCountMode_ENTRY_COUNT;
	loQuAccCfg.multiQueueMode = Qmss_AccQueueMode_MULTI_QUEUE;
	KeyStone_Qmss_Config_Acc_Channel(Qmss_PdspId_PDSP1, &loQuAccCfg);

}

/*AIF Queue initialization*/
void AIF_Queue_init()
{
	/*-----------------QMSS queue configuration------------------*/
	KeyStone_QMSS_Linking_RAM_init(NULL, 0);

	KeyStone_QMSS_Descriptor_Regions_init(&descMemRegionsCfg, 1);

	uiNumInitialFdq= sizeof(freeQueueCfgTable)/sizeof(FreeMonoQueueCfg);
	KeyStone_Mono_Descriptor_Queues_init(freeQueueCfgTable, uiNumInitialFdq);

	/*-----------------QMSS PDSP fireware configuration------------------*/
#ifdef _BIG_ENDIAN
	KeyStone_Qmss_Download_Firmware(Qmss_PdspId_PDSP1, &acc48_be, sizeof(acc48_be));
#else
	KeyStone_Qmss_Download_Firmware(Qmss_PdspId_PDSP1, &acc48_le, sizeof(acc48_le));
#endif

	/*clear QMSS interrupts*/
	gpQM_INTD_regs->STATUS_CLR_REG0= 0xFFFFFFFF;
	gpQM_INTD_regs->STATUS_CLR_REG1= 0xFFFF;
	
	QMSS_Accumulation_config();

}

/*AIF packet DMA initialization
must be called after AIF PSC enabled*/
void AIF_PktDMA_init()
{
	int i;
	
	uiNumUsedRxFlow= sizeof(flowCfgTable)/sizeof(PktDma_RxFlowCfg);
	if(uiNumUsedRxFlow>AIF_PKTDMA_MAX_RX_FLOW_NUM)
		uiNumUsedRxFlow= AIF_PKTDMA_MAX_RX_FLOW_NUM;
	if(uiNumUsedRxFlow>uiNumAifChannel)
		uiNumUsedRxFlow= uiNumAifChannel;

	/*-----------------AIF PktDMA configuration------------------*/
	KeyStone_pktDma_Global_Control(aifDmaCfgRegs, 256, 0, 0);

	aifDmaCfgRegs->EMULATION_CONTROL_REG= 
		(0<<CSL_CPPIDMA_GLOBAL_CONFIG_EMULATION_CONTROL_REG_LOOPBACK_EN_SHIFT)
		|(1<<CSL_CPPIDMA_GLOBAL_CONFIG_EMULATION_CONTROL_REG_SOFT_SHIFT)
		|(0<<CSL_CPPIDMA_GLOBAL_CONFIG_EMULATION_CONTROL_REG_FREE_SHIFT);

	for(i= uiNumAifChannel-uiNumCtlChannel; i<uiNumAifChannel; i++)
	{
		KeyStone_pktDma_RxCh_enable(aifDmaRxChCfgRegs, i);
		KeyStone_pktDma_TxCh_enable(aifDmaTxChCfgRegs, i);

#if 0
		/*Enable Tx AIF Specific Monolithic Packet Mode*/
		aifDmaTxChCfgRegs->TX_CHANNEL_GLOBAL_CONFIG[i].TX_CHANNEL_GLOBAL_CONFIG_REG_B= 
			(1<<CSL_CPPIDMA_TX_CHANNEL_CONFIG_TX_CHANNEL_GLOBAL_CONFIG_REG_B_TX_AIF_MONO_MODE_SHIFT);
#endif
	}
	//DIO is actually channel 128 of PktDMA
	KeyStone_pktDma_RxCh_enable(aifDmaRxChCfgRegs, 128);
	KeyStone_pktDma_TxCh_enable(aifDmaTxChCfgRegs, 128);

	for(i= uiNumAifChannel-uiNumCtlChannel; i<uiNumAifChannel; i++)
	{
		KeyStone_pktDma_configureRxFlow(
			&aifDmaRxFlowCfgRegs->RX_FLOW_CONFIG[i], 
			&flowCfgTable[i&1], 1);
	}
	
}

/*check the descriptors in the queue, for debug purpose*/
void AIF_Check_Free_Queues()
{
	Uint32 uiEntryCount;
	
	_mfence(); 	//force all memory operation complete
	_mfence(); 	//force all memory operation complete

	uiEntryCount= KeyStone_GetQueueEntryCount(AIF_RX_FDQ);
	if(uiEntryCount!=freeQueueCfgTable[0].uiDescriptorNumber)
		printf("RX free queue entry count = %d (initial value %d)\n", 
			uiEntryCount, freeQueueCfgTable[0].uiDescriptorNumber);

	uiEntryCount= KeyStone_GetQueueEntryCount(AIF_TX_FDQ);
	if(uiEntryCount!=freeQueueCfgTable[1].uiDescriptorNumber)
		printf("TX free queue entry count = %d (initial value %d)\n", 
			uiEntryCount, freeQueueCfgTable[0].uiDescriptorNumber);
}

/*print Non-empty queue for debug*/
void printNonEmptyQueue()
{
	int i;
	int iEntryCount;
	
	for(i= 0; i<8192; i++)
	{
		iEntryCount= KeyStone_GetQueueEntryCount(i);
		if(iEntryCount)
			printf("Queue %d entry count = %d\n", i, iEntryCount);
	}
}

/*print monolithic descriptor for debug*/
void printMonoDescriptor(MonolithicPacketDescriptor * descriptor)
{
	printf("0x%x packet_length  =%d, \
data_offset    =%d, \
packet_type    =%d, \
type_id        =%d, \
dest_tag_lo    =%d, \
dest_tag_hi    =%d, \
src_tag_lo     =%d, \
src_tag_hi     =%d, \
pkt_return_qnum=%d, \
pkt_return_qmgr=%d, \
ret_push_policy=%d, \
reserved_w2b   =%d, \
ps_flags       =%d, \
err_flags      =%d, \
psv_word_count =%d, \
reserved_w2    =%d, \
epib           =%d\n",
	(Uint32)descriptor,
	descriptor->packet_length  ,
	descriptor->data_offset    ,
	descriptor->packet_type    ,
	descriptor->type_id        ,
	descriptor->dest_tag_lo    ,
	descriptor->dest_tag_hi    ,
	descriptor->src_tag_lo     ,
	descriptor->src_tag_hi     ,
	descriptor->pkt_return_qnum,
	descriptor->pkt_return_qmgr,
	descriptor->ret_push_policy,
	descriptor->reserved_w2b   ,
	descriptor->ps_flags       ,
	descriptor->err_flags      ,
	descriptor->psv_word_count ,
	descriptor->reserved_w2    ,
	descriptor->epib           );
}

/*check the descriptors in the queue, for debug purpose*/
void AIF_Check_TxRx_Queues()
{
	int i;
	Uint32 uiQuNum, uiEntryCount;

	
	_mfence(); 	//force all memory operation complete
	_mfence(); 	//force all memory operation complete
	for(i= uiNumAifChannel-uiNumCtlChannel; i<uiNumAifChannel; i++)
	{
		uiEntryCount= 
				gpQueueStatusConfigRegs[QMSS_AIF_QUEUE_BASE+i].REG_A_EntryCount;
		if(uiEntryCount)
		{
			printf("AIF TX queue %d entry count = %d\n", i, uiEntryCount);
		}
			
	}

	for(i=0; i<uiNumUsedRxFlow ; i++)
	{
		uiQuNum= flowCfgTable[i].rx_dest_qnum;
		uiEntryCount= KeyStone_GetQueueEntryCount(uiQuNum);
		if(uiEntryCount)
		{
			printf("number of descriptors in control channel RX queue %d = %d\n", 
				uiQuNum, uiEntryCount);
#if 1 	/*for debug*/
			while(KeyStone_GetQueueEntryCount(uiQuNum))
			{
				Uint32 uiDescriptor;
				AifCtlChTestInfo * testInfo; 
				MonolithicPacketDescriptor * Descriptor;

				uiDescriptor= KeyStone_queuePop(uiQuNum);
				Descriptor= (MonolithicPacketDescriptor *)uiDescriptor;
				printMonoDescriptor(Descriptor);
				testInfo= (AifCtlChTestInfo *)(uiDescriptor+ Descriptor->data_offset);
				printTestInfo(testInfo);			
			}
#endif
		}
	}
	
	uiEntryCount= 
			gpQueueStatusConfigRegs[0].REG_A_EntryCount;
	if(uiEntryCount)
		printf("Queue 0 entry count = %d\n", uiEntryCount);

}


