
/*  ============================================================================
 *   Copyright (c) Texas Instruments Inc 2012
 *
 *   Use of this software is controlled by the terms and conditions found in the
 *   license agreement under which this software has been supplied.
* ============================================================================
Example to show the initialization of Multicore Navigator on KeyStone DSP
* =============================================================================
 * Created by :                                                             *
 *            Brighton Feng                                                 *
 *            Texas Instruments                                             * 
 *            Jan 7, 2012                                                 *
 * ============================================================================
 */
#ifndef _AIF_PKT_DMA_INIT_H_
#define _AIF_PKT_DMA_INIT_H_

#include <csl_qm_queue.h>
#include "KeyStone_Navigator_init_drv.h"

/*size of monolithic descriptor must be multiple of 16B
In this test, one control packet contain the control words of one hyper frame 
for CPRI(for 8x link, it is 256*8=2048 bytes); 
or 16 control message of OBSAI (for 8x link, it is 16*16*8 = 2048 bytes)*/
#define WCDMA_CONTROL_PACKET_SIZE 	(2048)
#define AIF_MONO_PACKET_SIZE 		(WCDMA_CONTROL_PACKET_SIZE+128)

/*the linking RAM entry number for each descriptor region 
must be power of 2, and no less than 32.
Actual descriptor number may less than linking entry number, this may 
leave hole in linking RAM, but it still works*/
#define NUM_AIF_CTL_MONO_PACKET 	(256)

#define QUEUE_ACC_THRESH            (NUM_AIF_CTL_MONO_PACKET/4)

#define TEST_QUEUE_BASE 		2048

/*Low priority accumulation will be used for these queue,
so, their index start at 32 boundary, and use 32 queues per channel*/
#define AIF_RX_FLOW_DST_Q 	(TEST_QUEUE_BASE)
#define AIF_RX_FDQ    		(TEST_QUEUE_BASE+32)
#define AIF_TX_FDQ    		(TEST_QUEUE_BASE+33)

extern CSL_Cppidma_global_configRegs * aifDmaCfgRegs;
extern CSL_Cppidma_rx_channel_configRegs * aifDmaRxChCfgRegs;
extern CSL_Cppidma_rx_flow_configRegs * aifDmaRxFlowCfgRegs;
extern CSL_Cppidma_tx_channel_configRegs * aifDmaTxChCfgRegs;
extern volatile Uint32 * aifDmaTxChPriority;
extern PKT_DMA_Regs aifPktDmaRegs;

extern Uint32 uiaAifDescAccList[];
extern Uint32 uiQueueAccThresh;
/*AIF Queue initialization*/
extern void AIF_Queue_init();

/*AIF packet DMA initialization
must be called after AIF PSC enabled*/
extern void AIF_PktDMA_init();

/*check the descriptors in the queue, for debug purpose*/
extern void AIF_Check_Free_Queues();

/*check the descriptors in the queue, for debug purpose*/
extern void AIF_Check_TxRx_Queues();

/*print Non-empty queue for debug*/
extern void printNonEmptyQueue();
#endif
