/****************************************************************************\
 *           Copyright (C) 2011, 2012 Texas Instruments Incorporated.             *
 *                           All Rights Reserved                            *
 *                                                                          *
 * GENERAL DISCLAIMER                                                       *
 * ------------------                                                       *
 * All software and related documentation is provided "AS IS" and without   *
 * warranty or support of any kind and Texas Instruments expressly disclaims*
 * all other warranties, express or implied, including, but not limited to, *
 * the implied warranties of merchantability and fitness for a particular   *
 * purpose.  Under no circumstances shall Texas Instruments be liable for   *
 * any incidental, special or consequential damages that result from the    *
 * use or inability to use the software or related documentation, even if   *
 * Texas Instruments has been advised of the liability.                     *
 ****************************************************************************
 * Created by :                                                             *
 *            Brighton Feng                                                 *
 *            Texas Instruments                                             * 
 *            August 10, 2011                                                 *
Updated on June 18, 2012 for TCI6614 by Brighton
\***************************************************************************/
#ifndef _AIF_DATA_H
#define _AIF_DATA_H
#include <tistdtypes.h>

/*the internal antenna buffer size are limited by some facts:
1. the size of a frame must be multiple of the size of the buffer because 
   DIO will reload parameters every frame
2. DIO blocks number is configured with 13-bit field
3. DIO block address stride is configured with 12-bit field*/
#define NUM_SLOTS_IN_BUF 	3 	/*3, 5*/

/*test information fill at TX side, transfered as payload, verify at RX side*/
typedef struct
{
	Uint16 uiLinkNum;
	Uint16 uiChannelNum;
	Uint32 uiPacketSize;
	Uint32 uiCtlPktNum;
}AifCtlChTestInfo;

typedef struct
{
	Uint32 uiNumTXPackets;
	Uint32 uiNumGoodPackets;
	Uint32 uiNumBadPackets;
	Uint32 uiBandwidth;
	unsigned long long ullNumBytes;
}AifChStat;

//Aif state for all channels
extern AifChStat      aifChStat[NUM_AIF_CHANNEL+1];

extern Uint32 RAC_In_Buf[4][64][8]; 	/*32 chips x 64 antennas*/
extern Uint32 RAC_Out_Buf[4][64][8]; 	/*32 chips x 64 antennas*/
extern Uint32 TAC_In_Buf[64][4]; 	/*4 chips x 64 antennas*/
extern Uint32 TAC_Out_Buf[64][4]; 	/*4 chips x 64 antennas*/
extern Uint32 Core_In_Buf[NUM_AxC_CHANNEL][NUM_SLOTS_IN_BUF*WCDMA_SLOT_NUM_CHIP];
extern Uint32 Core_Out_Buf[NUM_AxC_CHANNEL][NUM_SLOTS_IN_BUF*WCDMA_SLOT_NUM_CHIP];

//Initialize Antenna Data buffer 
extern void InitAntDataBuf();
extern int processTacAntData(Uint32 uiAntStreamNum);
extern int processRacAntData(Uint32 uiAntStreamNum);
extern int processCoreAntData(Uint32 uiAntStreamNum);
/*process antenna data*/
extern void processAntData();
/*send Control data*/
extern void sendCtlData();
/*process received packet*/
extern void receiveCtlData(Uint32 uiDescriptor);
extern void PrintTestResult();
/*print test info for debug*/
extern void printTestInfo(AifCtlChTestInfo * testInfo);

#endif
