/****************************************************************************\
 *           Copyright (C) 2011, 2012 Texas Instruments Incorporated.       *
 *                           All Rights Reserved                            *
 *                                                                          *
 * GENERAL DISCLAIMER                                                       *
 * ------------------                                                       *
 * All software and related documentation is provided "AS IS" and without   *
 * warranty or support of any kind and Texas Instruments expressly disclaims*
 * all other warranties, express or implied, including, but not limited to, *
 * the implied warranties of merchantability and fitness for a particular   *
 * purpose.  Under no circumstances shall Texas Instruments be liable for   *
 * any incidental, special or consequential damages that result from the    *
 * use or inability to use the software or related documentation, even if   *
 * Texas Instruments has been advised of the liability.                     *
 ****************************************************************************
This example shows data transfer over AIF2 between two DSP on dual Nyquist EVM
The examples can also be run on C6670 or TCI6614 EVM, but only internal loopback 
test can be run on those boards because there is only one DSP on them.
This file includes basic configuration structure for test, user can modify 
the basic parameters to change the test mode to verify most functionality 
of AIF. The codes in this file calculate many other parameters based on the 
basic configuration.
****************************************************************************
 * Created by :                                                             *
 *            Brighton Feng                                                 *
 *            Texas Instruments                                             * 
 *            August 4, 2011                                                *
Updated on June 18, 2012 for TCI6614 by Brighton
***************************************************************************/

#include <stdio.h>
#include <string.h>
#include "aif_test_config.h"
#include "aif_data.h"

/*Change following basic parameters to switch between different test modes
Please note, AIF link 0~1 is connected between the two DSP on dual Nyquist EVM, 
AIF link 2~5 on the dual Nyquist EVM is connected to AMC interface.
Without external connection, a link can only be used for internal loopback test*/
AifLinkConfig aifLinkCfg[6]=
{
	/*Configuration for link 0*/ 
	{ 	
		1, 			/*Link Enable: 1=enable, 0=disable*/
		CSL_AIF2_LINK_RATE_4x,
		TEST_PATH_INTERNAL_LOOPBACK, 	/*test data path*/
		WCDMA_AIF2_RAC_TEST,		/*testType*/
		4, 					/*numberAxC: 0 means maximum number*/
#ifdef AIF2_LINK_PROTOCOL_OBSAI
		TRUE  			/*bUseControlMsg*/
#else //AIF2_LINK_PROTOCOL_CPRI
		20, 			/*firstCtlWordUsed*/
		20-20, 			/*numCtlWord*/
		CSL_AIF2_DATA_WIDTH_7_BIT 	/*dataWidth*/
#endif
	},
	/*Configuration for link 1*/
	{ 	
		1, 			/*Link Enable: 1=enable, 0=disable*/
		CSL_AIF2_LINK_RATE_4x,
		TEST_PATH_INTERNAL_LOOPBACK, 	/*test data path*/
		WCDMA_AIF2_RAC_TEST,		/*testType*/
		4, 					/*numberAxC: 0 means maximum number*/
#ifdef AIF2_LINK_PROTOCOL_OBSAI
		FALSE  			/*bUseControlMsg*/
#else //AIF2_LINK_PROTOCOL_CPRI
		20, 			/*firstCtlWordUsed*/
		20-20, 			/*numCtlWord*/
		CSL_AIF2_DATA_WIDTH_7_BIT 	/*dataWidth*/
#endif
	},
	/*Configuration for link 2*/
	{ 	
		0, 			/*Link Enable: 1=enable, 0=disable*/
		CSL_AIF2_LINK_RATE_4x,
		TEST_PATH_INTERNAL_LOOPBACK, 	/*test data path*/
		WCDMA_AIF2_CORE_TEST,		/*testType*/
		2, 					/*numberAxC: 0 means maximum number*/
#ifdef AIF2_LINK_PROTOCOL_OBSAI
		FALSE  			/*bUseControlMsg*/
#else //AIF2_LINK_PROTOCOL_CPRI
		32, 			/*firstCtlWordUsed*/
		32-32, 			/*numCtlWord*/
		CSL_AIF2_DATA_WIDTH_16_BIT 	/*dataWidth*/
#endif
	},
	/*Configuration for link 3*/
	{ 	
		0, 			/*Link Enable: 1=enable, 0=disable*/
		CSL_AIF2_LINK_RATE_4x,
		TEST_PATH_INTERNAL_LOOPBACK,  	/*test data path*/
		WCDMA_AIF2_RAC_TEST,		/*testType*/
		3, 					/*numberAxC: 0 means maximum number*/
#ifdef AIF2_LINK_PROTOCOL_OBSAI
		TRUE  			/*bUseControlMsg*/
#else //AIF2_LINK_PROTOCOL_CPRI
		20, 			/*firstCtlWordUsed*/
		64-20, 			/*numCtlWord*/
		CSL_AIF2_DATA_WIDTH_7_BIT 	/*dataWidth*/
#endif
	},
	/*Configuration for link 4*/
	{ 	
		1, 			/*Link Enable: 1=enable, 0=disable*/
		CSL_AIF2_LINK_RATE_4x,
		TEST_PATH_INTERNAL_LOOPBACK, 	/*test data path*/
		WCDMA_AIF2_TAC_TEST,		/*testType*/
		8, 					/*numberAxC: 0 means maximum number*/
#ifdef AIF2_LINK_PROTOCOL_OBSAI
		FALSE  			/*bUseControlMsg*/
#else //AIF2_LINK_PROTOCOL_CPRI
		32, 			/*firstCtlWordUsed*/
		32-32, 			/*numCtlWord*/
		CSL_AIF2_DATA_WIDTH_16_BIT 	/*dataWidth*/
#endif
	},
	/*Configuration for link 5*/
	{ 	
		0, 			/*Link Enable: 1=enable, 0=disable*/
		CSL_AIF2_LINK_RATE_4x,
		TEST_PATH_INTERNAL_LOOPBACK, 	/*test data path*/
		WCDMA_AIF2_CORE_TEST,		/*testType*/
		2, 					/*numberAxC: 0 means maximum number*/
#ifdef AIF2_LINK_PROTOCOL_OBSAI
		FALSE  			/*bUseControlMsg*/
#else //AIF2_LINK_PROTOCOL_CPRI
		32, 			/*firstCtlWordUsed*/
		32-32, 			/*numCtlWord*/
		CSL_AIF2_DATA_WIDTH_16_BIT 	/*dataWidth*/
#endif
	}
};

/*look up table of different link rate*/
unsigned char linkRate[4]= 
{
	8, /*CSL_AIF2_LINK_RATE_8x*/
	4, /*CSL_AIF2_LINK_RATE_4x*/
	2  /*CSL_AIF2_LINK_RATE_2x*/
};

/*look up table of Serdes delay for different link rate*/
unsigned char SerdesDelay[4]= 
{
	SERDES_8X_DELAY,
	SERDES_4X_DELAY,
	SERDES_2X_DELAY
};

Uint32 uiNumAifChannel; //Total number of channels used
Uint32 uiNumCtlChannel; //number of control channels used
Uint32 uiIgrDioOffset; 	//Ingress DIO offset
Uint32 uiRM_PI_MAX = 0; 	//maximum pi value between links

//calculate AIF link timing parameters
void calcAifLinkTimingParameters(AifLinkConfig * aifLinkCfg, Uint32 uiLinkNum)
{
	Uint32 dmaDelay;
	Uint32 peOffset;
	Uint32 tmOffset;
	Uint32 rmOffset;
	
	aifLinkCfg->RT_cfg= CSL_AIF2_RT_MODE_TRANSMIT;
	aifLinkCfg->PE_RT_ctl= CSL_AIF2_PE_RT_INSERT;

	//MAX delay for DIO transfer
	dmaDelay = DIO_OVERHEAD + DDR_DELAY*16;//uiNumAifChannel;
	if(dmaDelay < 16*WCDMA_CHIP_NUM_CLOCK)
		dmaDelay = 16*WCDMA_CHIP_NUM_CLOCK;
	if(dmaDelay > 24*WCDMA_CHIP_NUM_CLOCK)
		dmaDelay = 24*WCDMA_CHIP_NUM_CLOCK;

	if(TEST_PATH_NORMAL_OPERATION == aifLinkCfg->testDataPath)
	{
		//PePrepSync should start at or after DMA done
		peOffset= dmaDelay;
		aifLinkCfg->PeEvent1Offset= peOffset;

		/*for normal operation, PeEvent1Offset and piOffset
		should be set manually or calculated based on real test scenario*/
		peOffset = aifLinkCfg->PeEvent1Offset+ PE1_PE2_DELAY;
		aifLinkCfg->PeEvent2Offset= peOffset;

		tmOffset= peOffset + PE2_TM_DELAY;
		aifLinkCfg->deltaOffset= tmOffset;

		/*------------timing of RX node---------------*/
		/*Rx offset= Tx offset on previous node + SerDes delay*/
		rmOffset= tmOffset + SerdesDelay[aifLinkCfg->linkRate];
		aifLinkCfg->piOffset= rmOffset;

	}
	else if((TEST_PATH_INTERNAL_LOOPBACK == aifLinkCfg->testDataPath)
		||(TEST_PATH_EXTERLNAL_LINE_LOOPBACK == aifLinkCfg->testDataPath))
	{
		/*------------timing of TX node---------------*/
		//PePrepSync should start at or after DMA done
		peOffset= dmaDelay;
		aifLinkCfg->PeEvent1Offset= peOffset;

		peOffset += PE1_PE2_DELAY;
		aifLinkCfg->PeEvent2Offset= peOffset;

		tmOffset= peOffset + PE2_TM_DELAY;
		aifLinkCfg->deltaOffset= tmOffset;

		/*------------timing of RX node---------------*/
		/*Rx offset= Tx offset on previous node + SerDes delay*/
		rmOffset= tmOffset + SerdesDelay[aifLinkCfg->linkRate];
		if(TEST_PATH_EXTERLNAL_LINE_LOOPBACK == aifLinkCfg->testDataPath)
			rmOffset += 2 * PROPAGATION_DELAY;
		aifLinkCfg->piOffset= rmOffset;
	}
	else if(TEST_PATH_INTERNAL_REDIRECTION == aifLinkCfg->testDataPath)
	{
		aifLinkCfg->RT_cfg= CSL_AIF2_RT_MODE_RETRANSMIT;
		aifLinkCfg->PE_RT_ctl= CSL_AIF2_PE_RT_RETRANS;

		peOffset= aifLinkCfg->piOffset + RM_PE1_DELAY;
		aifLinkCfg->PeEvent1Offset= peOffset;

		peOffset += PE1_PE2_DELAY;
		aifLinkCfg->PeEvent2Offset= peOffset;

		tmOffset= peOffset + PE2_TM_DELAY;
		aifLinkCfg->deltaOffset= tmOffset;
	}
	else	/*TEST_PATH_REDIRECTION*/
	{
		if(0!=uiDSPNum)
		{
			aifLinkCfg->RT_cfg= CSL_AIF2_RT_MODE_RETRANSMIT;
			aifLinkCfg->PE_RT_ctl= CSL_AIF2_PE_RT_RETRANS;
		}

		/*------------timing of first TX node---------------*/
		//PePrepSync should start at or after DMA done
		peOffset= dmaDelay;
		if(0==uiDSPNum)
			aifLinkCfg->PeEvent1Offset= peOffset;

		peOffset += PE1_PE2_DELAY;
		if(0==uiDSPNum)
			aifLinkCfg->PeEvent2Offset= peOffset;

		tmOffset= peOffset + PE2_TM_DELAY;
		if(0==uiDSPNum)
			aifLinkCfg->deltaOffset= tmOffset;

		/*------------timing of first RX node---------------*/
		/*Rx offset= Tx offset on previous node + SerDes delay*/
		rmOffset= tmOffset + SerdesDelay[aifLinkCfg->linkRate]
			+ PROPAGATION_DELAY;
		if(0!=uiDSPNum)
			aifLinkCfg->piOffset= rmOffset;

		/*------------timing of second TX node---------------*/
		/*For daisy chain redirection, the TM offset is calculated according 
		to RX offset*/
		peOffset= rmOffset + RM_PE1_DELAY;
		if(0!=uiDSPNum)
			aifLinkCfg->PeEvent1Offset= peOffset;

		peOffset += PE1_PE2_DELAY;
		if(0!=uiDSPNum)
			aifLinkCfg->PeEvent2Offset= peOffset;

		tmOffset= peOffset + PE2_TM_DELAY;
		if(0!=uiDSPNum)
			aifLinkCfg->deltaOffset= tmOffset;

		/*------------timing of last RX node---------------*/
		/*Rx offset= Tx offset on previous node + SerDes delay*/
		rmOffset= tmOffset + SerdesDelay[aifLinkCfg->linkRate]
			+ PROPAGATION_DELAY;
		if(0==uiDSPNum)
			aifLinkCfg->piOffset= rmOffset;
	}

	if(uiRM_PI_MAX < aifLinkCfg->piOffset)
		uiRM_PI_MAX = aifLinkCfg->piOffset;
	
}

char * testPathStr[]=
{
	"normal operation",
	"internal loopback test",
	"internal redirection test",
	"external redirection test",
	"external line loopback test"
};

char * testTypeStr[]=
{
	"RAC antenna data",
	"TAC antenna data",
	"DSP core antenna"
};

/*print test configuration*/
void PrintTestConfiguration()
{
	Int32 i;
	
#ifdef AIF2_LINK_PROTOCOL_OBSAI
		printf("=================AIF OBSAI mode test for %d ms (%d WCDMA frames)=================\n",
			TEST_NUM_FRAME*10, TEST_NUM_FRAME);
#else
		printf("=================AIF CPRI mode test for %d ms (%d WCDMA frames)=================\n",
			TEST_NUM_FRAME*10, TEST_NUM_FRAME);
#endif

	for(i=0; i<NUM_AIF2_LINK; i++)	//for all links
	{
		if(1==aifLinkCfg[i].linkEnable)
		{
			printf("link %d runs at %dx rate, %s", i, 
				linkRate[aifLinkCfg[i].linkRate], 
				testPathStr[aifLinkCfg[i].testDataPath]);

#ifdef AIF2_LINK_PROTOCOL_OBSAI
			if(aifLinkCfg[i].bUseControlMsg)
				printf(", %s and control messages\n", 
					testTypeStr[aifLinkCfg[i].testType]);
#else
			if(aifLinkCfg[i].numCtlWord)
				printf(", %s and control words\n",
					testTypeStr[aifLinkCfg[i].testType]);
#endif
			else
				printf(", %s data only\n",
					testTypeStr[aifLinkCfg[i].testType]);
		}
		else
			printf("link %d is disabled\n", i);
	}
}

//Calculate some other AIF Parameters according above pre-set paremeters
//this function should called before DMA, and AIF configuration
void InitAifTestParameters()
{
	Int32 i, j;
	Uint32 uiDBM_X=0;	
	Int32 iLowNeighbor=-1;

	uiNumAifChannel= 0;
	uiNumCtlChannel= 0;

	/*antenna data channel allocation*/
	for(i=0; i<NUM_AIF2_LINK; i++)	//for all links
	{
		if(0==aifLinkCfg[i].linkEnable)
			continue;

		if(WCDMA_AIF2_RAC_TEST==aifLinkCfg[i].testType)
		{
			aifLinkCfg[i].dataFormat = CSL_AIF2_LINK_DATA_TYPE_RSA;
			aifLinkCfg[i].AxCSwap= CSL_AIF2_DB_BYTE_SWAP;
		}
		else if(WCDMA_AIF2_TAC_TEST==aifLinkCfg[i].testType)
		{
			aifLinkCfg[i].dataFormat = CSL_AIF2_LINK_DATA_TYPE_NORMAL;
			aifLinkCfg[i].AxCSwap= CSL_AIF2_DB_WORD_SWAP;
		}
		else //WCDMA_AIF2_CORE_TEST
		{
#ifdef CORE_DATA_TYPE_RSA 	//8 bit uplink RSA type
			aifLinkCfg[i].dataFormat = CSL_AIF2_LINK_DATA_TYPE_RSA;
			aifLinkCfg[i].AxCSwap= CSL_AIF2_DB_BYTE_SWAP;
#else 	//16 bit downlink normal type
			aifLinkCfg[i].dataFormat = CSL_AIF2_LINK_DATA_TYPE_NORMAL;
			aifLinkCfg[i].AxCSwap= CSL_AIF2_DB_WORD_SWAP;
#endif
		}

		aifLinkCfg[i].firstAxcNum = uiNumAifChannel;
		uiDBM_X = linkRate[aifLinkCfg[i].linkRate]*4;
#ifdef AIF2_LINK_PROTOCOL_OBSAI
		if(CSL_AIF2_LINK_DATA_TYPE_RSA == aifLinkCfg[i].dataFormat)	
			aifLinkCfg[i].dataWidth = CSL_AIF2_DATA_WIDTH_8_BIT;
		else 	//16 bit downlink normal type
			aifLinkCfg[i].dataWidth = CSL_AIF2_DATA_WIDTH_16_BIT;
#else //AIF2_LINK_PROTOCOL_CPRI
		if(((CSL_AIF2_DATA_WIDTH_16_BIT==aifLinkCfg[i].dataWidth)||
			(CSL_AIF2_DATA_WIDTH_15_BIT==aifLinkCfg[i].dataWidth))&&
			(CSL_AIF2_LINK_DATA_TYPE_RSA==aifLinkCfg[i].dataFormat))
		{
			printf("ERROR: link %d data width for RSA type must be 7 or 8 bits!\n", i);
		}
		if(((CSL_AIF2_DATA_WIDTH_8_BIT==aifLinkCfg[i].dataWidth)||
			(CSL_AIF2_DATA_WIDTH_7_BIT==aifLinkCfg[i].dataWidth))&&
			(CSL_AIF2_LINK_DATA_TYPE_NORMAL==aifLinkCfg[i].dataFormat))
		{
			printf("ERROR: link %d data width for normal type must be 15 or 16 bits!\n", i);
		}
		if((CSL_AIF2_DATA_WIDTH_16_BIT==aifLinkCfg[i].dataWidth)||
			(CSL_AIF2_DATA_WIDTH_8_BIT==aifLinkCfg[i].dataWidth))
			uiDBM_X = linkRate[aifLinkCfg[i].linkRate]*15/4;
#endif
		aifLinkCfg[i].DBM_X= uiDBM_X;
		if((0==aifLinkCfg[i].numberAxC)||(aifLinkCfg[i].numberAxC>uiDBM_X))
			aifLinkCfg[i].numberAxC= uiDBM_X;

		if(aifLinkCfg[i].numberAxC>64) 	/*max AxC per channel is 64*/
			aifLinkCfg[i].numberAxC= 64;

		uiNumAifChannel += aifLinkCfg[i].numberAxC;

		if(uiNumAifChannel > NUM_AxC_CHANNEL)
		{/*maximum channel number = 128*/
			for(j=i; j<6; j++)
				aifLinkCfg[j].linkEnable= 0;

			printf("AxC channel number larger than %d, the links with AxC channel number larger than %d is disabled!\n",
				NUM_AxC_CHANNEL, NUM_AxC_CHANNEL);
			break;
		}

		calcAifLinkTimingParameters(&aifLinkCfg[i], i);


		/*setup Serdes Sync Master according to following rules:
		1. Lanes being aggregated must have set this bit low.
		2. One macro must be nominated as the master.(B8 if it is used)
		3. The master macro must have one lane which tied this high. 
			(lower numbered lane)
		4. All other lanes on all other macros being aggregated should 
			be tied low.
		5. Transmit lanes that have a lower-numbered neighbor of a different 
			rate or width configuration must have set this bit high.*/
		if(-1==iLowNeighbor)/*first used lane*/
			aifLinkCfg[i].bEnableTxSyncMater= TRUE;
		else if(aifLinkCfg[i].linkRate!=
			aifLinkCfg[iLowNeighbor].linkRate)/*different link rate*/
			aifLinkCfg[i].bEnableTxSyncMater= TRUE;
		else/*lane same as previous lane*/
			aifLinkCfg[i].bEnableTxSyncMater= FALSE;
		
		iLowNeighbor= i;
	}

	//for this test, wait 16 chips data is availible in DB before Ingress DIO start
	uiIgrDioOffset = uiRM_PI_MAX + RM_DB_DELAY + 16 * WCDMA_CHIP_NUM_CLOCK;

	/*control channel allocation*/
	for(i=0; i<NUM_AIF2_LINK; i++)	//for all links
	{
		if(0==aifLinkCfg[i].linkEnable)
			continue;

#ifdef AIF2_LINK_PROTOCOL_OBSAI
		if(aifLinkCfg[i].bUseControlMsg)
		{
			aifLinkCfg[i].controlChNum= uiNumAifChannel++;
			uiNumCtlChannel++;
		}
#else
		if(aifLinkCfg[i].numCtlWord)
		{
			aifLinkCfg[i].controlChNum= uiNumAifChannel++;
			uiNumCtlChannel++;
		}
#endif

		if(uiNumAifChannel > NUM_AIF_CHANNEL)
		{/*maximum channel number = 128*/
			for(j=i; j<6; j++)
			{
#ifdef AIF2_LINK_PROTOCOL_OBSAI
				aifLinkCfg[j].bUseControlMsg = FALSE;
#else
				aifLinkCfg[j].numCtlWord = 0;
#endif
			}
			printf("channel number larger than %d, control channel number larger than %d is disabled!\n",
				NUM_AIF_CHANNEL, NUM_AIF_CHANNEL);
			uiNumAifChannel--;
			uiNumCtlChannel--;
			break;
		}
	}

	memset(aifChStat, 0, sizeof(aifChStat));
	
	PrintTestConfiguration();
}


