/****************************************************************************\
 *           Copyright (C) 2011, 2012 Texas Instruments Incorporated.             *
 *                           All Rights Reserved                            *
 *                                                                          *
 * GENERAL DISCLAIMER                                                       *
 * ------------------                                                       *
 * All software and related documentation is provided "AS IS" and without   *
 * warranty or support of any kind and Texas Instruments expressly disclaims*
 * all other warranties, express or implied, including, but not limited to, *
 * the implied warranties of merchantability and fitness for a particular   *
 * purpose.  Under no circumstances shall Texas Instruments be liable for   *
 * any incidental, special or consequential damages that result from the    *
 * use or inability to use the software or related documentation, even if   *
 * Texas Instruments has been advised of the liability.                     *
 ****************************************************************************
This file record and print AIF status and error information for debug.
Because AIF2 is very timing sensitive, the print functions here should only 
be called after test complete.
****************************************************************************
 * Created by :                                                             *
 *            Brighton Feng                                                 *
 *            Texas Instruments                                             * 
 *            August 12, 2011                                                 *
Updated on June 18, 2012 for TCI6614 by Brighton
Fixed SD_PLL_Bx_STS check bug on Dec 8, 2014 by Brighton
***************************************************************************/
#include <stdio.h>
#include <c6x.h>
#include <string.h>
#include <cslr_aif2.h>
#include "KeyStone_common.h"
#include "aif_test_config.h"
#include "aif_setup.h"
#include "aif_debug.h"
#include "aif_intc.h"

#pragma DATA_SECTION(aif2ErrStsRegs,".far:AifDebug")//use DDR3 memory for test/debug
/*copy of AIF2 error/status registers*/
Aif2ErrorStatusRegs aif2ErrStsRegs;

#pragma DATA_SECTION(aif2RegsLog,".far:AifDebug")//use DDR3 memory for test/debug
/*copy of AIF2 registers*/
CSL_Aif2Regs aif2RegsLog;

/*--------------------error information strings-----------------*/
char * ee_db_irs_str[]=
{
	"0 db_ee_i_trc_ram_ovfl_err: (Error) Data Trace RAM overflowed. This is not a fatal error because it only affects the Data Trace RAM.",
	"1 db_ee_i_token_ovfl_err: (Error) Ingress Packet Data Token FIFO overflowed. This is a catastrophic condition and software needs to effectively flush all of the data buffers in the Ingress DB RAM and re-start the PD",
	"2 db_ee_i_fifo_ovfl_err: (Information) Ingress FIFO buffer channel overflowed. This can happen if the AIF2 VBUSM interface is stalled by the system. This is not a fatal error. The AIF2 can gracefully recover from this condition",
	"3 db_ee_i_pd2db_full_err: (Error) PD-to-DB Bridge full. This is a catastrophic condition and indicates that the VBUS clock rate is too slow. Although this bridge is 64-deep, the error event will first occur when the bridge depth is 56 as an early warning",
	"4 db_ee_e_ps_axc_err: (Error) AxC in Multicore Navigator Protocol Specific data does not match Multicore Navigator thread id. This is an indication of an application error. This event occurs if the application put data for an AxC in the wrong buffer in external memory",
	"5 db_ee_e_cd_data_err: (Information) Activity monitor that is active each time the Egress DB receives data from the PKTDMA",
	"6 db_ee_e_cd_data_type_err: (Error) EDB received data from PKTDMA with undefined Multicore Navigator data type. This is an indication of an application error. This event occurs if the Multicore Navigator Data Type received from the PKTDMA is not one of the ones supported."
};

char * ee_ad_irs_str[]=
{
	"0 ad_ee_i_cd_data_err: (Information) Activity monitor that is active whenever the Ingress AD sends data to the PKTDMA.",
	"1 ad_ee_e_cd_sch_err: (Information) Activity monitor that is active high each time the Egress AD Scheduler sends a request to read data to the PKTDMA.",
	"",
	"",
	"",
	"",
	"",
	"",
	"",
	"",
	"",
	"",
	"",
	"",
	"",
	"",
	"16 ad_ee_i_dma_0_err: (Error) For each of three Ingress DMA channels, if a trigger request is received while a request for that channel is still pending, a DMA error event will occur",
	"17 ad_ee_i_dma_1_err: (Error) For each of three Ingress DMA channels, if a trigger request is received while a request for that channel is still pending, a DMA error event will occur",
	"18 ad_ee_i_dma_2_err: (Error) For each of three Ingress DMA channels, if a trigger request is received while a request for that channel is still pending, a DMA error event will occur.",
	"",
	"",
	"",
	"",
	"",
	"24 ad_ee_e_dma_0_err: (Error) For each of three Egress DMA channels, if a trigger request is received while a request for that channel is still pending, a DMA error event will occur",
	"25 ad_ee_e_dma_1_err: (Error) For each of three Egress DMA channels, if a trigger request is received while a request for that channel is still pending, a DMA error event will occur",
	"26 ad_ee_e_dma_2_err: (Error) For each of three Egress DMA channels, if a trigger request is received while a request for that channel is still pending, a DMA error event will occur."
};

char * ee_cd_irs_str[]=
{
	"0 cd_ee_sop_desc_starve_err: (Error) PKTDMA SOP Descriptor Starvation",
	"1 cd_ee_mop_desc_starve_err: (Error) PKTDMA MOP Descriptor Starvation"
};

char * ee_sd_irs_str[]=
{
	"0 sd_ee_stspll_b4_err: (Information) If B4 PLL is locked correctly, this field will be set",
	"1 sd_ee_stspll_b8_err: (Information) If B8 PLL is locked correctly, this field will be set"
};

char * ee_vc_irs_str[]=
{
	"0 vc_ee_vbus_err: (Error) shows VBUS data error"
};

char * ee_lk_irs_a_str[]=
{
	"0 rm_ee_sync_status_change_err: Per Link (Information): Indicates that the RX state machine changed state",
	"1 rm_ee_num_los_det_err: Per Link (Error): los detection error",
	"2 rm_ee_lcv_det_err: Per Link (Error): lcv detection error",
	"3 rm_ee_frame_bndry_det_err: Per Link (Info): Indicates that a frame boundary was detected.",
	"4 rm_ee_block_bndry_det_err: Per Link (Info): Indicates that a block boundary was detected. CPRI: K28.5 character & HFN!= 149 OBSAI: K28.5 character.",
	"5 rm_ee_missing_k28p5_err: Per Link (Error): Indicates that a k28.5 character was not received when it was supposed to have been received.",
	"6 rm_ee_missing_k28p7_err: Per Link, OBSAI Only (Error): Indicates that a k28.7 character was not received when it was supposed to have been received.",
	"7 rm_ee_k30p7_det_err: Per Link, OBSAI Only (Error): Indicates that a k30.7 error character was received.",
	"8 rm_ee_loc_det_err: Per Link (Error): Indicates that the loss of clock watch dog timer has timed out that is, SerDes clock was not detected within the specified window.",
	"9 rm_ee_rx_fifo_ovf_err: Per Link (Error): Indicates that the RX FIFO has overflowed.",
	"10 rm_ee_rcvd_los_err: Per Link, CPRI only (Error): Indicates a received, L1 Inband Loss Of Signal event (Z.130.0, bit3 as defined by CPRI)",
	"11 rm_ee_rcvd_lof_err: Per Link, CPRI only (Error): Indicates a received, L1 Inband Loss Of Frame",
	"12 rm_ee_rcvd_rai_err: Per Link, CPRI only (Error): Indicates RAI error.",
	"13 rm_ee_rcvd_sdi_err: Per Link, CPRI only (Error): Indicates SDI error.",
	"14 rm_ee_los_err: Per Link, CPRI only (Error): Indicates that the RX FSM is in the Loss Of Signal state that is, ST0 (as defined by CPRI)",
	"15 rm_ee_lof_err: Per Link, CPRI only (Error): Indicates that the RX FSM is in the Loss Of Frame state that is, ST0 or St1 (as defined by CPRI)",
	"16 rm_ee_hfnsync_state_err: Per Link, CPRI only (Error): Indicates RX FSM in the hyperframe state that is, state ST3. (as defined by CPRI) ",
	"17 rm_ee_lof_state_err: Per Link, CPRI only (Error): Indicates that the RX FSM is in the Loss Of Signal state that is, ST0 (as defined by CPRI) ",
	"",
	"",
	"",
	"",
	"",
	"",
	"24 tm_ee_frm_misalign_err: (Informational or Error), Per Link. The TM state machine detected a mis-aligned data frame on link. This bit is active on any clock cycle that the frame indication for the TM from the AT block does not line up with the frame indication in the data path. A misalignment will also occur if a frame indication from the AT block happens but the TM Fifo is empty",
	"25 tm_ee_fifo_starve_err: (Informational or Error), Per link. The TM FIFO for link does not have data to transmit. This bit is active when the TM FIFO does not have data in it"
};

char * ee_lk_irs_b_str[]=
{
	"0 pd_ee_eop_err: Link-by-Link (Error), Received a second EOP without an SOP in between. For generic packet mode with 4B/5B encoding only",
	"1 pd_ee_crc_err: Link-by-Link (Error), CRC failure for any packet (with CRC checking) within the given link.",
	"2 pd_ee_cpri_frame_err: Link-by-Link (Error), Arriving traffic (and user configuration of DBM) is non-multiple of 4 ",
	"3 pd_ee_axc_fail_err: Link-by-Link (Error), Unrecoverable OBSAI Timestamp error for at least 1 AxC of a given link",
	"",
	"5 pd_ee_sop_err: Link-by-Link (Error), Received a second SOP without an EOP in between. For generic packet mode with 4B/5B encoding only",
	"6 pd_ee_obsai_frm_err: Link-by-Link (Error), Wrap of the PD_Frame Counters did not predict a Radio Frame Boundary consistent with TS=0 falling within the reception timing window. (PD_Frame)",
	"7 pd_ee_wr2db_err: Link-by-Link (Information), If any value is written to the DB. Used as debug indicating PD is processing traffic for a given link.",
	"",
	"",
	"",
	"",
	"",
	"",
	"",
	"",
	"16 pe_ee_modrule_err: Link-by-Link (Error), More than one modulo rule fired in a single clock cycle",
	"17 pe_ee_sym_err: Link-by-Link (Error), Symbol index in Multicore Navigator protocol specific header did not match for one or more symbol (Multicore Navigator packet).",
	"18 pe_ee_mf_fifo_overflow_err: Link-by-Link (Error), MF FIFO was full when a write occurred.",
	"19 pe_ee_mf_fifo_underflow_err: Link-by-Link (Error), MF FIFO was empty when a read occurred.",
	"20 pe_ee_db_starve_err: Link-by-Link (Error), DB did not have antenna data for a AxC channel. Likely to occur if DMA was late.",
	"21 pe_ee_rt_if_err: Link-by-Link (Error), RT Interface Error",
	"22 pe_ee_pkt_starve_err: Link-by-Link (Error), DB did not have packet data for a pkt channel. This error is only for CPRI packets and occurs when an active packet runs out of data in the middle of a packet.",
	"",
	"24 rt_ee_frm_err: (Error), Per Link. The Retransmit frame no longer matches the Transmit frame in Aggregate mode for Link. The RT checks the frame structure of the data from the RM. If the frame structure received is not consistent with the programmed frame type(OBSAI or CPRI, link_rate), a frame error condition will occur. When a frame error condition occurs, this bit is made active, the RT will flush the RT Fifo so that it is empty, and a state machine will require a new frame boundary from the RM to begin buffering data again",
	"25 rt_ee_ovfl_err: (Informational or Error), Per Link. The RT FIFO has overflowed in Retransmit or Aggregate mode for Link. The RT fifo contains the data that is to be retransmitted from the RM block while the RT is in Retransmit or Aggregate mode. The RT fifo is emptied for the TM to transmit. If the TM does not empty the Fifo as fast as the RM fills it, the RT fifo will suffer an overflow condition. When an overflow condition occurs, this bit is made active, and a state machine will require a new frame boundary from the RM to begin buffering data again.",
	"26 rt_ee_unfl_err: (Informational or Error), Per Link. The RT FIFO has underflowed in Retransmit or Aggregate mode for Link. The RT fifo contains the data that is to be retransmitted from the RM block while the RT is in Retransmit or Aggregate mode. The RT fifo is emptied for the TM to transmit. If the TM expects data and there is not any data for the TM to transmit, this situation defines an underflow condition. When an underflow condition occurs, this bit is made active, and a state machine will require a new frame boundary from the RM to begin buffering data again.",
	"27 rt_ee_em_err: (Information), Per Link. The RT block created an Empty message in OBSAI Aggregate mode for Link. The RT block will replace a message with an empty message in OBSAI Retransmit mode if it detects that a message header is corrupted as indicated by error indication bits supplied by the RM. When this occurs, this bit is made active",
	"28 rt_ee_hdr_err: (Error), Per Link. The Retransmit header does not match the transmit header in OBSAI Aggregate mode for Link. While in OBSAI aggregate mode, the RT block expects that the header will match between the transmitted message from the PE and the Retransmitted message received by the RM. If the header does not match, the PE header is assumed to be the one used to transmit out of the TM and this indication is made active."
};

char * ee_at_irs_str[]=
{
	"0 at_ee_rp1_type_sys_rcvd_err: (Information) AT RP1 Type SYS data Receive",
	"1 at_ee_rp1_type_rp3_rcvd_err: (Information) AT RP1 Type RP3 data Receive",
	"2 at_ee_rp1_type_tod_rcvd_err: (Information) AT RP1 Type TOD data Receive",
	"3 at_ee_rp1_type_unsel_err: (Error) AT RP1 Type Unsel Error.   ",
	"4 at_ee_rp1_type_spare_err: (Error) AT RP1 Type Spare Error.   ",
	"5 at_ee_rp1_type_rsvd_err: (Error) AT RP1 Type Reserved Error. ",
	"6 at_ee_rp1_bit_width_err: (Error) AT RP1 Type Bit Width Error.",
	"7 at_ee_rp1_crc_err: (Error) AT RP1 Type CRC Error.",
	"8 at_ee_rp1_rp3_err: (Error) AT PHYT selected width Frame number does not match the received RP1 RP3 frame number Error.",
	"9 at_ee_rp1_sys_err: (Error) AT RADT selected width Frame number does not match the received RP1 SYS frame number Error.",                                                               
	"10 at_ee_pi0_err: (Error) AT Link 0 Captured Pi is not within pi window",                                                                      
	"11 at_ee_pi1_err: (Error) AT Link 1 Captured Pi is not within pi window",                                                                      
	"12 at_ee_pi2_err: (Error) AT Link 2 Captured Pi is not within pi window",                                                                      
	"13 at_ee_pi3_err: (Error) AT Link 3 Captured Pi is not within pi window",                                                                      
	"14 at_ee_pi4_err: (Error) AT Link 4 Captured Pi is not within pi window",                                                                      
	"15 at_ee_pi5_err: (Error) AT Link 5 Captured Pi is not within pi window",                                                                      
	"16 at_ee_phyt_sync_err: (Error) AT PHYT sync input is not aligned to the PHYT counter frame boundary Error.",                                          
	"17 at_ee_radt_sync_err: (Error) AT RADT sync input is not aligned to the RADT counter frame boundary Error."
};

/*lookup table of IRS error information*/
char ** IRS_info[NUM_ERR_STS_REGS]=
{
	ee_lk_irs_a_str,	/*EE_LK_STS_A0*/
	ee_lk_irs_b_str,	/*EE_LK_STS_B0*/
	ee_lk_irs_a_str,	/*EE_LK_STS_A1*/
	ee_lk_irs_b_str,	/*EE_LK_STS_B1*/
	ee_lk_irs_a_str,	/*EE_LK_STS_A2*/
	ee_lk_irs_b_str,	/*EE_LK_STS_B2*/
	ee_lk_irs_a_str,	/*EE_LK_STS_A3*/
	ee_lk_irs_b_str,	/*EE_LK_STS_B3*/
	ee_lk_irs_a_str,	/*EE_LK_STS_A4*/
	ee_lk_irs_b_str,	/*EE_LK_STS_B4*/
	ee_lk_irs_a_str,	/*EE_LK_STS_A5*/
	ee_lk_irs_b_str,	/*EE_LK_STS_B5*/
	ee_at_irs_str,	/*EE_AT_STS   */
	ee_sd_irs_str,	/*EE_SD_STS   */
	ee_db_irs_str,	/*EE_DB_STS   */
	ee_ad_irs_str,	/*EE_AD_STS   */
	ee_cd_irs_str,	/*EE_CD_STS   */
	ee_vc_irs_str	/*EE_VC_STS   */
};

char * irsStr[]=
{
	"EE_LK_STS_A0 ",
	"EE_LK_STS_B0 ",
	"EE_LK_STS_A1 ",
	"EE_LK_STS_B1 ",
	"EE_LK_STS_A2 ",
	"EE_LK_STS_B2 ",
	"EE_LK_STS_A3 ",
	"EE_LK_STS_B3 ",
	"EE_LK_STS_A4 ",
	"EE_LK_STS_B4 ",
	"EE_LK_STS_A5 ",
	"EE_LK_STS_B5 ",
	"EE_AT_STS    ",
	"EE_SD_STS    ",
	"EE_DB_STS    ",
	"EE_AD_STS    ",
	"EE_CD_STS    ",
	"EE_VC_STS    "
};

/*-----------------------status information string--------------------------*/
char * rmStsStr[]=
{
	"RM ST3 State FRAME_SYNC           ",
	"RM ST2 State WAIT_FOR_FRAME_SYNC_T",
	"RM ST1 State WAIT_FOR_K28p7_IDLES ",
	"RM ST0 State UNSYNC               ",
	"RM ST4 State WAIT_FOR_SEED        ",
	"RM ST5 State WAIT_FOR_ACK         "
};

char * tmStsStr[]=
{
	"TM FSM in OFF state       ",
	"TM FSM in IDLE state      ",
	"TM FSM in RE_SYNC state   ",
	"TM FSM in FRAME_SYNC state"
};

//print AIF MAC status for a link
void print_AIF_Link_MAC_status(Uint32 uiLinkNumber)
{
	Uint32 LinkStatus, uiStatus;
	Int32 iClockError;

	LinkStatus = aif2ErrStsRegs.G_RM_LKS[uiLinkNumber].RM_LK_STS0 ;

	uiStatus= (LinkStatus & CSL_AIF2_RM_LK_STS0_SYNC_STATUS_MASK)>>
		CSL_AIF2_RM_LK_STS0_SYNC_STATUS_SHIFT;
	uiStatus= 31-_lmbd(1, uiStatus);
	printf("%s\n", rmStsStr[uiStatus]);

	uiStatus= (LinkStatus & CSL_AIF2_RM_LK_STS0_NUM_LOS_DET_MASK)>>
		CSL_AIF2_RM_LK_STS0_NUM_LOS_DET_SHIFT;
	if(uiStatus)
		printf("RM los counter has reached the programable los_det_thold limit\n");

	uiStatus= (LinkStatus & CSL_AIF2_RM_LK_STS0_LOC_MASK)>>
		CSL_AIF2_RM_LK_STS0_LOC_SHIFT;
	if(uiStatus)
		printf("RM clock watchdog circuit detected a missing clock\n");

	uiStatus= (LinkStatus & CSL_AIF2_RM_LK_STS0_FIFO_OVF_MASK)>>
		CSL_AIF2_RM_LK_STS0_FIFO_OVF_SHIFT;
	if(uiStatus)
		printf("RM FIFO Overflow occured\n");

	LinkStatus = aif2ErrStsRegs.G_RM_LKS[uiLinkNumber].RM_LK_STS1 ;

	uiStatus= (LinkStatus & CSL_AIF2_RM_LK_STS1_NUM_LOS_MASK)>>
		CSL_AIF2_RM_LK_STS1_NUM_LOS_SHIFT;
	if(uiStatus)
		printf("RM number of los_det = %d\n", uiStatus);

	uiStatus= (LinkStatus & CSL_AIF2_RM_LK_STS1_LCV_CNTR_VALUE_MASK)>>
		CSL_AIF2_RM_LK_STS1_LCV_CNTR_VALUE_SHIFT;
	if(uiStatus)
		printf("RM Number of Line Code Violations = %d\n", uiStatus);

	LinkStatus = aif2ErrStsRegs.G_RM_LKS[uiLinkNumber].RM_LK_STS2 ;

	iClockError= ((LinkStatus & CSL_AIF2_RM_LK_STS2_CLK_QUAL_MASK)>>
		CSL_AIF2_RM_LK_STS2_CLK_QUAL_SHIFT) - 
		(((aif2ErrStsRegs.G_RM_LKS[uiLinkNumber].RM_LK_CFG1&
		CSL_AIF2_RM_LK_CFG1_MON_WRAP_MASK)
		>>CSL_AIF2_RM_LK_CFG1_MON_WRAP_SHIFT)
		<<aifLinkCfg[uiLinkNumber].linkRate);
	if(_abs(iClockError)>4) 
		printf("RM clock error = %d clocks in window\n", iClockError);

	if(aif2ErrStsRegs.G_RM_LKS[uiLinkNumber].RM_LK_CFG0&
		CSL_AIF2_RM_LK_CFG0_SCR_EN_MASK)
	{
		uiStatus= (LinkStatus & CSL_AIF2_RM_LK_STS2_SCR_VALUE_MASK)>>
			CSL_AIF2_RM_LK_STS2_SCR_VALUE_SHIFT;
		printf("RM captured scrambling code = 0x%x\n", uiStatus);
	}
	
	LinkStatus = aif2ErrStsRegs.TM_LK_STAT[uiLinkNumber];

	uiStatus= (LinkStatus & CSL_AIF2_TM_LK_STAT_TM_FAIL_MASK)>>
		CSL_AIF2_TM_LK_STAT_TM_FAIL_SHIFT;
	if(uiStatus)
		printf("TM Fail condition met\n");

	uiStatus= (LinkStatus & CSL_AIF2_TM_LK_STAT_FIFO_UNFL_MASK)>>
		CSL_AIF2_TM_LK_STAT_FIFO_UNFL_SHIFT;
	if(uiStatus)
		printf("Tx FIFO underflow\n");

	uiStatus= (LinkStatus & CSL_AIF2_TM_LK_STAT_FRM_MISALIGN_MASK)>>
		CSL_AIF2_TM_LK_STAT_FRM_MISALIGN_SHIFT;
	if(uiStatus)
		printf("CO Frame alignment mismatch from transmit FSM\n");

	uiStatus= (LinkStatus & CSL_AIF2_TM_LK_STAT_FRM_STATE_MASK)>>
		CSL_AIF2_TM_LK_STAT_FRM_STATE_SHIFT;
	uiStatus= 31-_lmbd(1, uiStatus);
	printf("%s\n", tmStsStr[uiStatus]);

	
}

//print DMA status for AIF
void print_AIF_DMA_status()
{
	Uint32 ErrorStatus, uiStatus;

	printf("Ingress End Of Packet count = %d\n",aif2ErrStsRegs.AD_ISCH_EOP_CNT);
	printf("Egress End Of Packet count = %d\n",aif2ErrStsRegs.DB_EDB_EOP_CNT);

	ErrorStatus= aif2ErrStsRegs.EE_DB_IRS;

	uiStatus= (ErrorStatus&CSL_AIF2_EE_DB_IRS_DB_EE_I_TOKEN_OVFL_ERR_MASK)
		>>CSL_AIF2_EE_DB_IRS_DB_EE_I_TOKEN_OVFL_ERR_SHIFT;
	if(uiStatus)
		puts("Ingress Packet Data Token FIFO overflowed.");

	uiStatus= (ErrorStatus&CSL_AIF2_EE_DB_IRS_DB_EE_I_FIFO_OVFL_ERR_MASK)
		>>CSL_AIF2_EE_DB_IRS_DB_EE_I_FIFO_OVFL_ERR_SHIFT;
	if(uiStatus)
		puts("Ingress FIFO buffer channel overflowed.");

	uiStatus= (ErrorStatus&CSL_AIF2_EE_DB_IRS_DB_EE_I_PD2DB_FULL_ERR_MASK)
		>>CSL_AIF2_EE_DB_IRS_DB_EE_I_PD2DB_FULL_ERR_SHIFT;
	if(uiStatus)
		puts("PD-to-DB Bridge full.");

	uiStatus= (ErrorStatus&CSL_AIF2_EE_DB_IRS_DB_EE_E_PS_AXC_ERR_MASK)
		>>CSL_AIF2_EE_DB_IRS_DB_EE_E_PS_AXC_ERR_SHIFT;
	if(uiStatus)
		puts("AxC in Navigator Protocol Specific data does not match Navigator thread id.");

	uiStatus= (ErrorStatus&CSL_AIF2_EE_DB_IRS_DB_EE_E_CD_DATA_TYPE_ERR_MASK)
		>>CSL_AIF2_EE_DB_IRS_DB_EE_E_CD_DATA_TYPE_ERR_SHIFT;
	if(uiStatus)
		puts("EDB received data from PKTDMA with undefined Navigator data type.");

	ErrorStatus= aif2ErrStsRegs.EE_CD_IRS;

	uiStatus= (ErrorStatus&CSL_AIF2_EE_CD_IRS_CD_EE_SOP_DESC_STARVE_ERR_MASK)
		>>CSL_AIF2_EE_CD_IRS_CD_EE_SOP_DESC_STARVE_ERR_SHIFT;
	if(uiStatus)
		puts("PKTDMA SOP Descriptor Starvation.");

	uiStatus= (ErrorStatus&CSL_AIF2_EE_CD_IRS_CD_EE_MOP_DESC_STARVE_ERR_MASK)
		>>CSL_AIF2_EE_CD_IRS_CD_EE_MOP_DESC_STARVE_ERR_SHIFT;
	if(uiStatus)
		puts("PKTDMA MOP Descriptor Starvation.");

	ErrorStatus= aif2ErrStsRegs.EE_VC_IRS;

	uiStatus= (ErrorStatus&CSL_AIF2_EE_VC_IRS_VC_EE_VBUS_ERR_MASK)
		>>CSL_AIF2_EE_VC_IRS_VC_EE_VBUS_ERR_SHIFT;
	if(uiStatus)
		puts("VBUS data error.");

	ErrorStatus= aif2ErrStsRegs.EE_PE_COMMON_IRS;

	uiStatus= (ErrorStatus&CSL_AIF2_EE_PE_COMMON_IRS_PE_EE_TOKEN_REQ_OVFL_ERR_MASK)
		>>CSL_AIF2_EE_PE_COMMON_IRS_PE_EE_TOKEN_REQ_OVFL_ERR_SHIFT;
	if(uiStatus)
		puts("PE Token request FIFO overflow.");

	uiStatus= (ErrorStatus&CSL_AIF2_EE_PE_COMMON_IRS_PE_EE_DAT_REQ_OVFL_ERR_MASK)
		>>CSL_AIF2_EE_PE_COMMON_IRS_PE_EE_DAT_REQ_OVFL_ERR_SHIFT;
	if(uiStatus)
		puts("PE Data request FIFO overflow.");

}

//print AIF SerDes status
void print_AIF_SerDes_status()
{
	if(aifLinkCfg[0].linkEnable|aifLinkCfg[1].linkEnable
		|aifLinkCfg[2].linkEnable|aifLinkCfg[3].linkEnable)
	{	/*Serdes B8 is used for link 0~3*/
		if(0==(aif2ErrStsRegs.SD_PLL_B8_STS&1))
			puts("Serdes B8 PLL is not locked\n");
	}
	
	if(aifLinkCfg[4].linkEnable|aifLinkCfg[5].linkEnable)
	{	/*Serdes B4 is used for link 4~5*/
		if(0==(aif2ErrStsRegs.SD_PLL_B4_STS&1))
			puts("Serdes B4 PLL is not locked\n");
	}
}

//print AIF link SerDes status
void print_AIF_Link_SerDes_status(Uint32 uiLinkNumber)
{
	Uint32 uiSerDesStatus;

	uiSerDesStatus= aif2ErrStsRegs.SD_LK[uiLinkNumber].SD_RX_STS;

	if((uiSerDesStatus&CSL_AIF2_SD_RX_STS_RXLOSS_OF_SIGNAL_MASK)
		&CSL_AIF2_SD_RX_STS_RXLOSS_OF_SIGNAL_SHIFT)
		printf("Serdes LOS\n");

	if((uiSerDesStatus&CSL_AIF2_SD_RX_STS_RXOFFSET_COMPENSATION_MASK)
		&CSL_AIF2_SD_RX_STS_RXOFFSET_COMPENSATION_SHIFT)
		printf("Serdes Offset compensation in progress\n");

	if((uiSerDesStatus&CSL_AIF2_SD_RX_STS_RXSIGNAL_UNDEREQUALIZED_MASK)
		&CSL_AIF2_SD_RX_STS_RXSIGNAL_UNDEREQUALIZED_SHIFT)
		printf("Serdes equalizer analysis is under equalized\n");

	if((uiSerDesStatus&CSL_AIF2_SD_RX_STS_RXSIGNAL_OVEREQUALIZED_MASK)
		&CSL_AIF2_SD_RX_STS_RXSIGNAL_OVEREQUALIZED_SHIFT)
		printf("Serdes equalizer analysis is over equalized\n");
}

//print AIF Link Sync Timer status
void print_AIF_Link_AT_status(Uint32 uiLinkNumber)
{
	Uint32 uiATStatus;

	uiATStatus= aif2ErrStsRegs.PI_DATA[uiLinkNumber].AT_PIVALUE_LK;
	printf("captured PI value = %d\n", uiATStatus);
}

//print AIF Sync Timer status
void print_AIF_AT_status()
{
	Uint32 uiATStatus, uiStatus;

	printf("AT PHYT Frame= %d, Clock= %d\n",
		aif2ErrStsRegs.AT_PHYT_FRM_VALUE_LSBS, aif2ErrStsRegs.AT_PHYT_CLKCNT_VALUE);

	uiATStatus= aif2ErrStsRegs.AT_RADT_VALUE_LSBS;
	printf("AT RADT Frame= %d, Symbol= %d, Clock= %d\n",
		aif2ErrStsRegs.AT_RADT_VALUE_MID,
		(uiATStatus&CSL_AIF2_AT_RADT_VALUE_LSBS_RADTSYMBOL_COUNT_VALUE_MASK)
			>>CSL_AIF2_AT_RADT_VALUE_LSBS_RADTSYMBOL_COUNT_VALUE_SHIFT,
		(uiATStatus&CSL_AIF2_AT_RADT_VALUE_LSBS_RADTCLOCK_COUNT_VALUE_MASK)
			>>CSL_AIF2_AT_RADT_VALUE_LSBS_RADTCLOCK_COUNT_VALUE_SHIFT);
#if 0
	uiATStatus= aif2ErrStsRegs.AT_CAPTRADT;
	printf("AT RADT captures Frame= %d, Symbol= %d, Clock= %d at PHYT boundary\n",
		(uiATStatus&CSL_AIF2_AT_CAPTRADT_RADTFRAMECOUNT_CAPTURE_MASK)
			>>CSL_AIF2_AT_CAPTRADT_RADTFRAMECOUNT_CAPTURE_SHIFT,
		(uiATStatus&CSL_AIF2_AT_CAPTRADT_RADTSYMBOLCOUNT_CAPTURE_MASK)
			>>CSL_AIF2_AT_CAPTRADT_RADTSYMBOLCOUNT_CAPTURE_SHIFT,
		(uiATStatus&CSL_AIF2_AT_CAPTRADT_RADTCLOCKCOUNT_CAPTURE_MASK)
			>>CSL_AIF2_AT_CAPTRADT_RADTCLOCKCOUNT_CAPTURE_SHIFT);
#endif	
	uiATStatus= aif2ErrStsRegs.EE_AT_IRS;

	uiStatus= (uiATStatus&CSL_AIF2_EE_AT_IRS_AT_EE_PHYT_SYNC_ERR_MASK)
		>>CSL_AIF2_EE_AT_IRS_AT_EE_PHYT_SYNC_ERR_SHIFT;
	if(uiStatus)
		puts("AT PHYT sync input is not aligned to the PHYT counter frame boundary."); 

	uiStatus= (uiATStatus&CSL_AIF2_EE_AT_IRS_AT_EE_RADT_SYNC_ERR_MASK)
		>>CSL_AIF2_EE_AT_IRS_AT_EE_RADT_SYNC_ERR_SHIFT;
	if(uiStatus)
		puts("AT RADT sync input is not aligned to the RADT counter frame boundary."); 

	uiStatus= (uiATStatus&CSL_AIF2_EE_AT_IRS_AT_EE_PI0_ERR_MASK)
		>>CSL_AIF2_EE_AT_IRS_AT_EE_PI0_ERR_SHIFT;
	if(uiStatus)
		puts("AT Link 0 Captured Pi is not within pi window"); 

	uiStatus= (uiATStatus&CSL_AIF2_EE_AT_IRS_AT_EE_PI1_ERR_MASK)
		>>CSL_AIF2_EE_AT_IRS_AT_EE_PI1_ERR_SHIFT;
	if(uiStatus)
		puts("AT Link 1 Captured Pi is not within pi window"); 

	uiStatus= (uiATStatus&CSL_AIF2_EE_AT_IRS_AT_EE_PI2_ERR_MASK)
		>>CSL_AIF2_EE_AT_IRS_AT_EE_PI2_ERR_SHIFT;
	if(uiStatus)
		puts("AT Link 2 Captured Pi is not within pi window"); 

	uiStatus= (uiATStatus&CSL_AIF2_EE_AT_IRS_AT_EE_PI3_ERR_MASK)
		>>CSL_AIF2_EE_AT_IRS_AT_EE_PI3_ERR_SHIFT;
	if(uiStatus)
		puts("AT Link 3 Captured Pi is not within pi window"); 

	uiStatus= (uiATStatus&CSL_AIF2_EE_AT_IRS_AT_EE_PI4_ERR_MASK)
		>>CSL_AIF2_EE_AT_IRS_AT_EE_PI4_ERR_SHIFT;
	if(uiStatus)
		puts("AT Link 4 Captured Pi is not within pi window"); 

	uiStatus= (uiATStatus&CSL_AIF2_EE_AT_IRS_AT_EE_PI5_ERR_MASK)
		>>CSL_AIF2_EE_AT_IRS_AT_EE_PI5_ERR_SHIFT;
	if(uiStatus)
		puts("AT Link 5 Captured Pi is not within pi window"); 
}

//print AIF retransmition/aggregator status
void print_AIF_Link_RT_status(Uint32 uiLinkNumber)
{
	Uint32 uiRTStatus, uiStatus;

	uiRTStatus= aif2ErrStsRegs.G_RT_LKS[uiLinkNumber].RT_LK_DPTH;
	uiStatus = (uiRTStatus&CSL_AIF2_RT_LK_DPTH_RT_DEPTH_MASK)>>
		CSL_AIF2_RT_LK_DPTH_RT_DEPTH_SHIFT;
	if(uiStatus)
		printf("RT FIFO depth = %d\n", uiStatus);

	uiRTStatus= aif2ErrStsRegs.G_RT_LKS[uiLinkNumber].RT_HDR_ERR;
	uiStatus = (uiRTStatus&CSL_AIF2_RT_HDR_ERR_HDR_ERR_MASK)>>
		CSL_AIF2_RT_HDR_ERR_HDR_ERR_SHIFT;
	if(uiStatus)
		printf("link %d RT header error at channel %d\n", 
			(uiRTStatus&CSL_AIF2_RT_HDR_ERR_DMA_CHAN_MASK)>>
				CSL_AIF2_RT_HDR_ERR_DMA_CHAN_SHIFT);

	uiRTStatus= aif2ErrStsRegs.G_RT_LKS[uiLinkNumber].RT_LK_STAT;
	
	uiStatus = (uiRTStatus&CSL_AIF2_RT_LK_STAT_RT_HDR_ERR_STS_MASK)>>
		CSL_AIF2_RT_LK_STAT_RT_HDR_ERR_STS_SHIFT;
	if(uiStatus)
		printf("link %d RT header error has occurred\n"); 

	uiStatus = (uiRTStatus&CSL_AIF2_RT_LK_STAT_RT_EM_STS_MASK)>>
		CSL_AIF2_RT_LK_STAT_RT_EM_STS_SHIFT;
	if(uiStatus)
		printf("link %d RT empty message was inserted\n"); 

	uiStatus = (uiRTStatus&CSL_AIF2_RT_LK_STAT_RT_UNFL_STS_MASK)>>
		CSL_AIF2_RT_LK_STAT_RT_UNFL_STS_SHIFT;
	if(uiStatus)
		printf("link %d RT Underflow of the FIFO has occurred\n"); 

	uiStatus = (uiRTStatus&CSL_AIF2_RT_LK_STAT_RT_OVFL_STS_MASK)>>
		CSL_AIF2_RT_LK_STAT_RT_OVFL_STS_SHIFT;
	if(uiStatus)
		printf("link %d RT Overflow of the FIFO has occurred\n"); 

	uiStatus = (uiRTStatus&CSL_AIF2_RT_LK_STAT_RT_FRM_ERR_STS_MASK)>>
		CSL_AIF2_RT_LK_STAT_RT_FRM_ERR_STS_SHIFT;
	if(uiStatus)
		printf("link %d RT frame error of the FIFO has occurred\n"); 

	uiRTStatus= aif2ErrStsRegs.EE_LK[uiLinkNumber].EE_LK_IRS_B;

	uiStatus= (uiRTStatus&CSL_AIF2_EE_LK_IRS_B_RT_EE_FRM_ERR_MASK)
		>>CSL_AIF2_EE_LK_IRS_B_RT_EE_FRM_ERR_SHIFT;
	if(uiStatus)
		printf("RT The Retransmit frame no longer matches the Transmit frame in Aggregate mode for Link. \n"); 

}

//print AIF Link Protocol Decoder status
void print_AIF_Link_PD_status(Uint32 uiLinkNumber)
{
	Uint32 uiPDStatus, uiStatus;

	uiPDStatus= aif2ErrStsRegs.EE_LK[uiLinkNumber].EE_LK_IRS_B;

	uiStatus= (uiPDStatus&CSL_AIF2_EE_LK_IRS_B_PD_EE_EOP_ERR_MASK)
		>>CSL_AIF2_EE_LK_IRS_B_PD_EE_EOP_ERR_SHIFT;
	if(uiStatus)
		printf("PD Received a second EOP without an SOP in between.\n"); 

	uiStatus= (uiPDStatus&CSL_AIF2_EE_LK_IRS_B_PD_EE_CRC_ERR_MASK)
		>>CSL_AIF2_EE_LK_IRS_B_PD_EE_CRC_ERR_SHIFT;
	if(uiStatus)
		printf("PD CRC failure for any packet (with CRC checking) within the given link\n"); 

	uiStatus= (uiPDStatus&CSL_AIF2_EE_LK_IRS_B_PD_EE_CPRI_FRAME_ERR_MASK)
		>>CSL_AIF2_EE_LK_IRS_B_PD_EE_CPRI_FRAME_ERR_SHIFT;
	if(uiStatus)
		printf("PD Arriving traffic (and user configuration of DBM) is non-multiple of 4 \n"); 

	uiStatus= (uiPDStatus&CSL_AIF2_EE_LK_IRS_B_PD_EE_AXC_FAIL_ERR_MASK)
		>>CSL_AIF2_EE_LK_IRS_B_PD_EE_AXC_FAIL_ERR_SHIFT;
	if(uiStatus)
		printf("PD Unrecoverable OBSAI Time Stamp error for at least 1 AxC of a given link.\n"); 

	uiStatus= (uiPDStatus&CSL_AIF2_EE_LK_IRS_B_PD_EE_SOP_ERR_MASK)
		>>CSL_AIF2_EE_LK_IRS_B_PD_EE_SOP_ERR_SHIFT;
	if(uiStatus)
		printf("PD Received a second SOP without an EOP in between.\n"); 

	uiStatus= (uiPDStatus&CSL_AIF2_EE_LK_IRS_B_PD_EE_OBSAI_FRM_ERR_MASK)
		>>CSL_AIF2_EE_LK_IRS_B_PD_EE_OBSAI_FRM_ERR_SHIFT;
	if(uiStatus)
		printf("PD Wrap of the PD_Frame Counters did not predict a Radio Frame Boundary consistent with TS=0 falling within the reception timing window.\n"); 

}

//print AIF Link Protocol Encoder status
void print_AIF_Link_PE_status(Uint32 uiLinkNumber)
{

	Uint32 uiPEStatus, uiStatus;

	uiPEStatus= aif2ErrStsRegs.EE_LK[uiLinkNumber].EE_LK_IRS_B;

	uiStatus= (uiPEStatus&CSL_AIF2_EE_LK_IRS_B_PE_EE_MODRULE_ERR_MASK)
		>>CSL_AIF2_EE_LK_IRS_B_PE_EE_MODRULE_ERR_SHIFT;
	if(uiStatus)
		printf("PE More than one modulo rule fired in a single clock cycle.\n"); 

	uiStatus= (uiPEStatus&CSL_AIF2_EE_LK_IRS_B_PE_EE_SYM_ERR_MASK)
		>>CSL_AIF2_EE_LK_IRS_B_PE_EE_SYM_ERR_SHIFT;
	if(uiStatus)
		printf("PE Symbol index in Navigator protocol specific header did not match for one or more symbol\n"); 

	uiStatus= (uiPEStatus&CSL_AIF2_EE_LK_IRS_B_PE_EE_MF_FIFO_OVERFLOW_ERR_MASK)
		>>CSL_AIF2_EE_LK_IRS_B_PE_EE_MF_FIFO_OVERFLOW_ERR_SHIFT;
	if(uiStatus)
		printf("PE MF FIFO was full when a write occurred.\n"); 

	uiStatus= (uiPEStatus&CSL_AIF2_EE_LK_IRS_B_PE_EE_MF_FIFO_UNDERFLOW_ERR_MASK)
		>>CSL_AIF2_EE_LK_IRS_B_PE_EE_MF_FIFO_UNDERFLOW_ERR_SHIFT;
	if(uiStatus)
		printf("PE MF FIFO was empty when a read occurred.\n"); 

	uiStatus= (uiPEStatus&CSL_AIF2_EE_LK_IRS_B_PE_EE_DB_STARVE_ERR_MASK)
		>>CSL_AIF2_EE_LK_IRS_B_PE_EE_DB_STARVE_ERR_SHIFT;
	if(uiStatus)
		printf("PE DB did not have antenna data for a AxC channel. Likely to occur if DMA was late.\n"); 

	uiStatus= (uiPEStatus&CSL_AIF2_EE_LK_IRS_B_PE_EE_RT_IF_ERR_MASK)
		>>CSL_AIF2_EE_LK_IRS_B_PE_EE_RT_IF_ERR_SHIFT;
	if(uiStatus)
		printf("PE RT Interface Error.\n"); 

	uiStatus= (uiPEStatus&CSL_AIF2_EE_LK_IRS_B_PE_EE_PKT_STARVE_ERR_MASK)
		>>CSL_AIF2_EE_LK_IRS_B_PE_EE_PKT_STARVE_ERR_SHIFT;
	if(uiStatus)
		printf("PE DB did not have packet data for a pkt channel. \n"); 

}

/*copy AIF2 error status registers to a local data structure for later check/print*/
void copyAif2ErrStsRegs()
{
	int i;

	aif2ErrStsRegs.EE_DB_IRS               = aif2Regs->EE_DB_IRS              ;
	aif2ErrStsRegs.EE_AD_IRS               = aif2Regs->EE_AD_IRS              ;
	aif2ErrStsRegs.EE_CD_IRS               = aif2Regs->EE_CD_IRS              ;
	aif2ErrStsRegs.EE_SD_IRS               = aif2Regs->EE_SD_IRS              ;
	aif2ErrStsRegs.EE_VC_IRS               = aif2Regs->EE_VC_IRS              ;
	aif2ErrStsRegs.EE_AIF2_RUN_STS         = aif2Regs->EE_AIF2_RUN_STS        ;
	aif2ErrStsRegs.EE_AIF2_RUN_CTL         = aif2Regs->EE_AIF2_RUN_CTL        ;
	aif2ErrStsRegs.EE_ERR_ALRM_ORGN        = aif2Regs->EE_ERR_ALRM_ORGN       ;
	aif2ErrStsRegs.SD_PLL_B8_STS           = aif2Regs->SD_PLL_B8_STS          ;
	aif2ErrStsRegs.SD_PLL_B4_STS           = aif2Regs->SD_PLL_B4_STS          ;
	aif2ErrStsRegs.EE_AT_IRS               = aif2Regs->EE_AT_IRS              ;
	aif2ErrStsRegs.EE_PD_COMMON_IRS        = aif2Regs->EE_PD_COMMON_IRS       ;
	aif2ErrStsRegs.EE_PE_COMMON_IRS        = aif2Regs->EE_PE_COMMON_IRS       ;
	aif2ErrStsRegs.AT_CAPTRADT             = aif2Regs->AT_CAPTRADT            ;
	aif2ErrStsRegs.AT_PHYT_CLKCNT_VALUE    = aif2Regs->AT_PHYT_CLKCNT_VALUE   ;
	aif2ErrStsRegs.AT_PHYT_FRM_VALUE_LSBS  = aif2Regs->AT_PHYT_FRM_VALUE_LSBS ;
	aif2ErrStsRegs.AT_PHYT_FRM_VALUE_MSBS  = aif2Regs->AT_PHYT_FRM_VALUE_MSBS ;
	aif2ErrStsRegs.AT_RADT_VALUE_LSBS      = aif2Regs->AT_RADT_VALUE_LSBS     ;
	aif2ErrStsRegs.AT_RADT_VALUE_MID       = aif2Regs->AT_RADT_VALUE_MID      ;
	aif2ErrStsRegs.AT_RADT_VALUE_MSBS      = aif2Regs->AT_RADT_VALUE_MSBS     ;
	aif2ErrStsRegs.AT_ULRADT_VALUE_LSBS    = aif2Regs->AT_ULRADT_VALUE_LSBS   ;
	aif2ErrStsRegs.AT_ULRADT_VALUE_MID     = aif2Regs->AT_ULRADT_VALUE_MID    ;
	aif2ErrStsRegs.AT_ULRADT_VALUE_MSBS    = aif2Regs->AT_ULRADT_VALUE_MSBS   ;
	aif2ErrStsRegs.AT_DLRADT_VALUE_LSBS    = aif2Regs->AT_DLRADT_VALUE_LSBS   ;
	aif2ErrStsRegs.AT_DLRADT_VALUE_MID     = aif2Regs->AT_DLRADT_VALUE_MID    ;
	aif2ErrStsRegs.AT_DLRADT_VALUE_MSBS    = aif2Regs->AT_DLRADT_VALUE_MSBS   ;
	aif2ErrStsRegs.AT_RADT_WCDMA_VALUE     = aif2Regs->AT_RADT_WCDMA_VALUE    ;
	aif2ErrStsRegs.AT_ULRADT_WCDMA_VALUE   = aif2Regs->AT_ULRADT_WCDMA_VALUE  ;
	aif2ErrStsRegs.AT_DLRADT_WCDMA_VALUE   = aif2Regs->AT_DLRADT_WCDMA_VALUE  ;
	aif2ErrStsRegs.PD_GLOBAL_EN_STS        = aif2Regs->PD_GLOBAL_EN_STS       ;
	aif2ErrStsRegs.AD_ISCH_EOP_CNT         = aif2Regs->AD_ISCH_EOP_CNT  ;
	aif2ErrStsRegs.DB_EDB_EOP_CNT          = aif2Regs->DB_EDB_EOP_CNT       ;

	for(i=0; i<NUM_AIF2_LINK; i++)	//for all links
	{
		if(!aifLinkCfg[i].linkEnable)
			continue;

		aif2ErrStsRegs.SD_LK[i].SD_RX_STS = aif2Regs->SD_LK[i].SD_RX_STS;
		aif2ErrStsRegs.SD_LK[i].SD_TX_STS = aif2Regs->SD_LK[i].SD_TX_STS;

		aif2ErrStsRegs.EE_LK[i].EE_LK_IRS_A = aif2Regs->EE_LK[i].EE_LK_IRS_A;
		aif2ErrStsRegs.EE_LK[i].EE_LK_IRS_B = aif2Regs->EE_LK[i].EE_LK_IRS_B;

		aif2ErrStsRegs.PI_DATA[i].AT_PIMAX_LK   = aif2Regs->PI_DATA[i].AT_PIMAX_LK  ; 
		aif2ErrStsRegs.PI_DATA[i].AT_PIMIN_LK   = aif2Regs->PI_DATA[i].AT_PIMIN_LK  ; 
		aif2ErrStsRegs.PI_DATA[i].AT_PIVALUE_LK = aif2Regs->PI_DATA[i].AT_PIVALUE_LK; 

		aif2ErrStsRegs.TM_LK_STAT[i] = aif2Regs->G_TM_LKS[i].TM_LK_STAT;

		aif2ErrStsRegs.G_RM_LKS[i].RM_LK_CFG0 = aif2Regs->G_RM_LKS[i].RM_LK_CFG0;
		aif2ErrStsRegs.G_RM_LKS[i].RM_LK_CFG1 = aif2Regs->G_RM_LKS[i].RM_LK_CFG1;
		aif2ErrStsRegs.G_RM_LKS[i].RM_LK_STS0 = aif2Regs->G_RM_LKS[i].RM_LK_STS0;
		aif2ErrStsRegs.G_RM_LKS[i].RM_LK_STS1 = aif2Regs->G_RM_LKS[i].RM_LK_STS1;
		aif2ErrStsRegs.G_RM_LKS[i].RM_LK_STS2 = aif2Regs->G_RM_LKS[i].RM_LK_STS2;
		aif2ErrStsRegs.G_RM_LKS[i].RM_LK_STS3 = aif2Regs->G_RM_LKS[i].RM_LK_STS3;

		aif2ErrStsRegs.G_RT_LKS[i].RT_LK_DPTH = aif2Regs->G_RT_LKS[i].RT_LK_DPTH;
		aif2ErrStsRegs.G_RT_LKS[i].RT_HDR_ERR = aif2Regs->G_RT_LKS[i].RT_HDR_ERR;
		aif2ErrStsRegs.G_RT_LKS[i].RT_LK_STAT = aif2Regs->G_RT_LKS[i].RT_LK_STAT;

	}

    memcpy((void *)aif2ErrStsRegs.PD_CHAN_STS, (void *)aif2Regs->PD_CHAN_STS, sizeof(aif2ErrStsRegs.PD_CHAN_STS));
    memcpy((void *)aif2ErrStsRegs.PD_PKT_STS , (void *)aif2Regs->PD_PKT_STS , sizeof(aif2ErrStsRegs.PD_PKT_STS ));
    memcpy((void *)aif2ErrStsRegs.PE_CHAN_STS, (void *)aif2Regs->PE_CHAN_STS, sizeof(aif2ErrStsRegs.PE_CHAN_STS));
    memcpy((void *)aif2ErrStsRegs.PE_PKT_STS , (void *)aif2Regs->PE_PKT_STS , sizeof(aif2ErrStsRegs.PE_PKT_STS ));

}

/*copy all used AIF2 registers to a local data structure for later check*/
void copyAif2Regs()
{
	int i;
	Uint32 uiDBM_X, uiMax_AT_TC, uiMax_PE_TC, uiMax_PD_TC;

	memset((void *)&aif2RegsLog,0,sizeof(aif2RegsLog));
	
	aif2RegsLog.AIF2_PID                    = aif2Regs->AIF2_PID                 ;
	aif2RegsLog.AIF2_SCRATCH                = aif2Regs->AIF2_SCRATCH             ;
	aif2RegsLog.AIF2_RESET                  = aif2Regs->AIF2_RESET               ;
	aif2RegsLog.AIF2_EMU                    = aif2Regs->AIF2_EMU                 ;
	aif2RegsLog.VC_STAT                     = aif2Regs->VC_STAT                  ;
	aif2RegsLog.EE_VB_EOI                   = aif2Regs->EE_VB_EOI                ;
	aif2RegsLog.EE_DB_IRS                   = aif2Regs->EE_DB_IRS                ;
	aif2RegsLog.EE_DB_EN_EV0                = aif2Regs->EE_DB_EN_EV0             ;
	aif2RegsLog.EE_DB_EN_EV1                = aif2Regs->EE_DB_EN_EV1             ;
	aif2RegsLog.EE_DB_EN_STS_EV0            = aif2Regs->EE_DB_EN_STS_EV0         ;
	aif2RegsLog.EE_DB_EN_STS_EV1            = aif2Regs->EE_DB_EN_STS_EV1         ;
	aif2RegsLog.EE_AD_IRS                   = aif2Regs->EE_AD_IRS                ;
	aif2RegsLog.EE_AD_EN_EV0                = aif2Regs->EE_AD_EN_EV0             ;
	aif2RegsLog.EE_AD_EN_EV1                = aif2Regs->EE_AD_EN_EV1             ;
	aif2RegsLog.EE_AD_EN_STS_EV0            = aif2Regs->EE_AD_EN_STS_EV0         ;
	aif2RegsLog.EE_AD_EN_STS_EV1            = aif2Regs->EE_AD_EN_STS_EV1         ;
	aif2RegsLog.EE_CD_IRS                   = aif2Regs->EE_CD_IRS                ;
	aif2RegsLog.EE_CD_EN_EV                 = aif2Regs->EE_CD_EN_EV              ;
	aif2RegsLog.EE_CD_EN_STS_EV             = aif2Regs->EE_CD_EN_STS_EV          ;
	aif2RegsLog.EE_SD_IRS                   = aif2Regs->EE_SD_IRS                ;
	aif2RegsLog.EE_SD_EN_EV0                = aif2Regs->EE_SD_EN_EV0             ;
	aif2RegsLog.EE_SD_EN_EV1                = aif2Regs->EE_SD_EN_EV1             ;
	aif2RegsLog.EE_SD_EN_STS_EV0            = aif2Regs->EE_SD_EN_STS_EV0         ;
	aif2RegsLog.EE_SD_EN_STS_EV1            = aif2Regs->EE_SD_EN_STS_EV1         ;
	aif2RegsLog.EE_VC_IRS                   = aif2Regs->EE_VC_IRS                ;
	aif2RegsLog.EE_VC_EN_EV0                = aif2Regs->EE_VC_EN_EV0             ;
	aif2RegsLog.EE_VC_EN_EV1                = aif2Regs->EE_VC_EN_EV1             ;
	aif2RegsLog.EE_VC_EN_STS_EV0            = aif2Regs->EE_VC_EN_STS_EV0         ;
	aif2RegsLog.EE_VC_EN_STS_EV1            = aif2Regs->EE_VC_EN_STS_EV1         ;
	aif2RegsLog.EE_AIF2_RUN_STS             = aif2Regs->EE_AIF2_RUN_STS          ;
	aif2RegsLog.EE_AIF2_RUN_CTL             = aif2Regs->EE_AIF2_RUN_CTL          ;
	aif2RegsLog.EE_ERR_ALRM_ORGN            = aif2Regs->EE_ERR_ALRM_ORGN         ;
	aif2RegsLog.SD_PLL_B8_EN_CFG            = aif2Regs->SD_PLL_B8_EN_CFG         ;
	aif2RegsLog.SD_PLL_B4_EN_CFG            = aif2Regs->SD_PLL_B4_EN_CFG         ;
	aif2RegsLog.SD_PLL_B8_CFG               = aif2Regs->SD_PLL_B8_CFG            ;
	aif2RegsLog.SD_PLL_B4_CFG               = aif2Regs->SD_PLL_B4_CFG            ;
	aif2RegsLog.SD_PLL_B8_STS               = aif2Regs->SD_PLL_B8_STS            ;
	aif2RegsLog.SD_PLL_B4_STS               = aif2Regs->SD_PLL_B4_STS            ;
	aif2RegsLog.SD_CLK_SEL_CFG              = aif2Regs->SD_CLK_SEL_CFG           ;
	aif2RegsLog.SD_LK_CLK_DIS_CFG           = aif2Regs->SD_LK_CLK_DIS_CFG        ;
	aif2RegsLog.AD_DIO_DT_DMA_CFG0          = aif2Regs->AD_DIO_DT_DMA_CFG0       ;
	aif2RegsLog.AD_DIO_DT_DMA_CFG1          = aif2Regs->AD_DIO_DT_DMA_CFG1       ;
	aif2RegsLog.AD_DIO_DT_DMA_CFG2          = aif2Regs->AD_DIO_DT_DMA_CFG2       ;
	aif2RegsLog.AD_DIO_DT_DMA_CFG3          = aif2Regs->AD_DIO_DT_DMA_CFG3       ;
	aif2RegsLog.AD_DIO_I_GLOBAL_EN_STS      = aif2Regs->AD_DIO_I_GLOBAL_EN_STS   ;
	aif2RegsLog.AD_DIO_E_GLOBAL_EN_STS      = aif2Regs->AD_DIO_E_GLOBAL_EN_STS   ;
	aif2RegsLog.AD_ISCH_CFG                 = aif2Regs->AD_ISCH_CFG              ;
	aif2RegsLog.AD_ISCH_GLOBAL_EN_STS       = aif2Regs->AD_ISCH_GLOBAL_EN_STS    ;
	aif2RegsLog.AD_ISCH_EOP_CNT             = aif2Regs->AD_ISCH_EOP_CNT          ;
	aif2RegsLog.AD_ESCH_CFG                 = aif2Regs->AD_ESCH_CFG              ;
	aif2RegsLog.AD_ESCH_GLOBAL_EN_STS       = aif2Regs->AD_ESCH_GLOBAL_EN_STS    ;
	aif2RegsLog.DB_IDB_CFG                  = aif2Regs->DB_IDB_CFG               ;
	aif2RegsLog.DB_IDB_GLOBAL_EN_STS        = aif2Regs->DB_IDB_GLOBAL_EN_STS     ;
	aif2RegsLog.DB_EDB_CFG                  = aif2Regs->DB_EDB_CFG               ;
	aif2RegsLog.DB_EDB_GLOBAL_EN_STS        = aif2Regs->DB_EDB_GLOBAL_EN_STS     ;
	aif2RegsLog.DB_EDB_EOP_CNT              = aif2Regs->DB_EDB_EOP_CNT           ;
	aif2RegsLog.REVISION_REG                = aif2Regs->REVISION_REG             ;
	aif2RegsLog.PERF_CTRL_REG               = aif2Regs->PERF_CTRL_REG            ;
	aif2RegsLog.EMU_CTRL_REG                = aif2Regs->EMU_CTRL_REG             ;
	aif2RegsLog.PRI_CTRL_REG                = aif2Regs->PRI_CTRL_REG             ;
	aif2RegsLog.QM0_BA_REG                  = aif2Regs->QM0_BA_REG               ;
	aif2RegsLog.QM1_BA_REG                  = aif2Regs->QM1_BA_REG               ;
	aif2RegsLog.QM2_BA_REG                  = aif2Regs->QM2_BA_REG               ;
	aif2RegsLog.QM3_BA_REG                  = aif2Regs->QM3_BA_REG               ;
	aif2RegsLog.EE_AT_IRS                   = aif2Regs->EE_AT_IRS                ;
	aif2RegsLog.EE_AT_EN_EV0                = aif2Regs->EE_AT_EN_EV0             ;
	aif2RegsLog.EE_AT_EN_EV1                = aif2Regs->EE_AT_EN_EV1             ;
	aif2RegsLog.EE_AT_EN_STS_EV0            = aif2Regs->EE_AT_EN_STS_EV0         ;
	aif2RegsLog.EE_AT_EN_STS_EV1            = aif2Regs->EE_AT_EN_STS_EV1         ;
	aif2RegsLog.EE_PD_COMMON_IRS            = aif2Regs->EE_PD_COMMON_IRS         ;
	aif2RegsLog.EE_PE_COMMON_IRS            = aif2Regs->EE_PE_COMMON_IRS         ;
	aif2RegsLog.AT_CONTROL1                 = aif2Regs->AT_CONTROL1              ;
	aif2RegsLog.AT_CONTROL2                 = aif2Regs->AT_CONTROL2              ;
	aif2RegsLog.AT_SW_SYNC                  = aif2Regs->AT_SW_SYNC               ;
	aif2RegsLog.AT_PHYT_CMP_RADSYNC         = aif2Regs->AT_PHYT_CMP_RADSYNC      ;
	aif2RegsLog.AT_RP1_TYPE                 = aif2Regs->AT_RP1_TYPE              ;
	aif2RegsLog.AT_CAPTRADT                 = aif2Regs->AT_CAPTRADT              ;
	aif2RegsLog.AT_RP1_TYPE_CAPTURE         = aif2Regs->AT_RP1_TYPE_CAPTURE      ;
	aif2RegsLog.AT_RP1_TOD_CAPTURE_L        = aif2Regs->AT_RP1_TOD_CAPTURE_L     ;
	aif2RegsLog.AT_RP1_TOD_CAPTURE_H        = aif2Regs->AT_RP1_TOD_CAPTURE_H     ;
	aif2RegsLog.AT_RP1_RP3_CAPTURE_L        = aif2Regs->AT_RP1_RP3_CAPTURE_L     ;
	aif2RegsLog.AT_RP1_RP3_CAPTURE_H        = aif2Regs->AT_RP1_RP3_CAPTURE_H     ;
	aif2RegsLog.AT_RP1_RAD_CAPTURE_L        = aif2Regs->AT_RP1_RAD_CAPTURE_L     ;
	aif2RegsLog.AT_RP1_RAD_CAPTURE_H        = aif2Regs->AT_RP1_RAD_CAPTURE_H     ;
	aif2RegsLog.AT_PHYT_CLKCNT_VALUE        = aif2Regs->AT_PHYT_CLKCNT_VALUE     ;
	aif2RegsLog.AT_PHYT_FRM_VALUE_LSBS      = aif2Regs->AT_PHYT_FRM_VALUE_LSBS   ;
	aif2RegsLog.AT_PHYT_FRM_VALUE_MSBS      = aif2Regs->AT_PHYT_FRM_VALUE_MSBS   ;
	aif2RegsLog.AT_RADT_VALUE_LSBS          = aif2Regs->AT_RADT_VALUE_LSBS       ;
	aif2RegsLog.AT_RADT_VALUE_MID           = aif2Regs->AT_RADT_VALUE_MID        ;
	aif2RegsLog.AT_RADT_VALUE_MSBS          = aif2Regs->AT_RADT_VALUE_MSBS       ;
	aif2RegsLog.AT_ULRADT_VALUE_LSBS        = aif2Regs->AT_ULRADT_VALUE_LSBS     ;
	aif2RegsLog.AT_ULRADT_VALUE_MID         = aif2Regs->AT_ULRADT_VALUE_MID      ;
	aif2RegsLog.AT_ULRADT_VALUE_MSBS        = aif2Regs->AT_ULRADT_VALUE_MSBS     ;
	aif2RegsLog.AT_DLRADT_VALUE_LSBS        = aif2Regs->AT_DLRADT_VALUE_LSBS     ;
	aif2RegsLog.AT_DLRADT_VALUE_MID         = aif2Regs->AT_DLRADT_VALUE_MID      ;
	aif2RegsLog.AT_DLRADT_VALUE_MSBS        = aif2Regs->AT_DLRADT_VALUE_MSBS     ;
	aif2RegsLog.AT_RADT_WCDMA_VALUE         = aif2Regs->AT_RADT_WCDMA_VALUE      ;
	aif2RegsLog.AT_ULRADT_WCDMA_VALUE       = aif2Regs->AT_ULRADT_WCDMA_VALUE    ;
	aif2RegsLog.AT_DLRADT_WCDMA_VALUE       = aif2Regs->AT_DLRADT_WCDMA_VALUE    ;
	aif2RegsLog.AT_RADT_WCDMA_DIV           = aif2Regs->AT_RADT_WCDMA_DIV        ;
	aif2RegsLog.AT_PHYT_INIT_LSBS           = aif2Regs->AT_PHYT_INIT_LSBS        ;
	aif2RegsLog.AT_PHYT_INIT_MID            = aif2Regs->AT_PHYT_INIT_MID         ;
	aif2RegsLog.AT_PHYT_INIT_MSBS           = aif2Regs->AT_PHYT_INIT_MSBS        ;
	aif2RegsLog.AT_PHYT_TC_LSBS             = aif2Regs->AT_PHYT_TC_LSBS          ;
	aif2RegsLog.AT_PHYT_FRAME_TC_LSBS       = aif2Regs->AT_PHYT_FRAME_TC_LSBS    ;
	aif2RegsLog.AT_PHYT_FRAME_TC_MSBS       = aif2Regs->AT_PHYT_FRAME_TC_MSBS    ;
	aif2RegsLog.AT_RADT_INIT_LSBS           = aif2Regs->AT_RADT_INIT_LSBS        ;
	aif2RegsLog.AT_RADT_INIT_MID            = aif2Regs->AT_RADT_INIT_MID         ;
	aif2RegsLog.AT_RADT_INIT_MSBS           = aif2Regs->AT_RADT_INIT_MSBS        ;
	aif2RegsLog.AT_ULRADT_INIT_LSBS         = aif2Regs->AT_ULRADT_INIT_LSBS      ;
	aif2RegsLog.AT_RADT_TSTAMP_VALUE        = aif2Regs->AT_RADT_TSTAMP_VALUE     ;
	aif2RegsLog.AT_DLRADT_INIT_LSBS         = aif2Regs->AT_DLRADT_INIT_LSBS      ;
	aif2RegsLog.AT_GSM_TCOUNT_INIT          = aif2Regs->AT_GSM_TCOUNT_INIT       ;
	aif2RegsLog.AT_GSM_TCOUNT_VALUE         = aif2Regs->AT_GSM_TCOUNT_VALUE      ;
	aif2RegsLog.AT_RADT_SYMB_LUT_INDEX_TC   = aif2Regs->AT_RADT_SYMB_LUT_INDEX_TC;
	aif2RegsLog.AT_RADT_FRAME_TC_LSBS       = aif2Regs->AT_RADT_FRAME_TC_LSBS    ;
	aif2RegsLog.AT_RADT_FRAME_TC_MSBS       = aif2Regs->AT_RADT_FRAME_TC_MSBS    ;
	aif2RegsLog.AT_ULRADT_INIT_MID          = aif2Regs->AT_ULRADT_INIT_MID       ;
	aif2RegsLog.AT_ULRADT_INIT_MSBS         = aif2Regs->AT_ULRADT_INIT_MSBS      ;
	aif2RegsLog.AT_DLRADT_INIT_MID          = aif2Regs->AT_DLRADT_INIT_MID       ;
	aif2RegsLog.AT_DLRADT_INIT_MSBS         = aif2Regs->AT_DLRADT_INIT_MSBS      ;
	aif2RegsLog.AT_RADT_INIT_LUT_INDEX      = aif2Regs->AT_RADT_INIT_LUT_INDEX   ;
	aif2RegsLog.AT_ULRADT_INIT_LUT_INDEX    = aif2Regs->AT_ULRADT_INIT_LUT_INDEX ;
	aif2RegsLog.AT_DLRADT_INIT_LUT_INDEX    = aif2Regs->AT_DLRADT_INIT_LUT_INDEX ;
	aif2RegsLog.AT_NEG_DELTA                = aif2Regs->AT_NEG_DELTA             ;
	aif2RegsLog.AT_EVT_ENABLE               = aif2Regs->AT_EVT_ENABLE            ;
	aif2RegsLog.AT_EVT_FORCE                = aif2Regs->AT_EVT_FORCE             ;
	aif2RegsLog.AT_INTERNAL_EVT_ENABLE      = aif2Regs->AT_INTERNAL_EVT_ENABLE   ;
	aif2RegsLog.AT_INTERNAL_EVT_FORCE       = aif2Regs->AT_INTERNAL_EVT_FORCE    ;
	aif2RegsLog.RM_CFG                      = aif2Regs->RM_CFG                   ;
	aif2RegsLog.PD_GLOBAL                   = aif2Regs->PD_GLOBAL                ;
	aif2RegsLog.PD_GLOBAL_EN_STS            = aif2Regs->PD_GLOBAL_EN_STS         ;
	aif2RegsLog.PD_DMA                      = aif2Regs->PD_DMA                   ;
	aif2RegsLog.PD_RADT_TC                  = aif2Regs->PD_RADT_TC               ;
	aif2RegsLog.PE_GLOBAL                   = aif2Regs->PE_GLOBAL                ;
	aif2RegsLog.PE_GLOBAL_EN_STS            = aif2Regs->PE_GLOBAL_EN_STS         ;

	for(i=0; i<NUM_AIF2_LINK; i++)	//for all links
	{
		if(!aifLinkCfg[i].linkEnable)
			continue;

		memcpy((void *)&aif2RegsLog.SD_LK[i]   , (void *)&aif2Regs->SD_LK[i]   ,sizeof(CSL_Aif2Sd_lkRegs   )-2016);
		memcpy((void *)&aif2RegsLog.EE_LK[i]   , (void *)&aif2Regs->EE_LK[i]   ,sizeof(CSL_Aif2Ee_lkRegs   )-40  );
		memcpy((void *)&aif2RegsLog.G_TM_LKS[i], (void *)&aif2Regs->G_TM_LKS[i],sizeof(CSL_Aif2G_tm_lksRegs)-1984);
		memcpy((void *)&aif2RegsLog.G_RM_LKS[i], (void *)&aif2Regs->G_RM_LKS[i],sizeof(CSL_Aif2G_rm_lksRegs)-2008);
		memcpy((void *)&aif2RegsLog.G_RT_LKS[i], (void *)&aif2Regs->G_RT_LKS[i],sizeof(CSL_Aif2G_rt_lksRegs)-2032);

		aif2RegsLog.G_CI_LKS[i].CI_LK_CFG   = aif2Regs->G_CI_LKS[i].CI_LK_CFG  ;
		aif2RegsLog.G_CO_LKS[i].CO_LK_CFG   = aif2Regs->G_CO_LKS[i].CO_LK_CFG  ;

		aif2RegsLog.G_PD_LKS[i].PD_LINK_A   = aif2Regs->G_PD_LKS[i].PD_LINK_A  ;
		aif2RegsLog.G_PD_LKS[i].PD_LINK_B   = aif2Regs->G_PD_LKS[i].PD_LINK_B  ;

		aif2RegsLog.G_PE_LKS[i].PE_LINK     = aif2Regs->G_PE_LKS[i].PE_LINK    ;
		aif2RegsLog.G_PE_LKS[i].PE_CRC      = aif2Regs->G_PE_LKS[i].PE_CRC     ;

#ifdef AIF2_LINK_PROTOCOL_OBSAI
		memcpy((void *)aif2RegsLog.G_PD_LKS[i].PD_TYPE_LUT   , (void *)aif2Regs->G_PD_LKS[i].PD_TYPE_LUT   , sizeof(aif2RegsLog.G_PD_LKS[i].PD_TYPE_LUT   ));
#else 	//CPRI
		aif2RegsLog.G_PD_LKS[i].PD_CPRI_CRC = aif2Regs->G_PD_LKS[i].PD_CPRI_CRC;
		aif2RegsLog.G_PD_LKS[i].PD_PACK_MAP = aif2Regs->G_PD_LKS[i].PD_PACK_MAP;
		aif2RegsLog.G_PD_LKS[i].PD_DBM      = aif2Regs->G_PD_LKS[i].PD_DBM     ;

		memcpy((void *)aif2RegsLog.G_PD_LKS[i].PD_DBM_1MAP   , (void *)aif2Regs->G_PD_LKS[i].PD_DBM_1MAP   , sizeof(aif2RegsLog.G_PD_LKS[i].PD_DBM_1MAP   ));
		memcpy((void *)aif2RegsLog.G_PD_LKS[i].PD_DBM_2MAP   , (void *)aif2Regs->G_PD_LKS[i].PD_DBM_2MAP   , sizeof(aif2RegsLog.G_PD_LKS[i].PD_DBM_2MAP   ));
		uiDBM_X= (aif2Regs->G_PD_LKS[i].PD_DBM&CSL_AIF2_PD_DBM_DBM_X_MASK)+1;
		memcpy((void *)aif2RegsLog.G_PD_LKS[i].PD_CPRI_ID_LUT, (void *)aif2Regs->G_PD_LKS[i].PD_CPRI_ID_LUT, uiDBM_X*4);
		memcpy((void *)aif2RegsLog.G_PD_LKS[i].PD_CW_LUT     , (void *)aif2Regs->G_PD_LKS[i].PD_CW_LUT     , sizeof(aif2RegsLog.G_PD_LKS[i].PD_CW_LUT     ));

		aif2RegsLog.G_PE_LKS[i].PE_CPRI_DBM = aif2Regs->G_PE_LKS[i].PE_CPRI_DBM;
		aif2RegsLog.G_PE_LKS[i].PE_CPRI0    = aif2Regs->G_PE_LKS[i].PE_CPRI0   ;
		aif2RegsLog.G_PE_LKS[i].PE_CPRI1    = aif2Regs->G_PE_LKS[i].PE_CPRI1   ;
		memcpy((void *)aif2RegsLog.G_PE_LKS[i].PE_CPRIDBM_1MAP, (void *)aif2Regs->G_PE_LKS[i].PE_CPRIDBM_1MAP, sizeof(aif2RegsLog.G_PE_LKS[i].PE_CPRIDBM_1MAP));
		memcpy((void *)aif2RegsLog.G_PE_LKS[i].PE_CPRIDBM_2MAP, (void *)aif2Regs->G_PE_LKS[i].PE_CPRIDBM_2MAP, sizeof(aif2RegsLog.G_PE_LKS[i].PE_CPRIDBM_2MAP));
		memcpy((void *)aif2RegsLog.G_PE_LKS[i].PE_CPRI_CW_LUT , (void *)aif2Regs->G_PE_LKS[i].PE_CPRI_CW_LUT , sizeof(aif2RegsLog.G_PE_LKS[i].PE_CPRI_CW_LUT ));
#endif
	}

#ifdef AIF2_LINK_PROTOCOL_OBSAI
	memcpy((void *)aif2RegsLog.PE_MODTXRULE    , (void *)aif2Regs->PE_MODTXRULE    , sizeof(aif2RegsLog.PE_MODTXRULE    ));
	for(i=0;i<64;i++)
	{
		if(aif2RegsLog.PE_MODTXRULE[i]&CSL_AIF2_PE_MODTXRULE_RULE_EN_MASK)
		{
			aif2RegsLog.PE_OBSAI_DBM[i] = aif2Regs->PE_OBSAI_DBM[i];
			uiDBM_X = (aif2RegsLog.PE_OBSAI_DBM[i]&CSL_AIF2_PE_OBSAI_DBM_DBM_X_MASK)+1;
			memcpy((void *)&aif2RegsLog.PE_DBM_MAP[i*8], (void *)&aif2Regs->PE_DBM_MAP[i*8], 8*4);
			memcpy((void *)&aif2RegsLog.PE_RULE_CHANLUT0[i*64], (void *)&aif2Regs->PE_RULE_CHANLUT0[i*64], uiDBM_X*4);
		}
	}
#else 	//CPRI
	for(i=0;i<6;i++)
	{
		uiDBM_X= (aif2Regs->G_PE_LKS[i].PE_CPRI_DBM&CSL_AIF2_PE_CPRI_DBM_CPRI_DBM_X_MASK)+1;
		memcpy((void *)&aif2RegsLog.PE_RULE_CHANLUT0[i*512], (void *)&aif2Regs->PE_RULE_CHANLUT0[i*512], uiDBM_X*4);
	}
#endif

	memcpy((void *)aif2RegsLog.DB_IDB_PTR_CH, (void *)aif2Regs->DB_IDB_PTR_CH, uiNumAifChannel*4);
	memcpy((void *)aif2RegsLog.DB_IDB_CFG_CH, (void *)aif2Regs->DB_IDB_CFG_CH, uiNumAifChannel*4);
	memcpy((void *)aif2RegsLog.DB_EDB_PTR_CH, (void *)aif2Regs->DB_EDB_PTR_CH, uiNumAifChannel*4);
	memcpy((void *)aif2RegsLog.DB_EDB_CFG_CH, (void *)aif2Regs->DB_EDB_CFG_CH, uiNumAifChannel*4);
	memcpy((void *)aif2RegsLog.PD_ROUTE     , (void *)aif2Regs->PD_ROUTE     , uiNumAifChannel*4);
	memcpy((void *)aif2RegsLog.PD_DMACHAN   , (void *)aif2Regs->PD_DMACHAN   , uiNumAifChannel*4);
	memcpy((void *)aif2RegsLog.PD_DMACHAN_A , (void *)aif2Regs->PD_DMACHAN_A , uiNumAifChannel*4);
	memcpy((void *)aif2RegsLog.PD_DMACHAN_B , (void *)aif2Regs->PD_DMACHAN_B , uiNumAifChannel*4);
	memcpy((void *)aif2RegsLog.PD_DMACHAN_C , (void *)aif2Regs->PD_DMACHAN_C , uiNumAifChannel*4);
	memcpy((void *)aif2RegsLog.PD_DMACHAN_D , (void *)aif2Regs->PD_DMACHAN_D , uiNumAifChannel*4);
	memcpy((void *)aif2RegsLog.PD_DMACHAN_E , (void *)aif2Regs->PD_DMACHAN_E , uiNumAifChannel*4);
	memcpy((void *)aif2RegsLog.PD_DMACHAN_F , (void *)aif2Regs->PD_DMACHAN_F , uiNumAifChannel*4);
	memcpy((void *)aif2RegsLog.PE_DMACHAN_EN, (void *)aif2Regs->PE_DMACHAN_EN, uiNumAifChannel*4);
	memcpy((void *)aif2RegsLog.PE_DMA0CHAN  , (void *)aif2Regs->PE_DMA0CHAN  , uiNumAifChannel*4);
	memcpy((void *)aif2RegsLog.PE_IN_FIFO   , (void *)aif2Regs->PE_IN_FIFO   , uiNumAifChannel*4);
	memcpy((void *)aif2RegsLog.PE_AXC_OFFSET, (void *)aif2Regs->PE_AXC_OFFSET, uiNumAifChannel*4);
	memcpy((void *)aif2RegsLog.PE_OBSAI_HDR , (void *)aif2Regs->PE_OBSAI_HDR , uiNumAifChannel*4);

	for(i= 0; i<uiNumAifChannel; i++)
	{
		aif2RegsLog.TCHAN[i].TCHAN_GCFG_REG_A = aif2Regs->TCHAN[i].TCHAN_GCFG_REG_A;
		aif2RegsLog.RCHAN[i].RCHAN_GCFG_REG_A = aif2Regs->RCHAN[i].RCHAN_GCFG_REG_A;
	}
	aif2RegsLog.TCHAN[128].TCHAN_GCFG_REG_A = aif2Regs->TCHAN[128].TCHAN_GCFG_REG_A;
	aif2RegsLog.RCHAN[128].RCHAN_GCFG_REG_A = aif2Regs->RCHAN[128].RCHAN_GCFG_REG_A;

	memcpy((void *)aif2RegsLog.DB_IDB_CH_EN   , (void *)aif2Regs->DB_IDB_CH_EN   , 16);
	memcpy((void *)aif2RegsLog.DB_IDB_CH_EMPTY, (void *)aif2Regs->DB_IDB_CH_EMPTY, 16);
	memcpy((void *)aif2RegsLog.DB_EDB_CH_EN   , (void *)aif2Regs->DB_EDB_CH_EN   , 16);
	memcpy((void *)aif2RegsLog.PD_CHAN_STS    , (void *)aif2Regs->PD_CHAN_STS    , 16);
	memcpy((void *)aif2RegsLog.PD_PKT_STS     , (void *)aif2Regs->PD_PKT_STS     , 16);
	memcpy((void *)aif2RegsLog.PE_CHAN_STS    , (void *)aif2Regs->PE_CHAN_STS    , 16);
	memcpy((void *)aif2RegsLog.PE_PKT_STS     , (void *)aif2Regs->PE_PKT_STS     , 16);

	if(aif2Regs->PD_GLOBAL&CSL_AIF2_PD_GLOBAL_DIO_CPPI_MASK)
	{
		for(i=0;i<3;i++)
		{
			memcpy((void *)&aif2RegsLog.AD_DIO_INGRESS[i], (void *)&aif2Regs->AD_DIO_INGRESS[i],sizeof(CSL_Aif2Ad_dio_ingressRegs)-108);
			memcpy((void *)&aif2RegsLog.AD_DIO_EGRESS[i] , (void *)&aif2Regs->AD_DIO_EGRESS[i] ,sizeof(CSL_Aif2Ad_dio_egressRegs) -108);
		}
	}
	
	memcpy((void *)aif2RegsLog.AT_EVENTS         , (void *)aif2Regs->AT_EVENTS         , sizeof(aif2RegsLog.AT_EVENTS         ));
	memcpy((void *)aif2RegsLog.AT_AD_INGR_EVENTS , (void *)aif2Regs->AT_AD_INGR_EVENTS , sizeof(aif2RegsLog.AT_AD_INGR_EVENTS ));
	memcpy((void *)aif2RegsLog.AT_AD_EGR_EVENTS  , (void *)aif2Regs->AT_AD_EGR_EVENTS  , sizeof(aif2RegsLog.AT_AD_EGR_EVENTS  ));

	memcpy((void *)aif2RegsLog.PI_DATA           , (void *)aif2Regs->PI_DATA           , sizeof(aif2RegsLog.PI_DATA           ));
	memcpy((void *)aif2RegsLog.AT_TM_DELTA_EVENTS, (void *)aif2Regs->AT_TM_DELTA_EVENTS, sizeof(aif2RegsLog.AT_TM_DELTA_EVENTS));
	memcpy((void *)aif2RegsLog.AT_PE_EVENTS      , (void *)aif2Regs->AT_PE_EVENTS      , sizeof(aif2RegsLog.AT_PE_EVENTS      ));
	memcpy((void *)aif2RegsLog.AT_PE_EVENT2S     , (void *)aif2Regs->AT_PE_EVENT2S     , sizeof(aif2RegsLog.AT_PE_EVENT2S     ));

	uiMax_AT_TC = (aif2Regs->AT_RADT_SYMB_LUT_INDEX_TC&CSL_AIF2_AT_RADT_SYMB_LUT_INDEX_TC_LUTINDEX_TC_MASK)+1;
	memcpy((void *)	aif2RegsLog.AT_RADT_SYM_LUT_RAM, (void *)aif2Regs->AT_RADT_SYM_LUT_RAM, uiMax_AT_TC*4);

	memcpy((void *)aif2RegsLog.PD_FRM_TC    , (void *)aif2Regs->PD_FRM_TC    , sizeof(aif2RegsLog.PD_FRM_TC))    ;
	memcpy((void *)aif2RegsLog.PE_FRM_TC    , (void *)aif2Regs->PE_FRM_TC    , sizeof(aif2RegsLog.PE_FRM_TC))    ;

	uiMax_PE_TC=0;
	uiMax_PD_TC=0;
	for(i=0;i<6;i++)
	{
		if(uiMax_PE_TC<(aif2RegsLog.PE_FRM_TC[i]&CSL_AIF2_PE_FRM_TC_FRM_INDEX_TC_MASK))
			uiMax_PE_TC=(aif2RegsLog.PE_FRM_TC[i]&CSL_AIF2_PE_FRM_TC_FRM_INDEX_TC_MASK);
		if(uiMax_PD_TC<(aif2RegsLog.PD_FRM_TC[i]&CSL_AIF2_PD_FRM_TC_FRM_INDEX_TC_MASK))
			uiMax_PD_TC=(aif2RegsLog.PD_FRM_TC[i]&CSL_AIF2_PD_FRM_TC_FRM_INDEX_TC_MASK);
	}	
	memcpy((void *)aif2RegsLog.PD_FRM_MSG_TC, (void *)aif2Regs->PD_FRM_MSG_TC, (uiMax_PD_TC+1)*4);
	memcpy((void *)aif2RegsLog.PE_FRM_MSG_TC, (void *)aif2Regs->PE_FRM_MSG_TC, (uiMax_PE_TC+1)*4);
}

/*printf error status information captured by error/status interrupt*/
void print_error_status_interrupt()
{
	int i,j;
	Uint32 EN_INT_STS; 	//enabled interrupt status
	Uint32 IRS_index, frame_cnt, AT_RADT_VALUE_LSBS;

	if(0==err_sts_log_cnt)
		return;
		
	puts("----------------runtime error/status interrupt log--------------");
	for(i=0; i<err_sts_log_cnt; i++)
	{
		frame_cnt= err_sts_log[i].frame_cnt;
		AT_RADT_VALUE_LSBS= err_sts_log[i].AT_RADT_VALUE_LSBS;
		IRS_index= err_sts_log[i].IRS_index;
		printf("%d error/status interrupt %s happens at frame %u, slot/symbol %u, clock %u\n",
			i, irsStr[IRS_index], frame_cnt,
			(AT_RADT_VALUE_LSBS&CSL_AIF2_AT_RADT_VALUE_LSBS_RADTSYMBOL_COUNT_VALUE_MASK)
				>>CSL_AIF2_AT_RADT_VALUE_LSBS_RADTSYMBOL_COUNT_VALUE_SHIFT,
			(AT_RADT_VALUE_LSBS&CSL_AIF2_AT_RADT_VALUE_LSBS_RADTCLOCK_COUNT_VALUE_MASK)
				>>CSL_AIF2_AT_RADT_VALUE_LSBS_RADTCLOCK_COUNT_VALUE_SHIFT);

		EN_INT_STS= err_sts_log[i].IRS_value;
		for(j=0;j<32;j++)
		{
			if(EN_INT_STS&1)
			{
				printf("  %s\n", IRS_info[IRS_index][j]);
			}
			EN_INT_STS >>= 1;
		}
	}
}

/*print AIF status, 
copyAif2ErrStsRegs() function must be called before this function*/
void print_AIF_status()
{
	Int32 i;
#if 0
	if(aif2ErrStsRegs.EE_AIF2_RUN_STS
		&CSL_AIF2_EE_AIF2_RUN_STS_AIF2_GLOBAL_RUN_MASK)
		puts("AIF2 gloabl runs stably");
	else
		puts("AIF2 is NOT in stable state");

	if(aif2ErrStsRegs.EE_AIF2_RUN_STS
		&CSL_AIF2_EE_AIF2_RUN_STS_AIF2_PHY_RUN_MASK)
		puts("AIF2 PHY runs stably");
	else
		puts("AIF2 PHY is NOT in stable state");
#endif

	print_error_status_interrupt();
	
	puts("------------------status when test complete---------------------");
	print_AIF_DMA_status();

	print_AIF_AT_status();

	print_AIF_SerDes_status();

	for(i=0; i<NUM_AIF2_LINK; i++)	//for all links
	{
		if(!aifLinkCfg[i].linkEnable)
			continue;
		printf("----------------link %d status----------------\n", i);
		print_AIF_Link_SerDes_status(i);
		
		print_AIF_Link_AT_status(i);

		print_AIF_Link_MAC_status(i);

		print_AIF_Link_PD_status(i);

		print_AIF_Link_PE_status(i);

		print_AIF_Link_RT_status(i);
	}
}

//check the read/write pointer of a channel in IDB, flush if not empty 
void check_IDB_channel(Uint32 uiChannel)
{
	Uint32 read_ptr, write_ptr;

	if((aif2Regs->DB_IDB_CH_EMPTY[uiChannel/32]&(1<<(uiChannel&0x1F))))
		return; 	//return if IDB channel is empty

	//enter IDB debug mode
	aif2Regs->DB_IDB_CFG= 1;

	aif2Regs->DB_IDB_DEBUG_OFS= 
		(uiChannel<<CSL_AIF2_DB_IDB_DEBUG_OFS_RADDR_SHIFT)
		|(uiChannel<<CSL_AIF2_DB_IDB_DEBUG_OFS_WADDR_SHIFT);

	read_ptr= (aif2Regs->DB_IDB_DEBUG_OFS_DAT&CSL_AIF2_DB_IDB_DEBUG_OFS_DAT_ROFF_MASK)>>
		CSL_AIF2_DB_IDB_DEBUG_OFS_DAT_ROFF_SHIFT;
	write_ptr= (aif2Regs->DB_IDB_DEBUG_OFS_DAT&CSL_AIF2_DB_IDB_DEBUG_OFS_DAT_WOFF_MASK)>>
		CSL_AIF2_DB_IDB_DEBUG_OFS_DAT_WOFF_SHIFT;

	printf("channel %d IDB not empty, read pointer = %d (quad words), write pointer= %d (quad words)\n",
		uiChannel, read_ptr, write_ptr);

	//generate an EOP to Packet DMA to flush the IDB
	aif2Regs->DB_IDB_DEBUG_SBND= 
		(1<<CSL_AIF2_DB_IDB_DEBUG_SBND_EOP_SHIFT)
		|(0<<CSL_AIF2_DB_IDB_DEBUG_SBND_XCNT_SHIFT)
		|(uiChannel<<CSL_AIF2_DB_IDB_DEBUG_SBND_CH_ID_SHIFT)
		|(1<<CSL_AIF2_DB_IDB_DEBUG_SBND_FIFO_WR_EN_SHIFT);
	aif2Regs->DB_IDB_DEBUG_DB_WR= 1;

	TSC_delay_us(1);
	
	//exit IDB debug mode
	aif2Regs->DB_IDB_CFG= 0;
}

