/****************************************************************************\
 *           Copyright (C) 2012 Texas Instruments Incorporated.             *
 *                           All Rights Reserved                            *
 *                                                                          *
 * GENERAL DISCLAIMER                                                       *
 * ------------------                                                       *
 * All software and related documentation is provided "AS IS" and without   *
 * warranty or support of any kind and Texas Instruments expressly disclaims*
 * all other warranties, express or implied, including, but not limited to, *
 * the implied warranties of merchantability and fitness for a particular   *
 * purpose.  Under no circumstances shall Texas Instruments be liable for   *
 * any incidental, special or consequential damages that result from the    *
 * use or inability to use the software or related documentation, even if   *
 * Texas Instruments has been advised of the liability.                     *
 ****************************************************************************
This example shows AIF data transfer between two DSP on dual Nyquist EVM
The examples can also be run on C6670 or TCI6614 EVM, but only internal loopback 
test can be run on those boards because there is only one DSP on them.

The steps to run the example on two DSPs on dual Nyquist EVM:
1, make sure all switches on SW6 are all ON before you run this program. 
   SW6 connects timer output of DSP0 to AIF Frame Sync input of the two DSPs.
2, load the program to two DSPs
3, run the second DSP
4, run the first DSP

To run internal loopback on one DSP, just load the program to the first DSP
and run it.
 ****************************************************************************
This file include main() function and the top level control codes
 ****************************************************************************
 * Created by :                                                             *
 *            Brighton Feng                                                 *
 *            Texas Instruments                                             * 
 *            August 12, 2011                                               *
Updated on June 18, 2012 for TCI6614 by Brighton
***************************************************************************/
#include <stdio.h>
#include <csl_bootcfgAux.h>
#include <cslr_chip.h>
#include "KeyStone_common.h"
#include "KeyStone_DDR_init.h"
#include "aif_setup.h"
#include "aif_test_config.h"
#include "aif_intc.h"
#include "aif_Data.h"
#include "aif_debug.h"
#include "aif_PktDMA_init.h"

Uint32 uiDSPNum; 	//identify the first and the second DSP on the EVM

/*On dual Nyquist EVM DSP0 timer output pin 0 are connected to 
PHYSYNC and RADSYNC pins of both DSP.
pulse by the timer will trigger AIF on both DSPs*/
void trigger_AIF_by_Timer()
{
	Timer64_Config tmrCfg;

	gpBootCfgRegs->TOUTSEL = (DNUM<<1);

	tmrCfg.timer_num= DNUM;
	tmrCfg.timerMode= TIMER_ONE_SHOT_PULSE;
	tmrCfg.period= 1664000; 	/*10ms periodic pulses*/
	tmrCfg.pulseWidth= 3;
	Timer64_Init(&tmrCfg);
}

void main()
{
	int i;

	/*enable TSC, memory protection interrupts, EDC for internal RAM;
    clear cache; protect L1 as cache*/
	KeyStone_common_CPU_init();
	/*print device information. 
	Enable memory protection interrupts, EDC for MSMC RAM*/
	KeyStone_common_device_init();

	//enable exception handling
	KeyStone_Exception_cfg(TRUE);

	//DSP core speed: 122.88*236/29= 999.9889655MHz
	KeyStone_main_PLL_init(122.88, 236, 29);

	//DDR init 66.667*20/1= 1333
	KeyStone_DDR_init (66.667, 20, 1, NULL);

    CACHE_setL1PSize(CACHE_L1_32KCACHE);
    CACHE_setL1DSize(CACHE_L1_32KCACHE);
    CACHE_setL2Size(CACHE_0KCACHE);

#if 1 	/*for debug only. Without these code you need reset DSP before your rerun the program*/
	//disable/reset AIF through PSC
	KeyStone_disable_PSC_module(CSL_PSC_PD_AI, CSL_PSC_LPSC_AI);
	KeyStone_disable_PSC_Power_Domain(CSL_PSC_PD_AI);
#endif

	/*make other cores local memory cacheable and prefetchable*/
	for(i=16; i<24; i++)
		gpCGEM_regs->MAR[i]=1|(1<<CSL_CGEM_MAR0_PFX_SHIFT);

	/*make DDR cacheable and prefetchable*/
	for(i=128; i<256; i++)
		gpCGEM_regs->MAR[i]=1|(1<<CSL_CGEM_MAR0_PFX_SHIFT);

	/*make other space non-cacheable and non-prefetchable*/
	for(i=24; i<128; i++)
		gpCGEM_regs->MAR[i]=0;

	uiDSPNum = KeyStone_Get_DSP_Number();

	InitAntDataBuf();
	
	/*Calculate AIF Parameters according some pre-set paremeters
	this function should called before AIF AND PktDMA initialization,
	becasue they need the parameters calculated by this function.*/
	InitAifTestParameters();

	AIF_Queue_init();

	AIF_Interrupts_Init();

	Counter_Init();
	
	ConfigAif2();

	/*for test between two DSPs, DSP0 is the master, it triggers AIF and send 
	packets. DSP1 only receive packet, and retransmit back the packet to DSP0*/
	if(0==uiDSPNum)
	{
#ifdef AIF2_TRIGGER_MANUALLY
		//Trigger manually
		aif2Regs->AT_SW_SYNC =
			(CSL_AIF2_AT_SW_SYNC_PHY_SYNC_MASK)
			|(CSL_AIF2_AT_SW_SYNC_RAD_SYNC_MASK);
#else
		trigger_AIF_by_Timer();
#endif
	}
	else
		puts("waiting for trigger from the first DSP...");

	while(FALSE==bTerminating); 	//main idle waiting loop

	//disable exception handling
	TSR = TSR&(~(1<<CSL_CHIP_TSR_XEN_SHIFT));

	copyAif2Regs(); 	//for debug
		
	//reset AIF
	AIF_soft_reset();

	PrintTestResult(); 	//for debug
}

