/****************************************************************************\
 *           Copyright (C) 2011, 2012 Texas Instruments Incorporated.             *
 *                           All Rights Reserved                            *
 *                                                                          *
 * GENERAL DISCLAIMER                                                       *
 * ------------------                                                       *
 * All software and related documentation is provided "AS IS" and without   *
 * warranty or support of any kind and Texas Instruments expressly disclaims*
 * all other warranties, express or implied, including, but not limited to, *
 * the implied warranties of merchantability and fitness for a particular   *
 * purpose.  Under no circumstances shall Texas Instruments be liable for   *
 * any incidental, special or consequential damages that result from the    *
 * use or inability to use the software or related documentation, even if   *
 * Texas Instruments has been advised of the liability.                     *
 ****************************************************************************
This example shows data transfer over AIF2 between two DSP on dual Nyquist EVM
The examples can also be run on C6670 or TCI6614 EVM, but only internal loopback 
test can be run on those boards because there is only one DSP on them.
This file setups AIF registers according the configuration parameters from 
aif_Test_Config.c.
 ****************************************************************************
 * Created by :                                                             *
 *            Brighton Feng                                                 *
 *            Texas Instruments                                             * 
 *            July 16, 2011                                                 *
Updated on June 18, 2012 for TCI6614 by Brighton
\***************************************************************************/
#include <stdio.h>
#include <string.h>
#include <tistdtypes.h>
#include <csl_psc.h>
#include <csl_pscAux.h>
#include <csl_aif2HwControlAux.h>
#include "AIF_PktDMA_init.h"
#include "KeyStone_common.h"
#include "aif_setup.h"
#include "aif_intc.h"
#include "aif_test_config.h"
#include "aif_data.h"

/*Number of synchronized raido frames before data receive*/
#define FRAME_SYNC_THRESH 	2

/*do not use this for uplink RAC data format*/
//#define DB_BUF_LEN_256 	

CSL_Aif2Regs * aif2Regs= (CSL_Aif2Regs *)CSL_AIF2_CONTROL_REGS;

/* Global structures and variables  */
CSL_Aif2Obj Aif2Obj;// Aif2 CSL object
CSL_Aif2Handle hAif2;// Aif2 handle 
 
CSL_Aif2Context Aif2Context;//Aif2 context
CSL_Aif2Param  aif2Param;//AIF2 module specific parameters
CSL_Status status; // CSL status
   
#pragma DATA_SECTION(aif2Setup,".far:AifInit")//use DDR3 memory for initialization data structure
#pragma DATA_SECTION(linkSetup,".far:AifInit")//use DDR3 memory for initialization data structure
#pragma DATA_SECTION(globalSetup,".far:AifInit")//use DDR3 memory for initialization data structure
#pragma DATA_SECTION(commonSetup,".far:AifInit")//use DDR3 memory for initialization data structure
CSL_Aif2Setup               aif2Setup;//Aif2 HW setup
CSL_Aif2LinkSetup           linkSetup[NUM_AIF2_LINK];// Setup for links 
CSL_Aif2GlobalSetup         globalSetup;// global config for AIF2 
CSL_Aif2CommonSetup         commonSetup; // Setup for common params

#pragma DATA_SECTION(SdCommonSetup,".far:AifInit")//use DDR3 memory for initialization data structure
#pragma DATA_SECTION(PdCommonSetup,".far:AifInit")//use DDR3 memory for initialization data structure
#pragma DATA_SECTION(PeCommonSetup,".far:AifInit")//use DDR3 memory for initialization data structure
#pragma DATA_SECTION(IngrDbSetup,".far:AifInit")//use DDR3 memory for initialization data structure
#pragma DATA_SECTION(EgrDbSetup,".far:AifInit")//use DDR3 memory for initialization data structure
#pragma DATA_SECTION(AdCommonSetup,".far:AifInit")//use DDR3 memory for initialization data structure
#pragma DATA_SECTION(AdDioSetup,".far:AifInit")//use DDR3 memory for initialization data structure
#pragma DATA_SECTION(AtCommonSetup,".far:AifInit")//use DDR3 memory for initialization data structure
#pragma DATA_SECTION(AtEventSetup,".far:AifInit")//use DDR3 memory for initialization data structure
#pragma DATA_SECTION(PhyTimerTc,".far:AifInit")//use DDR3 memory for initialization data structure
#pragma DATA_SECTION(RadTimerTc,".far:AifInit")//use DDR3 memory for initialization data structure
#pragma DATA_SECTION(PhyTimerInit,".far:AifInit")//use DDR3 memory for initialization data structure
#pragma DATA_SECTION(RadTimerInit,".far:AifInit")//use DDR3 memory for initialization data structure
#pragma DATA_SECTION(UlRadTimerInit,".far:AifInit")//use DDR3 memory for initialization data structure
#pragma DATA_SECTION(DlRadTimerInit,".far:AifInit")//use DDR3 memory for initialization data structure
CSL_Aif2SdCommonSetup       SdCommonSetup;//SERDES common setup
CSL_Aif2PdCommonSetup       PdCommonSetup;//PD common setup
CSL_Aif2PeCommonSetup       PeCommonSetup;//PE common setup
CSL_Aif2IngrDbSetup         IngrDbSetup;// Ingress data buffer setup 
CSL_Aif2EgrDbSetup          EgrDbSetup;// Egress data buffer setup 
CSL_Aif2AdCommonSetup       AdCommonSetup;// Aif2 DMA common setup 
CSL_Aif2AdDioSetup          AdDioSetup;// Aif2 DIO common setup 
CSL_Aif2AtCommonSetup       AtCommonSetup; // Aif2 Timer common  setup 
CSL_Aif2AtEventSetup        AtEventSetup; // Aif2 Timer external and internal event  setup 
CSL_Aif2AtCountObj          PhyTimerTc;// AT Phy Terminal Count setup
CSL_Aif2AtCountObj          RadTimerTc;// AT Rad Terminal Count setup
CSL_Aif2AtCountObj          PhyTimerInit;// AT Phy Init value setup
CSL_Aif2AtCountObj          RadTimerInit;// AT Rad Init value setup
CSL_Aif2AtCountObj          UlRadTimerInit;// AT Rad Init value setup
CSL_Aif2AtCountObj          DlRadTimerInit;// AT Rad Init value setup
   
#pragma DATA_SECTION(ComLinkSetup,".far:AifInit")//use DDR3 memory for initialization data structure
#pragma DATA_SECTION(SdLinkSetup,".far:AifInit")//use DDR3 memory for initialization data structure
#pragma DATA_SECTION(RmLinkSetup,".far:AifInit")//use DDR3 memory for initialization data structure
#pragma DATA_SECTION(TmLinkSetup,".far:AifInit")//use DDR3 memory for initialization data structure
#pragma DATA_SECTION(PdLinkSetup,".far:AifInit")//use DDR3 memory for initialization data structure
#pragma DATA_SECTION(PeLinkSetup,".far:AifInit")//use DDR3 memory for initialization data structure
#pragma DATA_SECTION(RtLinkSetup,".far:AifInit")//use DDR3 memory for initialization data structure
#pragma DATA_SECTION(AtLinkSetup,".far:AifInit")//use DDR3 memory for initialization data structure
CSL_Aif2CommonLinkSetup     ComLinkSetup[NUM_AIF2_LINK]; // Aif2 link common setup 
CSL_Aif2SdLinkSetup         SdLinkSetup[NUM_AIF2_LINK]; //SERDES link setup
CSL_Aif2RmLinkSetup         RmLinkSetup[NUM_AIF2_LINK]; //RM link setup
CSL_Aif2TmLinkSetup         TmLinkSetup[NUM_AIF2_LINK]; //TM link setup
CSL_Aif2PdLinkSetup         PdLinkSetup[NUM_AIF2_LINK]; //PD link setup
CSL_Aif2PeLinkSetup         PeLinkSetup[NUM_AIF2_LINK]; //PE link setup
CSL_Aif2RtLinkSetup         RtLinkSetup[NUM_AIF2_LINK]; //RT link setup
CSL_Aif2AtLinkSetup         AtLinkSetup[NUM_AIF2_LINK]; // Aif2 timer link setup (Pi, Delta, PE signal) 

int uiDioStreamCnt[3]={0, 0, 0}; 	//stream number count for each DIO engine
//Configure a specific link
void Aif2LinkConfig(Uint32 linkIndex)
{
	int i;
	Uint32 X; 	//X for Dual Bit Map configuration
	int Ns, Xs; 	//CPRI control words index
	Uint32 uiCtlChNum= aifLinkCfg[linkIndex].controlChNum;
	PeChannelLut * peChLut = (PeChannelLut * )&PeCommonSetup.ChIndex0[0];
	peChLut += linkIndex; 	//point to the data structure for this link
	
	/*---------------Link Common setup---------------*/
#ifdef AIF2_LINK_PROTOCOL_OBSAI
	ComLinkSetup[linkIndex].linkProtocol = CSL_AIF2_LINK_PROTOCOL_OBSAI;
#else
	ComLinkSetup[linkIndex].linkProtocol = CSL_AIF2_LINK_PROTOCOL_CPRI;
#endif
	ComLinkSetup[linkIndex].IngrDataWidth = aifLinkCfg[linkIndex].dataWidth;
	ComLinkSetup[linkIndex].EgrDataWidth = aifLinkCfg[linkIndex].dataWidth;
	ComLinkSetup[linkIndex].linkRate = aifLinkCfg[linkIndex].linkRate;

	/*---------------SD link setup---------------*/
	SdLinkSetup[linkIndex].rxAlign = CSL_AIF2_SD_RX_COMMA_ALIGNMENT_ENABLE;
	SdLinkSetup[linkIndex].rxLos = CSL_AIF2_SD_RX_LOS_ENABLE;
	SdLinkSetup[linkIndex].rxCdrAlgorithm = CSL_AIF2_SD_RX_CDR_FIRST_ORDER_THRESH_17;
	SdLinkSetup[linkIndex].rxInvertPolarity = CSL_AIF2_SD_RX_NORMAL_POLARITY;
	SdLinkSetup[linkIndex].rxTermination = CSL_AIF2_SD_RX_TERM_COMMON_POINT_0_7 ;//for AC coupled application
	SdLinkSetup[linkIndex].rxEqualizerConfig = CSL_AIF2_SD_RX_EQ_ADAPTIVE;//Equalizer On
	SdLinkSetup[linkIndex].bRxEqHold = FALSE;//fixed value
	SdLinkSetup[linkIndex].bRxOffsetComp = TRUE;//fixed value
	SdLinkSetup[linkIndex].bEnableTxSyncMater = aifLinkCfg[linkIndex].bEnableTxSyncMater;
	SdLinkSetup[linkIndex].txInvertPolarity = CSL_AIF2_SD_TX_PAIR_NORMAL_POLARITY;
	SdLinkSetup[linkIndex].txOutputSwing = CSL_AIF2_SD_TX_OUTPUT_SWING_15;
	SdLinkSetup[linkIndex].txPrecursorTapWeight = CSL_AIF2_SD_TX_PRE_TAP_WEIGHT_1;// -2.5%
	SdLinkSetup[linkIndex].txPostcursorTapWeight = CSL_AIF2_SD_TX_POST_TAP_WEIGHT_20;// -10%
	SdLinkSetup[linkIndex].bTxFirFilterUpdate = TRUE;//FIR filter update on
	SdCommonSetup.DisableLinkClock[linkIndex] = FALSE;

	/*---------------TM/RM link setup---------------*/
	TmLinkSetup[linkIndex].bEnableTmLink = TRUE;
	TmLinkSetup[linkIndex].bEnableRmLos = FALSE;

	//CPRI and OBSAI 2x, 4x doesn't use scrambler
	TmLinkSetup[linkIndex].bEnableScrambler = FALSE;
	RmLinkSetup[linkIndex].bEnableScrambler = FALSE;
#ifdef AIF2_LINK_PROTOCOL_OBSAI
	if(CSL_AIF2_LINK_RATE_8x==aifLinkCfg[linkIndex].linkRate)
	{
		/*use different scrambing seed for different link*/
		TmLinkSetup[linkIndex].SeedValue = linkIndex; 
		TmLinkSetup[linkIndex].bEnableScrambler = TRUE;
		RmLinkSetup[linkIndex].bEnableScrambler = TRUE;
	}
#else
	TmLinkSetup[linkIndex].pCpriTmSetup.L1InbandEn = 0;//disable 9 bits mask

	//select a links as error source
	TmLinkSetup[linkIndex].pCpriTmSetup.RmLinkLosError = linkIndex;
	TmLinkSetup[linkIndex].pCpriTmSetup.RmLinkLofError = linkIndex;
	TmLinkSetup[linkIndex].pCpriTmSetup.RmLinkLosRx = linkIndex;
	TmLinkSetup[linkIndex].pCpriTmSetup.RmLinkLofRx = linkIndex;
	TmLinkSetup[linkIndex].pCpriTmSetup.RmLinkRaiRx = linkIndex;
	TmLinkSetup[linkIndex].pCpriTmSetup.TxStartup = 0;
	TmLinkSetup[linkIndex].pCpriTmSetup.TxPointerP = aifLinkCfg[linkIndex].firstCtlWordP;
	TmLinkSetup[linkIndex].pCpriTmSetup.TxProtocolVer = 1;
#endif

	RmLinkSetup[linkIndex].bEnableRmLink = TRUE;
	RmLinkSetup[linkIndex].RmFifoThold = CSL_AIF2_RM_FIFO_THOLD_IMMEDIATELY;
	RmLinkSetup[linkIndex].RmErrorSuppress = CSL_AIF2_RM_ERROR_ALLOW;
	RmLinkSetup[linkIndex].bEnableSdAutoAlign = FALSE;
	RmLinkSetup[linkIndex].bEnableLcvUnsync = FALSE;
	RmLinkSetup[linkIndex].bEnableLcvControl = TRUE;
	RmLinkSetup[linkIndex].bEnableWatchDog = TRUE;
	RmLinkSetup[linkIndex].WatchDogWrap = 0xFF;//set watch dog wrap value
	RmLinkSetup[linkIndex].bEnableClockQuality = TRUE;
	RmLinkSetup[linkIndex].ClockMonitorWrap = 8000;
	RmLinkSetup[linkIndex].losDetThreshold = 2048;
#ifdef AIF2_LINK_PROTOCOL_OBSAI
	RmLinkSetup[linkIndex].SyncThreshold = 1920; /*number of consective good data blocks (400 bytes) for entering ST1*/
	RmLinkSetup[linkIndex].FrameSyncThreshold = 1920; /*number of consective good message groups for entering ST3*/
	RmLinkSetup[linkIndex].UnsyncThreshold = 1920; /*number of consective bad data blocks (400 bytes) for exiting ST1*/
	RmLinkSetup[linkIndex].FrameUnsyncThreshold = 1920; /*number of consective bad message groups for exiting ST3*/
#else
	RmLinkSetup[linkIndex].SyncThreshold = 150; /*number of consective good Hyperframes for entering ST2*/
	RmLinkSetup[linkIndex].FrameSyncThreshold = 150; /*number of consective good Hyperframes for entering ST3*/
	RmLinkSetup[linkIndex].UnsyncThreshold = 150*3; /*number of consective bad Hyperframes for exiting ST2*/
	RmLinkSetup[linkIndex].FrameUnsyncThreshold = 150*3; /*number of consective bad Hyperframes for exiting ST3*/
#endif

	/*---------------RT link setup---------------*/
	RtLinkSetup[linkIndex].CiSelect =  linkIndex;
	RtLinkSetup[linkIndex].bEnableEmptyMsg = TRUE;
	RtLinkSetup[linkIndex].RtConfig = aifLinkCfg[linkIndex].RT_cfg;

	/*---------------PE/PD link setup---------------*/
	X = aifLinkCfg[linkIndex].DBM_X;

	/*each link only support up to 64 AxC channels*/
	if(aifLinkCfg[linkIndex].numberAxC>64)
		aifLinkCfg[linkIndex].numberAxC=64;
	
	PeLinkSetup[linkIndex].bEnablePeLink = TRUE;
	PdLinkSetup[linkIndex].bEnablePdLink = TRUE;

	//AxC data uses DIO as DMA machine
	PeLinkSetup[linkIndex].PeCppiDioSel = CSL_AIF2_DIO;
	PeLinkSetup[linkIndex].PeDelay = DB_PE2_DELAY;

#ifdef AIF2_LINK_PROTOCOL_OBSAI
	/*In this test, each OBSAI link use a rule like CPRI*/
	PeCommonSetup.PeModuloTc[linkIndex*8].bEnableRule = TRUE;
	PeCommonSetup.PeModuloTc[linkIndex*8].RuleModulo = 0;
	PeCommonSetup.PeModuloTc[linkIndex*8].bRuleObsaiCtlMsg = FALSE;
	PeCommonSetup.PeModuloTc[linkIndex*8].RuleIndex = 0;
	PeCommonSetup.PeModuloTc[linkIndex*8].RuleLink = linkIndex;

	/*Control slots*/
	PeCommonSetup.PeModuloTc[linkIndex*8+1].bEnableRule = TRUE;
	PeCommonSetup.PeModuloTc[linkIndex*8+1].RuleModulo = 0;
	PeCommonSetup.PeModuloTc[linkIndex*8+1].bRuleObsaiCtlMsg = TRUE;
	PeCommonSetup.PeModuloTc[linkIndex*8+1].RuleIndex = 0;
	PeCommonSetup.PeModuloTc[linkIndex*8+1].RuleLink = linkIndex;

	PeCommonSetup.PeObsaiDualBitMap[linkIndex*8].DbmX = X-1;//set X-1
	PeCommonSetup.PeObsaiDualBitMap[linkIndex*8].DbmXBubble = 0;
	PeCommonSetup.PeObsaiDualBitMap[linkIndex*8].Dbm1Mult = 0;//set n-1
	PeCommonSetup.PeObsaiDualBitMap[linkIndex*8].Dbm1Size = 0;//set n-1
	PeCommonSetup.PeObsaiDualBitMap[linkIndex*8].Dbm1Map[0] = 0x0;
	PeLinkSetup[linkIndex].bEnObsaiBubbleBW = FALSE; 	/*no bubble*/

	/*Control slots*/
	PeCommonSetup.PeObsaiDualBitMap[linkIndex*8+1].DbmX = 0;//set X-1
	PeCommonSetup.PeObsaiDualBitMap[linkIndex*8+1].DbmXBubble = 0;
	PeCommonSetup.PeObsaiDualBitMap[linkIndex*8+1].Dbm1Mult = 0;//set n-1
	PeCommonSetup.PeObsaiDualBitMap[linkIndex*8+1].Dbm1Size = 0;//set n-1
	PeCommonSetup.PeObsaiDualBitMap[linkIndex*8+1].Dbm1Map[0] = 0x0;
	PeLinkSetup[linkIndex].bEnObsaiBubbleBW = FALSE; 	/*no bubble*/

	for(i=0; i<aifLinkCfg[linkIndex].numberAxC; i++)
	{
		Uint32 uiChIndex;
		peChLut->bEnableChIndex[i]= TRUE;

		uiChIndex = aifLinkCfg[linkIndex].firstAxcNum+i;
		peChLut->ChIndex[i]= uiChIndex;
		PeCommonSetup.PeChObsaiType[uiChIndex] = OBSAI_MESSAGE_TYPE;
		PeCommonSetup.PeChObsaiAddr[uiChIndex] = uiChIndex;
		PeCommonSetup.PeChObsaiTS[uiChIndex] = 0; /*TS auto generated*/
		PeCommonSetup.PeChObsaiTsMask[uiChIndex] = CSL_AIF2_ROUTE_MASK_NONE;
		PeCommonSetup.PeChObsaiTsfomat[uiChIndex] = CSL_AIF2_TSTAMP_FORMAT_NORM_TS;
		PeCommonSetup.PeObsaiPkt[uiChIndex] = FALSE;
		PeCommonSetup.PeBbHop[uiChIndex] = FALSE;

		PdCommonSetup.PdRoute[uiChIndex].RouteType = OBSAI_MESSAGE_TYPE;
		PdCommonSetup.PdRoute[uiChIndex].RouteAddr = uiChIndex;
		PdCommonSetup.PdRoute[uiChIndex].RouteLink = linkIndex;
		PdCommonSetup.PdRoute[uiChIndex].RouteTs = 0;/*TS auto generated*/
		PdCommonSetup.PdRoute[uiChIndex].RouteMask = CSL_AIF2_ROUTE_MASK_NONE;
	}

	/*control channel*/
	if(aifLinkCfg[linkIndex].bUseControlMsg)
	{
		peChLut->bEnableChIndex[64]= TRUE;
		peChLut->ChIndex[64]= uiCtlChNum;

		PeCommonSetup.PeChObsaiType[uiCtlChNum] = OBSAI_TYPE_GENERIC;
		PeCommonSetup.PeChObsaiAddr[uiCtlChNum] = uiCtlChNum;
		PeCommonSetup.PeChObsaiTS[uiCtlChNum] = 0; /*TS auto generated*/
		PeCommonSetup.PeChObsaiTsMask[uiCtlChNum] = CSL_AIF2_ROUTE_MASK_NONE;
		PeCommonSetup.PeChObsaiTsfomat[uiCtlChNum] = CSL_AIF2_TSTAMP_FORMAT_GEN_PKT;
		PeCommonSetup.PeObsaiPkt[uiCtlChNum] = TRUE;
		PeCommonSetup.PeBbHop[uiCtlChNum] = FALSE;

		PdCommonSetup.PdRoute[uiCtlChNum].RouteType = OBSAI_TYPE_GENERIC;
		PdCommonSetup.PdRoute[uiCtlChNum].RouteAddr = uiCtlChNum;
		PdCommonSetup.PdRoute[uiCtlChNum].RouteLink = linkIndex;
		PdCommonSetup.PdRoute[uiCtlChNum].RouteTs = 0;/*TS auto generated*/
		PdCommonSetup.PdRoute[uiCtlChNum].RouteMask = CSL_AIF2_ROUTE_MASK_NONE;
	}

	PdLinkSetup[linkIndex].PdTypeLut[OBSAI_MESSAGE_TYPE].ObsaiTsFormat = CSL_AIF2_TSTAMP_FORMAT_NORM_TS;
	PdLinkSetup[linkIndex].PdTypeLut[OBSAI_MESSAGE_TYPE].PdCrcType = CSL_AIF2_CRC_16BIT;
	PdLinkSetup[linkIndex].PdTypeLut[OBSAI_MESSAGE_TYPE].bEnableCrc = FALSE;
	PdLinkSetup[linkIndex].PdTypeLut[OBSAI_MESSAGE_TYPE].PdObsaiMode = CSL_AIF2_PD_DATA_AXC;
	PdLinkSetup[linkIndex].PdTypeLut[OBSAI_MESSAGE_TYPE].bEnableEnetStrip = FALSE;
	PdLinkSetup[linkIndex].PdTypeLut[OBSAI_MESSAGE_TYPE].bEnableCrcHeader = FALSE;

	PdLinkSetup[linkIndex].PdTypeLut[OBSAI_TYPE_GENERIC].ObsaiTsFormat = CSL_AIF2_TSTAMP_FORMAT_GEN_PKT;
	PdLinkSetup[linkIndex].PdTypeLut[OBSAI_TYPE_GENERIC].PdCrcType = CSL_AIF2_CRC_16BIT;
	PdLinkSetup[linkIndex].PdTypeLut[OBSAI_TYPE_GENERIC].bEnableCrc = FALSE;
	PdLinkSetup[linkIndex].PdTypeLut[OBSAI_TYPE_GENERIC].PdObsaiMode = CSL_AIF2_PD_DATA_PKT;
	PdLinkSetup[linkIndex].PdTypeLut[OBSAI_TYPE_GENERIC].bEnableEnetStrip = FALSE;
	PdLinkSetup[linkIndex].PdTypeLut[OBSAI_TYPE_GENERIC].bEnableCrcHeader = FALSE;
#else 	/*CPRI*/
	PeLinkSetup[linkIndex].PeCpriDualBitMap.DbmX = X-1;//set X-1
	PeLinkSetup[linkIndex].PeCpriDualBitMap.DbmXBubble = 0;
	PeLinkSetup[linkIndex].PeCpriDualBitMap.Dbm1Mult = 0;//set n-1
	PeLinkSetup[linkIndex].PeCpriDualBitMap.Dbm1Size = 0;//set n-1
	PeLinkSetup[linkIndex].PeCpriDualBitMap.Dbm1Map[0] = 0x0;
	PeLinkSetup[linkIndex].PeCpriDualBitMap.Dbm2Size = 0;
	PeLinkSetup[linkIndex].PeCpriDualBitMap.Dbm2Map[0] = 0x0;

	PdLinkSetup[linkIndex].PdCpriDualBitMap.DbmX = X-1;// set X-1
	PdLinkSetup[linkIndex].PdCpriDualBitMap.DbmXBubble = 0;
	PdLinkSetup[linkIndex].PdCpriDualBitMap.Dbm1Mult = 0;//set n-1
	PdLinkSetup[linkIndex].PdCpriDualBitMap.Dbm1Size = 0;//set n-1
	PdLinkSetup[linkIndex].PdCpriDualBitMap.Dbm1Map[0] = 0x0;
	PdLinkSetup[linkIndex].PdCpriDualBitMap.Dbm2Size = 0;
	PdLinkSetup[linkIndex].PdCpriDualBitMap.Dbm2Map[0] = 0x0;

	PeLinkSetup[linkIndex].CpriAxCPack= aifLinkCfg[linkIndex].dataWidth;
	PdLinkSetup[linkIndex].CpriAxCPack= aifLinkCfg[linkIndex].dataWidth;

	for(i=0; i<aifLinkCfg[linkIndex].numberAxC; i++)
	{
		peChLut->bEnableChIndex[i]= TRUE;
		PdLinkSetup[linkIndex].bEnableCpriX[i]= TRUE; 

		peChLut->ChIndex[i]= aifLinkCfg[linkIndex].firstAxcNum+i;
		PdLinkSetup[linkIndex].CpriDmaCh[i]= 
			aifLinkCfg[linkIndex].firstAxcNum+i;

		PdLinkSetup[linkIndex].Cpri8WordOffset[i]= 0;
	}

	/*map selected control words to a channel*/
	for(Xs=0; Xs<4; Xs++) //cpri cw lut setup
	for(Ns=aifLinkCfg[linkIndex].firstCtlWordP;
		Ns<aifLinkCfg[linkIndex].firstCtlWordP+aifLinkCfg[linkIndex].numCtlWord;Ns++)
	{
		//enable CPRI Control words
		PeLinkSetup[linkIndex].bEnableCpriCw[Xs*64+Ns]= TRUE;

		//use control channel 0 of this link
		PeLinkSetup[linkIndex].CpriCwChannel[Xs*64+Ns]= 0; 

		//enable CPRI Control words
		PdLinkSetup[linkIndex].bEnableCpriCw[Xs*64+Ns]= TRUE; 

		//use control channel 0 of this link
		PdLinkSetup[linkIndex].CpriCwChannel[Xs*64+Ns]= 0;
	}
	
#if 0
	PeLinkSetup[linkIndex].CpriCwPktDelimitor[0] = CSL_AIF2_CW_DELIM_4B5B;
	PdLinkSetup[linkIndex].CpriCwPktDelimitor[0] = CSL_AIF2_CW_DELIM_4B5B;//4
#else
	PeLinkSetup[linkIndex].CpriCwPktDelimitor[0] = CSL_AIF2_CW_DELIM_NULLDELM;
	PdLinkSetup[linkIndex].CpriCwPktDelimitor[0] = CSL_AIF2_CW_DELIM_NULLDELM;//4
	PeLinkSetup[linkIndex].CpriCwNullDelimitor = 0x1FB;//K 27.7 charactor
	PdLinkSetup[linkIndex].CpriCwNullDelimitor = 0x1FB;//K 27.7 charactor
#endif

	/*control channel*/
	if(aifLinkCfg[linkIndex].numCtlWord)
	{
		/*map control channels to DMA buffer channel*/
		PeLinkSetup[linkIndex].PePackDmaCh[0] = uiCtlChNum;
		PeLinkSetup[linkIndex].bEnablePack[0] = TRUE;

		PdLinkSetup[linkIndex].PdCpriCrcType[0] = CSL_AIF2_CRC_16BIT;
		PdLinkSetup[linkIndex].bEnableCpriCrc[0] = FALSE;
		PdLinkSetup[linkIndex].PdPackDmaCh[0] = uiCtlChNum;
		PdLinkSetup[linkIndex].bEnablePack[0] = TRUE;
	}
	
#endif

	/*---------------PE/PD DB setup---------------*/
#ifdef AIF2_LINK_PROTOCOL_OBSAI
	if(aifLinkCfg[linkIndex].bUseControlMsg)
#else
	if(aifLinkCfg[linkIndex].numCtlWord)
#endif
	{
		/*PE control channel*/
		PeCommonSetup.bEnableCh[uiCtlChNum] = TRUE;
		PeCommonSetup.PeDmaCh0[uiCtlChNum].bCrcEn = FALSE;
		PeCommonSetup.PeDmaCh0[uiCtlChNum].CrcType = CSL_AIF2_CRC_16BIT;
		PeCommonSetup.PeDmaCh0[uiCtlChNum].FrameTC = 0;
		PdCommonSetup.PdChConfig1[uiCtlChNum].FrameCounter = 0;
		PdCommonSetup.PdChConfig1[uiCtlChNum].DioOffset = 0;//Use zero offset for simple test
		PeCommonSetup.PeDmaCh0[uiCtlChNum].RtControl = aifLinkCfg[linkIndex].PE_RT_ctl;
		PeCommonSetup.PeDmaCh0[uiCtlChNum].isEthernet = FALSE;
		PeCommonSetup.PeDmaCh0[uiCtlChNum].CrcObsaiHeader = FALSE;
		PeCommonSetup.PeInFifo[uiCtlChNum].MFifoWmark = 2;
		PeCommonSetup.PeInFifo[uiCtlChNum].MFifoFullLevel = 3;
		PeCommonSetup.PeInFifo[uiCtlChNum].SyncSymbol = 0;
		PeCommonSetup.PeAxcOffset[uiCtlChNum] = aifLinkCfg[linkIndex].PeEvent2Offset;

		/*Will enable PD channel after number of FRAME_SYNC_THRESH
		consecutive radio frames*/ 
		PdCommonSetup.PdChConfig[uiCtlChNum].bChannelEn = FALSE;
		PdCommonSetup.PdChConfig[uiCtlChNum].DataFormat = 
			CSL_AIF2_LINK_DATA_TYPE_NORMAL;

#ifdef AIF2_LINK_PROTOCOL_OBSAI
		/*OBSAI: Antenna Carrier offset programmed in 307.2MHz clocks and 
		relative to the RadT frame boundary*/
		PdCommonSetup.AxCOffset[uiCtlChNum] = aifLinkCfg[linkIndex].piOffset;
#else
		/*CPRI: Antenna Carrier Offset programmed in the number of four 
		samples (QW), relative to the CPRI link frame boundary.*/
		PdCommonSetup.AxCOffset[uiCtlChNum] = 0;
#endif

		PdCommonSetup.PdChConfig1[uiCtlChNum].DataFormat = 
			CSL_AIF2_GSM_DATA_OTHER;//Non GSM data

		//Egress DB setup for control channel
		EgrDbSetup.bEnableChannel[uiCtlChNum] = TRUE; 
		EgrDbSetup.EgrDbChannel[uiCtlChNum].ChannelNum= uiCtlChNum;
#if 1
		EgrDbSetup.EgrDbChannel[uiCtlChNum].BaseAddress = 64+uiCtlChNum*2;
		EgrDbSetup.EgrDbChannel[uiCtlChNum].BufDepth = CSL_AIF2_DB_FIFO_DEPTH_QW16;
#else
		EgrDbSetup.EgrDbChannel[uiCtlChNum].BaseAddress = uiCtlChNum;
		EgrDbSetup.EgrDbChannel[uiCtlChNum].BufDepth = CSL_AIF2_DB_FIFO_DEPTH_QW8;
#endif
		EgrDbSetup.EgrDbChannel[uiCtlChNum].DataSwap = CSL_AIF2_DB_WORD_SWAP;
		EgrDbSetup.EgrDbChannel[uiCtlChNum].IQOrder = CSL_AIF2_DB_IQ_NO_SWAP;
		EgrDbSetup.EgrDbChannel[uiCtlChNum].EgressDioOffset = 0;//set zero for simple test

		//Ingress DB setup for control channel
		IngrDbSetup.bEnableChannel[uiCtlChNum] = TRUE;
		IngrDbSetup.IngrDbChannel[uiCtlChNum].ChannelNum= uiCtlChNum;
#if 1
		IngrDbSetup.IngrDbChannel[uiCtlChNum].BaseAddress = 64+uiCtlChNum*2;
		IngrDbSetup.IngrDbChannel[uiCtlChNum].BufDepth = CSL_AIF2_DB_FIFO_DEPTH_QW16;
#else
		IngrDbSetup.IngrDbChannel[uiCtlChNum].BaseAddress = uiCtlChNum;
		IngrDbSetup.IngrDbChannel[uiCtlChNum].BufDepth = CSL_AIF2_DB_FIFO_DEPTH_QW8;
#endif
		IngrDbSetup.IngrDbChannel[uiCtlChNum].DataSwap = CSL_AIF2_DB_WORD_SWAP;
		IngrDbSetup.IngrDbChannel[uiCtlChNum].IQOrder = CSL_AIF2_DB_IQ_NO_SWAP;
	
		/*4 bytes CPPI Protocol Specific data*/
		IngrDbSetup.IngrDbChannel[uiCtlChNum].bEnablePsData = FALSE; 
		IngrDbSetup.IngrDbChannel[uiCtlChNum].PacketType = 1; //User data
	}

	for(i=aifLinkCfg[linkIndex].firstAxcNum; 
		i<aifLinkCfg[linkIndex].firstAxcNum+
		aifLinkCfg[linkIndex].numberAxC; i++)
	{
		//PE AXC channels
		PeCommonSetup.bEnableCh[i] = TRUE;
		PeCommonSetup.PeDmaCh0[i].bCrcEn = FALSE;
		PeCommonSetup.PeDmaCh0[i].CrcType = CSL_AIF2_CRC_16BIT;
		PeCommonSetup.PeDmaCh0[i].FrameTC = 0;
		PdCommonSetup.PdChConfig1[i].FrameCounter = 0;
		PdCommonSetup.PdChConfig1[i].DioOffset = 0;//Use zero offset for simple test
		PeCommonSetup.PeDmaCh0[i].RtControl = aifLinkCfg[linkIndex].PE_RT_ctl;
		PeCommonSetup.PeDmaCh0[i].isEthernet = FALSE;
		PeCommonSetup.PeDmaCh0[i].CrcObsaiHeader = FALSE;
		PeCommonSetup.PeInFifo[i].MFifoWmark = 2;
		PeCommonSetup.PeInFifo[i].MFifoFullLevel = 3;
		PeCommonSetup.PeInFifo[i].SyncSymbol = 0;
		PeCommonSetup.PeAxcOffset[i] = aifLinkCfg[linkIndex].PeEvent2Offset;

		/*Will enable PD AXC channel after number of FRAME_SYNC_THRESH
		consecutive radio frames*/ 
		PdCommonSetup.PdChConfig[i].bChannelEn = FALSE;
		PdCommonSetup.PdChConfig[i].DataFormat = aifLinkCfg[linkIndex].dataFormat;

#ifdef AIF2_LINK_PROTOCOL_OBSAI
		/*OBSAI: Antenna Carrier offset programmed in 307.2MHz clocks and 
		relative to the RadT frame boundary*/
		PdCommonSetup.AxCOffset[i] = aifLinkCfg[linkIndex].piOffset;
#else
		/*CPRI: Antenna Carrier Offset programmed in the number of four 
		samples (QW), relative to the CPRI link frame boundary.*/
		PdCommonSetup.AxCOffset[i] = 0;
#endif

		PdCommonSetup.PdChConfig1[i].DataFormat = 
			CSL_AIF2_GSM_DATA_OTHER;//Non GSM data

		//enable all slot by default
		PdCommonSetup.PdChConfig1[i].TddEnable = 0xFFFF;
		PdCommonSetup.TddEnable1[i] = 0xFFFFFFFF;
		PdCommonSetup.TddEnable2[i] = 0xFFFFFFFF;
		PdCommonSetup.TddEnable3[i] = 0xFFFFFFFF;
		PdCommonSetup.TddEnable4[i] = 0xFFFFFFFF;

		//Egress DB setup for AxC
		EgrDbSetup.bEnableChannel[i] = TRUE; 
#ifdef DB_BUF_LEN_256
		EgrDbSetup.EgrDbChannel[i].BaseAddress = i*2;
		EgrDbSetup.EgrDbChannel[i].BufDepth = CSL_AIF2_DB_FIFO_DEPTH_QW16;
#else
		EgrDbSetup.EgrDbChannel[i].BaseAddress = i;
		EgrDbSetup.EgrDbChannel[i].BufDepth = CSL_AIF2_DB_FIFO_DEPTH_QW8;
#endif
		EgrDbSetup.EgrDbChannel[i].DataSwap = aifLinkCfg[linkIndex].AxCSwap;
		EgrDbSetup.EgrDbChannel[i].IQOrder = CSL_AIF2_DB_IQ_NO_SWAP;
		EgrDbSetup.EgrDbChannel[i].EgressDioOffset = 0;//set zero for simple test

		//Ingress DB setup for AxC
		IngrDbSetup.bEnableChannel[i] = TRUE; 
#ifdef DB_BUF_LEN_256
		IngrDbSetup.IngrDbChannel[i].BaseAddress = i*2;
		IngrDbSetup.IngrDbChannel[i].BufDepth = CSL_AIF2_DB_FIFO_DEPTH_QW16;
#else
		IngrDbSetup.IngrDbChannel[i].BaseAddress = i;
		IngrDbSetup.IngrDbChannel[i].BufDepth = CSL_AIF2_DB_FIFO_DEPTH_QW8;
#endif
		IngrDbSetup.IngrDbChannel[i].DataSwap = aifLinkCfg[linkIndex].AxCSwap;
		IngrDbSetup.IngrDbChannel[i].IQOrder = CSL_AIF2_DB_IQ_NO_SWAP;

	}

	/*---------------AT link setup---------------*/
	AtLinkSetup[linkIndex].PE1Offset = aifLinkCfg[linkIndex].PeEvent1Offset;
	AtLinkSetup[linkIndex].PE2Offset = aifLinkCfg[linkIndex].PeEvent2Offset;
	AtLinkSetup[linkIndex].DeltaOffset = aifLinkCfg[linkIndex].deltaOffset;
	AtLinkSetup[linkIndex].IsNegativeDelta = FALSE;//positive delta
	if((aifLinkCfg[linkIndex].piOffset-PI_WINDOW/2)>0)
		AtLinkSetup[linkIndex].PiMin = aifLinkCfg[linkIndex].piOffset-PI_WINDOW/2;
	else
		AtLinkSetup[linkIndex].PiMin = FRAME_CLOCK_COUNT +
			aifLinkCfg[linkIndex].piOffset-PI_WINDOW/2;
	AtLinkSetup[linkIndex].PiMax = aifLinkCfg[linkIndex].piOffset+PI_WINDOW/2;

	/************** populating AIF2 link setup structures *********************/
	linkSetup[linkIndex].linkIndex     = linkIndex;
	linkSetup[linkIndex].pComLinkSetup = &ComLinkSetup[linkIndex];
	linkSetup[linkIndex].pSdLinkSetup  = &SdLinkSetup[linkIndex];
	linkSetup[linkIndex].pRmLinkSetup  = &RmLinkSetup[linkIndex];
	linkSetup[linkIndex].pTmLinkSetup  = &TmLinkSetup[linkIndex];
	linkSetup[linkIndex].pPdLinkSetup  = &PdLinkSetup[linkIndex];
	linkSetup[linkIndex].pPeLinkSetup  = &PeLinkSetup[linkIndex];
	linkSetup[linkIndex].pRtLinkSetup  = &RtLinkSetup[linkIndex];
	linkSetup[linkIndex].pAtLinkSetup  = &AtLinkSetup[linkIndex];

	aif2Setup.linkSetup[linkIndex] = &linkSetup[linkIndex];
	globalSetup.ActiveLink[linkIndex] = TRUE;
}

/*AIF configuration common to all links*/
void Aif2CommonConfig()
{
	CSL_Aif2PllMpyFactor pllMpy;

	/*---------------SD common setup---------------*/
#ifdef AIF2_LINK_PROTOCOL_OBSAI
	if(DUAL_NYQUIST_EVM==gDSP_board_type)
		pllMpy= CSL_AIF2_PLL_MUL_FACTOR_20X; /*153.6*20=3072*/
	else
		pllMpy= CSL_AIF2_PLL_MUL_FACTOR_25X; /*122.88*25=3072*/
#else
	if(DUAL_NYQUIST_EVM==gDSP_board_type)
		pllMpy= CSL_AIF2_PLL_MUL_FACTOR_16X; /*153.6*16=2457.6*/
	else
		pllMpy= CSL_AIF2_PLL_MUL_FACTOR_20X; /*122.88*20=2457.6*/
#endif

	if(aifLinkCfg[4].linkEnable|aifLinkCfg[5].linkEnable)
	{	/*Serdes B4 is used for link 4~5*/
		SdCommonSetup.bEnablePllB4 = TRUE;
		SdCommonSetup.CLKBYP_B4 = CSL_AIF2_PLL_CLOCK_NO_BYPASS;
		SdCommonSetup.LB_B4 = CSL_AIF2_PLL_LOOP_BAND_MID;//High BW is also fine
		SdCommonSetup.VoltRangeB4 = CSL_AIF2_PLL_VOLTAGE_LOW;//fixed factor
		SdCommonSetup.SleepPllB4 = CSL_AIF2_PLL_AWAKE;
		SdCommonSetup.pllMpyFactorB4 = pllMpy;
		SdCommonSetup.SysClockSelect = CSL_AIF2_SD_BYTECLOCK_FROM_B4;
	}
	
	if(aifLinkCfg[0].linkEnable|aifLinkCfg[1].linkEnable
		|aifLinkCfg[2].linkEnable|aifLinkCfg[3].linkEnable)
	{	/*Serdes B8 is used for link 0~3*/
		SdCommonSetup.bEnablePllB8 = TRUE;
		SdCommonSetup.CLKBYP_B8 = CSL_AIF2_PLL_CLOCK_NO_BYPASS;
		SdCommonSetup.LB_B8 = CSL_AIF2_PLL_LOOP_BAND_MID;//High BW is also fine
		SdCommonSetup.VoltRangeB8 = CSL_AIF2_PLL_VOLTAGE_LOW;//fixed factor
		SdCommonSetup.SleepPllB8 = CSL_AIF2_PLL_AWAKE;
		SdCommonSetup.pllMpyFactorB8 = pllMpy;

		/*if both Serdes B4 and B8 are enable, prefer to select B8 as SYSCLK source*/
		SdCommonSetup.SysClockSelect = CSL_AIF2_SD_BYTECLOCK_FROM_B8;
	}

	/*---------------PE/PD common setup---------------*/
	//Phase alignment for scheduling DMA normally set to zero
	PeCommonSetup.PeTokenPhase = 0;
	PeCommonSetup.EnetHeaderSelect = 0;
#ifdef DB_BUF_LEN_256
	PeCommonSetup.GlobalDioLen = CSL_AIF2_DB_DIO_LEN_256;
#else
	PeCommonSetup.GlobalDioLen = CSL_AIF2_DB_DIO_LEN_128;
#endif

	//AxC data uses DIO as DMA machine
	PdCommonSetup.PdCppiDioSel = CSL_AIF2_DIO;

#ifdef AIF2_LINK_PROTOCOL_OBSAI
	/*Window for TS=0 checking*/
	PdCommonSetup.AxCOffsetWin = AXC_OFFSET_WIN;//AxC offset window
	PdCommonSetup.PdRadtTC = FRAME_CLOCK_COUNT-1;
#endif

	/*framing counter configuration, still needed for DIO mode*/
	PdCommonSetup.PdFrameTC[0].FrameIndexSc = 0;
	PeCommonSetup.PeFrameTC[0].FrameIndexSc = 0;
	PdCommonSetup.PdFrameTC[0].FrameIndexTc = 0;
	PeCommonSetup.PeFrameTC[0].FrameIndexTc = 0;
	PdCommonSetup.PdFrameMsgTc[0] = WCDMA_SLOT_NUM_MSG-1;
	PdCommonSetup.PdFrameTC[0].FrameSymbolTc = WCDMA_FRAME_NUM_SLOT-1;
	PeCommonSetup.PeFrameTC[0].FrameSymbolTc = WCDMA_FRAME_NUM_SLOT-1;

#ifdef AIF2_LINK_PROTOCOL_OBSAI
	/*OBSAI: number of 16-byte message*/
	PeCommonSetup.PeFrameMsgTc[0] = WCDMA_SLOT_NUM_MSG-1;
#else
	/*CPRI: number of four byte sample*/
 	PeCommonSetup.PeFrameMsgTc[0] = WCDMA_SLOT_NUM_CHIP-1;
#endif

	/*---------------DB Common setup---------------*/
	IngrDbSetup.bEnableIngrDb = TRUE; //Enable Ingress DB
	EgrDbSetup.bEnableEgrDb = TRUE; //Enable Ingress DB
#ifdef DB_BUF_LEN_256
	IngrDbSetup.DioBufferLen = CSL_AIF2_DB_DIO_LEN_256; //Ingress DB DIO buffer length
	EgrDbSetup.DioBufferLen = CSL_AIF2_DB_DIO_LEN_256; //Egress DB DIO buffer length
#else
	IngrDbSetup.DioBufferLen = CSL_AIF2_DB_DIO_LEN_128; //Ingress DB DIO buffer length
	EgrDbSetup.DioBufferLen = CSL_AIF2_DB_DIO_LEN_128; //Egress DB DIO buffer length
#endif
	EgrDbSetup.PmControl = CSL_AIF2_DB_AXC_TOKEN_FIFO;

	/*---------------AD Common setup---------------*/
	AdCommonSetup.IngrGlobalEnable = TRUE;
	AdCommonSetup.EgrGlobalEnable = TRUE;
	AdCommonSetup.IngrGlobalDioEnable = TRUE;
	AdCommonSetup.EgrGlobalDioEnable = TRUE;
	AdCommonSetup.FailMode = CSL_AIF2_AD_DROP;
	AdCommonSetup.IngrPriority = CSL_AIF2_AD_DIO_PRI;

	/*CPRI packets cannot be starved. We recommend that this bit be set to 1
	for CPRI applications that have non-AxC packets. For OBSAI, we expect this
	bit to be 0, because OBSAI has a mechanism for handling non-AxC packet 
	starvation.*/
#ifdef AIF2_LINK_PROTOCOL_OBSAI
	AdCommonSetup.EgrPriority = CSL_AIF2_AD_AXC_PRI;
#else 
	AdCommonSetup.EgrPriority = CSL_AIF2_AD_NON_AXC_PRI;
#endif
	AdCommonSetup.Tx_QueNum = QMSS_AIF_QUEUE_BASE;

	/*---------------DIO Common setup---------------*/
	/*** DIO engine 0 for RAC test ***/
	if(uiDioStreamCnt[0])
	{
		AdDioSetup.IngrDioEngine[0].NumAxC = uiDioStreamCnt[0]-1;
		AdDioSetup.IngrDioEngineEnable[0] = TRUE;//Enable DIO Engine 0
		AdDioSetup.IngrDioEngine[0].BcnTableSelect = CSL_AIF2_AD_DIO_BCN_TABLE_0;
		AdDioSetup.IngrDioEngine[0].NumQuadWord = CSL_AIF2_AD_2QUAD;
		AdDioSetup.IngrDioEngine[0].bEnDmaChannel = TRUE; //Enable Dma channel
		AdDioSetup.IngrDioEngine[0].DmaNumBlock = 4-1;
		AdDioSetup.IngrDioEngine[0].DmaBurstLen = CSL_AIF2_AD_4QUAD;
		AdDioSetup.IngrDioEngine[0].DmaBaseAddr = GLOBAL_ADDR(RAC_In_Buf);
		AdDioSetup.IngrDioEngine[0].DmaBurstAddrStride = 4;//in Quard word
		AdDioSetup.IngrDioEngine[0].DmaBlockAddrStride = 0x80;//in Quard word

		AdDioSetup.EgrDioEngine[0].NumAxC = uiDioStreamCnt[0]-1;
		AdDioSetup.EgrDioEngineEnable[0] = TRUE;//Enable DIO Engine 0
		AdDioSetup.EgrDioEngine[0].BcnTableSelect = CSL_AIF2_AD_DIO_BCN_TABLE_0;
		AdDioSetup.EgrDioEngine[0].NumQuadWord = CSL_AIF2_AD_2QUAD;
		AdDioSetup.EgrDioEngine[0].bEnDmaChannel = TRUE; //Enable Dma channel
		AdDioSetup.EgrDioEngine[0].bEnEgressRsaFormat = TRUE; 
		AdDioSetup.EgrDioEngine[0].DmaNumBlock = 4-1;
		AdDioSetup.EgrDioEngine[0].DmaBurstLen = CSL_AIF2_AD_4QUAD;
		AdDioSetup.EgrDioEngine[0].DmaBaseAddr = GLOBAL_ADDR(RAC_Out_Buf);
		AdDioSetup.EgrDioEngine[0].DmaBurstAddrStride = 4;//in Quard word
		AdDioSetup.EgrDioEngine[0].DmaBlockAddrStride = 0x80;//in Quard word
	}

	/*** DIO engine 1 for TAC test ***/
	if(uiDioStreamCnt[1])
	{
		AdDioSetup.IngrDioEngine[1].NumAxC = uiDioStreamCnt[1]-1;
		AdDioSetup.IngrDioEngineEnable[1] = TRUE;//Enable DIO Engine 0
		AdDioSetup.IngrDioEngine[1].BcnTableSelect = CSL_AIF2_AD_DIO_BCN_TABLE_0;
		AdDioSetup.IngrDioEngine[1].NumQuadWord = CSL_AIF2_AD_1QUAD;
		AdDioSetup.IngrDioEngine[1].bEnDmaChannel = TRUE; //Enable Dma channel
		AdDioSetup.IngrDioEngine[1].DmaNumBlock = 32-1;
		AdDioSetup.IngrDioEngine[1].DmaBurstLen = CSL_AIF2_AD_4QUAD;
		AdDioSetup.IngrDioEngine[1].DmaBaseAddr = GLOBAL_ADDR(TAC_In_Buf);
		AdDioSetup.IngrDioEngine[1].DmaBurstAddrStride = 4;//in Quard word
		AdDioSetup.IngrDioEngine[1].DmaBlockAddrStride = 0;//in Quard word

		AdDioSetup.EgrDioEngine[1].NumAxC = uiDioStreamCnt[1]-1;
		AdDioSetup.EgrDioEngineEnable[1] = TRUE;//Enable DIO Engine 0
		AdDioSetup.EgrDioEngine[1].BcnTableSelect = CSL_AIF2_AD_DIO_BCN_TABLE_0;
		AdDioSetup.EgrDioEngine[1].NumQuadWord = CSL_AIF2_AD_1QUAD;
		AdDioSetup.EgrDioEngine[1].bEnDmaChannel = TRUE; //Enable Dma channel
		AdDioSetup.EgrDioEngine[1].bEnEgressRsaFormat = FALSE; 
		AdDioSetup.EgrDioEngine[1].DmaNumBlock = 32-1;
		AdDioSetup.EgrDioEngine[1].DmaBurstLen = CSL_AIF2_AD_4QUAD;
		AdDioSetup.EgrDioEngine[1].DmaBaseAddr = GLOBAL_ADDR(TAC_Out_Buf);
		AdDioSetup.EgrDioEngine[1].DmaBurstAddrStride = 4;//in Quard word
		AdDioSetup.EgrDioEngine[1].DmaBlockAddrStride = 0;//in Quard word
	}

	/*** DIO engine 2 for DSP core test ***/
	if(uiDioStreamCnt[2])
	{
		AdDioSetup.IngrDioEngine[2].NumAxC = uiDioStreamCnt[2]-1;
		AdDioSetup.IngrDioEngineEnable[2] = TRUE;//Enable DIO Engine 0
		AdDioSetup.IngrDioEngine[2].BcnTableSelect = CSL_AIF2_AD_DIO_BCN_TABLE_0;
		AdDioSetup.IngrDioEngine[2].NumQuadWord = CSL_AIF2_AD_4QUAD;
		AdDioSetup.IngrDioEngine[2].bEnDmaChannel = TRUE; //Enable Dma channel
		AdDioSetup.IngrDioEngine[2].DmaNumBlock = 
			NUM_SLOTS_IN_BUF*WCDMA_SLOT_NUM_CHIP/16-1;
		AdDioSetup.IngrDioEngine[2].DmaBurstLen = CSL_AIF2_AD_4QUAD;
		AdDioSetup.IngrDioEngine[2].DmaBaseAddr = GLOBAL_ADDR(Core_In_Buf);
		AdDioSetup.IngrDioEngine[2].DmaBurstAddrStride = 
			NUM_SLOTS_IN_BUF*WCDMA_SLOT_NUM_CHIP/4;//in Quard word
		AdDioSetup.IngrDioEngine[2].DmaBlockAddrStride = 4;//in Quard word

		AdDioSetup.EgrDioEngine[2].NumAxC = uiDioStreamCnt[2]-1;
		AdDioSetup.EgrDioEngineEnable[2] = TRUE;//Enable DIO Engine 0
		AdDioSetup.EgrDioEngine[2].BcnTableSelect = CSL_AIF2_AD_DIO_BCN_TABLE_0;
		AdDioSetup.EgrDioEngine[2].NumQuadWord = CSL_AIF2_AD_4QUAD;
		AdDioSetup.EgrDioEngine[2].bEnDmaChannel = TRUE; //Enable Dma channel
		AdDioSetup.EgrDioEngine[2].DmaNumBlock = 
			NUM_SLOTS_IN_BUF*WCDMA_SLOT_NUM_CHIP/16-1;
		AdDioSetup.EgrDioEngine[2].DmaBurstLen = CSL_AIF2_AD_4QUAD;
		AdDioSetup.EgrDioEngine[2].DmaBaseAddr = GLOBAL_ADDR(Core_Out_Buf);
		AdDioSetup.EgrDioEngine[2].DmaBurstAddrStride = 
			NUM_SLOTS_IN_BUF*WCDMA_SLOT_NUM_CHIP/4;//in Quard word
		AdDioSetup.EgrDioEngine[2].DmaBlockAddrStride = 4;//in Quard word

#ifdef CORE_DATA_TYPE_RSA 	/*7 or 8 bit RSA type data for uplink*/
		AdDioSetup.EgrDioEngine[2].bEnEgressRsaFormat = TRUE; 
#else 	/*15 or 16 bit normal data type for downlink*/
		AdDioSetup.EgrDioEngine[2].bEnEgressRsaFormat = FALSE; 
#endif
	}

	/*---------------AT Common setup---------------*/
#ifdef AIF2_TRIGGER_MANUALLY
	//use Sync input from MMR writing 	
	AtCommonSetup.PhySyncSel = CSL_AIF2_SW_SYNC;
	AtCommonSetup.RadSyncSel = CSL_AIF2_SW_SYNC;
#else
	//use Sync input from pin PHYTSYNC 	
	AtCommonSetup.PhySyncSel = CSL_AIF2_CHIP_INPUT_SYNC;

	//RadT use same sync as PhyT 	
	AtCommonSetup.RadSyncSel = CSL_AIF2_PHYT_CMP_SYNC;
	AtCommonSetup.PhytCompValue = 0;
#endif

	AtCommonSetup.SyncMode = CSL_AIF2_NON_RP1_MODE;
	AtCommonSetup.AutoResyncMode = CSL_AIF2_NO_AUTO_RESYNC_MODE;
	AtCommonSetup.SyncSampleWindow = 15;
	AtCommonSetup.CrcMode = CSL_AIF2_AT_CRC_DONT_USE;//only use for RP1

	AtCommonSetup.AtInit.pPhyTimerInit = &PhyTimerInit;
	PhyTimerInit.ClockNum = 0;
	PhyTimerInit.FrameLsbNum = 0;
	PhyTimerInit.FrameMsbNum = 0;

	AtCommonSetup.AtInit.pRadTimerInit = &RadTimerInit;
	/*Set radt_clkcnt_init value to 2 and leave the compare phyt value 

	to zero, Phyt and Radt will be fully aligned from first frame boundary
	position. Two clock delay between phyt and radt will make those 
	timers fully aligned*/
	RadTimerInit.ClockNum = 2;
	RadTimerInit.SymbolNum = 0;
	RadTimerInit.LutIndexNum = 0;
	RadTimerInit.FrameLsbNum = 0;
	RadTimerInit.FrameMsbNum = 0;

	/*normally, uplink radio timer should be a little bit late,
	for this test, set it half symbol/slot later*/
	AtCommonSetup.AtInit.pUlRadTimerInit = &UlRadTimerInit;
	UlRadTimerInit.ClockNum = WCDMA_SLOT_NUM_CLOCK/2;
	UlRadTimerInit.SymbolNum = WCDMA_FRAME_NUM_SLOT-1;
	UlRadTimerInit.LutIndexNum= 0;
	UlRadTimerInit.FrameLsbNum = 0xFFF;
	UlRadTimerInit.FrameMsbNum = 0xFFFFFFF;
	DlRadTimerInit.FcbMinusOne = TRUE;

	/*normally, downlink radio timer should be a little bit early,
	this configuration is not used for this test*/
	AtCommonSetup.AtInit.pDlRadTimerInit = &DlRadTimerInit;
	DlRadTimerInit.ClockNum = 2+1000;
	DlRadTimerInit.SymbolNum = 0;
	DlRadTimerInit.FrameLsbNum = 0;
	DlRadTimerInit.FrameMsbNum = 0;
	DlRadTimerInit.FcbMinusOne = FALSE;

	AtCommonSetup.AtTerminalCount.pPhyTimerTc = &PhyTimerTc;
	PhyTimerTc.ClockNum = FRAME_CLOCK_COUNT-1;

	/*Maximu frame count, Frame count wrap around after 2^40 Frames=
	1099511627776/100/60/60/24/365= 394 years*/
	PhyTimerTc.FrameLsbNum = 0xFFFFFFFF;
	PhyTimerTc.FrameMsbNum = 0xFF;

	AtCommonSetup.WcdmaDivTC = WCDMA_CHIP_NUM_CLOCK-1;

	AtCommonSetup.AtTerminalCount.pRadTimerTc = &RadTimerTc;
	AtCommonSetup.AtTerminalCount.RadClockCountTc[0] = WCDMA_SLOT_NUM_CLOCK-1;
	RadTimerTc.LutIndexNum = 0; //set WCDMA lut index TC to zero
	RadTimerTc.SymbolNum = WCDMA_FRAME_NUM_SLOT-1; 
	RadTimerTc.FrameLsbNum = 0xFFFFFFFF;
	RadTimerTc.FrameMsbNum = 0xFF;

	//generate event 2 at frame boundary	
	AtEventSetup.bEnableRadEvent[2] = TRUE;
	AtEventSetup.AtRadEvent[2].EventSelect = CSL_AIF2_EVENT_2;
	AtEventSetup.AtRadEvent[2].EvtStrobeSel = CSL_AIF2_RADT_FRAME; 
	AtEventSetup.AtRadEvent[2].EventOffset = 0; 
	AtEventSetup.AtRadEvent[2].EventModulo = 0x3FFFFF;
	AtEventSetup.AtRadEvent[2].EventMaskLsb = 0xFFFFFFFF; 
	AtEventSetup.AtRadEvent[2].EventMaskMsb = 0xFFFFFFFF; 

	//generate event 3 at slot boundary	
	AtEventSetup.bEnableRadEvent[3] = TRUE;
	AtEventSetup.AtRadEvent[3].EventSelect = CSL_AIF2_EVENT_3;
	AtEventSetup.AtRadEvent[3].EvtStrobeSel = CSL_AIF2_RADT_SYMBOL; 
	AtEventSetup.AtRadEvent[3].EventOffset = 0; 
	AtEventSetup.AtRadEvent[3].EventModulo = 0x3FFFFF;
	AtEventSetup.AtRadEvent[3].EventMaskLsb = 0xFFFFFFFF; 
	AtEventSetup.AtRadEvent[3].EventMaskMsb = 0xFFFFFFFF; 

	//generate event 4 for control packet transfer
	AtEventSetup.bEnableRadEvent[4] = TRUE;
	AtEventSetup.AtRadEvent[4].EventSelect = CSL_AIF2_EVENT_4;
	AtEventSetup.AtRadEvent[4].EvtStrobeSel = CSL_AIF2_RADT_SYMBOL; 
	AtEventSetup.AtRadEvent[4].EventOffset = 0; 
	AtEventSetup.AtRadEvent[4].EventMaskLsb = 0xFFFFFFFF; 
	AtEventSetup.AtRadEvent[4].EventMaskMsb = 0xFFFFFFFF; 
#ifdef AIF2_LINK_PROTOCOL_OBSAI
	/*In this test, one control packet contain 16 control messages 
	of OBSAI, there are 1920 control messages per frame*/
	AtEventSetup.AtRadEvent[4].EventModulo = FRAME_CLOCK_COUNT/(1920/16)-1;
#else	/*CPRI*/
	/*In this test, one control packet contain the control words of one 
	hyper frame, there are 150 hyper frames per frame*/
	AtEventSetup.AtRadEvent[4].EventModulo = FRAME_CLOCK_COUNT/150-1;
#endif

	//generate event 5 at symbol/slot boundary with UL RADT	
	AtEventSetup.bEnableRadEvent[5] = TRUE;
	AtEventSetup.AtRadEvent[5].EventSelect = CSL_AIF2_EVENT_5;
	AtEventSetup.AtRadEvent[5].EvtStrobeSel = CSL_AIF2_ULRADT_SYMBOL; 
	AtEventSetup.AtRadEvent[5].EventOffset = 0; 
	AtEventSetup.AtRadEvent[5].EventModulo = 0x3FFFFF;
	AtEventSetup.AtRadEvent[5].EventMaskLsb = 0xFFFFFFFF; 
	AtEventSetup.AtRadEvent[5].EventMaskMsb = 0xFFFFFFFF; 

	//AT Event setup for DIO 0 for RAC test
	if(uiDioStreamCnt[0])
	{
		AtEventSetup.AtEgrDioEvent[0].EventSelect = CSL_AIF2_E_DIO_EVENT_0;
		AtEventSetup.AtEgrDioEvent[0].EventOffset = 0;
		AtEventSetup.AtEgrDioEvent[0].EvtStrobeSel = CSL_AIF2_RADT_FRAME; 
		AtEventSetup.AtEgrDioEvent[0].EventModulo = 8*WCDMA_CHIP_NUM_CLOCK-1;
		AtEventSetup.AtEgrDioEvent[0].DioFrameEventOffset = 0;
		AtEventSetup.AtEgrDioEvent[0].DioFrameStrobeSel = CSL_AIF2_RADT_FRAME;
		AtEventSetup.bEnableEgrDioEvent[0] = TRUE;

		AtEventSetup.AtIngrDioEvent[0].EventSelect = CSL_AIF2_IN_DIO_EVENT_0;
		AtEventSetup.AtIngrDioEvent[0].EventOffset = uiIgrDioOffset%(8*WCDMA_CHIP_NUM_CLOCK);
		AtEventSetup.AtIngrDioEvent[0].EvtStrobeSel = CSL_AIF2_RADT_FRAME; 
		AtEventSetup.AtIngrDioEvent[0].EventModulo = 8*WCDMA_CHIP_NUM_CLOCK-1;
		AtEventSetup.AtIngrDioEvent[0].DioFrameEventOffset = uiIgrDioOffset;
		AtEventSetup.AtIngrDioEvent[0].DioFrameStrobeSel = CSL_AIF2_RADT_FRAME;
		AtEventSetup.bEnableIngrDioEvent[0] = TRUE;
	}
	//AT Event setup for DIO 1 for TAC test
	if(uiDioStreamCnt[1])
	{
		AtEventSetup.AtEgrDioEvent[1].EventSelect = CSL_AIF2_E_DIO_EVENT_1;
		AtEventSetup.AtEgrDioEvent[1].EventOffset = 0;
		AtEventSetup.AtEgrDioEvent[1].EvtStrobeSel = CSL_AIF2_RADT_FRAME; 
		AtEventSetup.AtEgrDioEvent[1].EventModulo = 4*WCDMA_CHIP_NUM_CLOCK-1;
		AtEventSetup.AtEgrDioEvent[1].DioFrameEventOffset = 0;
		AtEventSetup.AtEgrDioEvent[1].DioFrameStrobeSel = CSL_AIF2_RADT_FRAME;
		AtEventSetup.bEnableEgrDioEvent[1] = TRUE;

		AtEventSetup.AtIngrDioEvent[1].EventSelect = CSL_AIF2_IN_DIO_EVENT_1;
		AtEventSetup.AtIngrDioEvent[1].EventOffset = uiIgrDioOffset%(4*WCDMA_CHIP_NUM_CLOCK);
		AtEventSetup.AtIngrDioEvent[1].EvtStrobeSel = CSL_AIF2_RADT_FRAME; 
		AtEventSetup.AtIngrDioEvent[1].EventModulo = 4*WCDMA_CHIP_NUM_CLOCK-1;
		AtEventSetup.AtIngrDioEvent[1].DioFrameEventOffset = uiIgrDioOffset;
		AtEventSetup.AtIngrDioEvent[1].DioFrameStrobeSel = CSL_AIF2_RADT_FRAME;
		AtEventSetup.bEnableIngrDioEvent[1] = TRUE;
	}

	//AT Event setup for DIO 2 for antenna to/from DSP core test
	if(uiDioStreamCnt[2])
	{
		AtEventSetup.AtEgrDioEvent[2].EventSelect = CSL_AIF2_E_DIO_EVENT_2;
		AtEventSetup.AtEgrDioEvent[2].EventOffset = 0;
		AtEventSetup.AtEgrDioEvent[2].EvtStrobeSel = CSL_AIF2_RADT_FRAME; 
		AtEventSetup.AtEgrDioEvent[2].EventModulo = 16*WCDMA_CHIP_NUM_CLOCK-1;
		AtEventSetup.AtEgrDioEvent[2].DioFrameEventOffset = 0;
		AtEventSetup.AtEgrDioEvent[2].DioFrameStrobeSel = CSL_AIF2_RADT_FRAME;
		AtEventSetup.bEnableEgrDioEvent[2] = TRUE;

		AtEventSetup.AtIngrDioEvent[2].EventSelect = CSL_AIF2_IN_DIO_EVENT_2;
		AtEventSetup.AtIngrDioEvent[2].EventOffset = uiIgrDioOffset%(16*WCDMA_CHIP_NUM_CLOCK);
		AtEventSetup.AtIngrDioEvent[2].EvtStrobeSel = CSL_AIF2_RADT_FRAME; 
		AtEventSetup.AtIngrDioEvent[2].EventModulo = 16*WCDMA_CHIP_NUM_CLOCK-1;
		AtEventSetup.AtIngrDioEvent[2].DioFrameEventOffset = uiIgrDioOffset;
		AtEventSetup.AtIngrDioEvent[2].DioFrameStrobeSel = CSL_AIF2_RADT_FRAME;
		AtEventSetup.bEnableIngrDioEvent[2] = TRUE;
	}

	/************** populating AIF2 global setup structures ***************************************************/
	commonSetup.pSdCommonSetup = &SdCommonSetup;
	commonSetup.pPdCommonSetup = &PdCommonSetup;
	commonSetup.pPeCommonSetup = &PeCommonSetup;
	commonSetup.pIngrDbSetup   = &IngrDbSetup;
	commonSetup.pEgrDbSetup    = &EgrDbSetup;
	commonSetup.pAdCommonSetup = &AdCommonSetup;
	commonSetup.pAdDioSetup    = &AdDioSetup;
	commonSetup.pAtCommonSetup = &AtCommonSetup;
	commonSetup.pAtEventSetup  = &AtEventSetup; 
	aif2Setup.commonSetup = &commonSetup;

	globalSetup.frameMode = CSL_AIF2_FRAME_MODE_NORMAL; 
	aif2Setup.globalSetup = &globalSetup;
}

/*enable PD after a number of synchronized frames received,
this function should be called in frame interrupt service routine*/
void Enable_PD_After_Sync(Uint32 linkIndex)
{
	int j;
	Bool bInPiWindow= FALSE, bFrameSync;
	Uint32 PI_error= _abs(aif2Regs->PI_DATA[linkIndex].AT_PIVALUE_LK-
		aifLinkCfg[linkIndex].piOffset);

	if((PI_error<PI_WINDOW/2)||(FRAME_CLOCK_COUNT-PI_error)<PI_WINDOW/2)
		bInPiWindow= TRUE;

	bFrameSync= (CSL_AIF2_RM_LK_STS0_SYNC_STATUS_ST3 == 
		(aif2Regs->G_RM_LKS[linkIndex].RM_LK_STS0&
		CSL_AIF2_RM_LK_STS0_SYNC_STATUS_MASK));
	
	if(bInPiWindow&&bFrameSync)
		aifLinkCfg[linkIndex].uiSyncFrameCnt++;
	else
		aifLinkCfg[linkIndex].uiSyncFrameCnt=0;

	if(aifLinkCfg[linkIndex].uiSyncFrameCnt==FRAME_SYNC_THRESH)
	{
		for(j=aifLinkCfg[linkIndex].firstAxcNum; 
			j<aifLinkCfg[linkIndex].firstAxcNum+ 
			aifLinkCfg[linkIndex].numberAxC; j++)
		{
			//enable PD channel, take effect on next frame bounday
			aif2Regs->PD_DMACHAN[j] |= 1;
		}

#ifdef AIF2_LINK_PROTOCOL_OBSAI
		if(aifLinkCfg[linkIndex].bUseControlMsg)
#else
		if(aifLinkCfg[linkIndex].numCtlWord)
#endif
		{
			//enable PD channel, take effect on next frame bounday
			aif2Regs->PD_DMACHAN[aifLinkCfg[linkIndex].controlChNum] |= 1;
		}
	}
	
	//PD channel enable take effect in next frame boundary
	if(aifLinkCfg[linkIndex].uiSyncFrameCnt>FRAME_SYNC_THRESH)
	{
		aifLinkCfg[linkIndex].bPDEnabled= TRUE;
	}
}

#define AIF_TEAR_DOWN_TIMEOUT_CYCLES 	1000000000
// This function is used to reset AIF
void AIF_soft_reset(void)
{
	int i, startTSC;
	volatile Uint32 uiEmptyFlag= 0;

	//disable AT events out of AIF2
	aif2Regs->AT_EVT_ENABLE = 0;

	//flush TX queues
	for(i= uiNumAifChannel-uiNumCtlChannel; i<uiNumAifChannel; i++)
	{
		while(KeyStone_GetQueueEntryCount(QMSS_AIF_QUEUE_BASE+i))
		{
			/*return the descriptor to free queue*/
			KeyStone_ReturnQueue(KeyStone_queuePop(QMSS_AIF_QUEUE_BASE+i));
		}
	}
	
	/*Wait 10ms*/
	TSC_delay_ms(10);

	startTSC= TSCL;
	/* PD should only be disabled when RM is in SYNC state*/
 	for(i=0; i<NUM_AIF2_LINK; i++)	//for all links
	{
		if(0==aifLinkCfg[i].linkEnable)
			continue;

		while(1!=aif2Regs->G_RM_LKS[i].RM_LK_STS0)
		{
			TSC_delay_us(1);
			if(TSC_getDelay(startTSC)>AIF_TEAR_DOWN_TIMEOUT_CYCLES)
			{
				printf("link %d RM is UNSYNC before reset\n", i);
				break;
			}
		}
	}

	//disable all PD/PE channesl
	for(i=0; i<uiNumAifChannel; i++)
	{
		aif2Regs->PD_DMACHAN[i]= 0; 
		aif2Regs->PE_DMACHAN_EN[i]= 0;
	}
	
	/*Wait 20ms*/
	TSC_delay_ms(20);

	//disable AT internal events for AIF2
	aif2Regs->AT_INTERNAL_EVT_ENABLE = 0;

	//Halt AT timer
	CSL_aif2AtHaltTimer(hAif2 , TRUE);

	//Disable Tx/Rx Serdes link
 	for(i=0; i<NUM_AIF2_LINK; i++)	//for all links
	{
		if(0==aifLinkCfg[i].linkEnable)
			continue;

		CSL_aif2EnDisTxLink(hAif2, FALSE);
		CSL_aif2EnDisRxLink(hAif2, FALSE);
	}


	/*tear down all Packet DMA channels*/
	for(i= uiNumAifChannel-uiNumCtlChannel; i<uiNumAifChannel; i++)
	{
		if(KeyStone_pktDma_RxCh_teardown(aifDmaRxChCfgRegs, i, AIF_TEAR_DOWN_TIMEOUT_CYCLES))
		{
			printf("Packet DMA Rx channel %d tear down timeout\n", i);
			KeyStone_pktDma_RxCh_disable(aifDmaRxChCfgRegs, i);
		}

		if(KeyStone_pktDma_TxCh_teardown(aifDmaTxChCfgRegs, i, AIF_TEAR_DOWN_TIMEOUT_CYCLES))
		{
			printf("Packet DMA Tx channel %d tear down timeout\n", i);
			KeyStone_pktDma_TxCh_disable(aifDmaTxChCfgRegs, i);
		}
	}

	CSL_aif2Reset(hAif2);

	/*Wait 10ms*/
	TSC_delay_ms(10);
	
}

void ConfigAif2()
{
	int i, j;
	WcdmaAif2TestType 	testType;

	// Initialize CSL library, this step is required 
	CSL_aif2Init(&Aif2Context);

	// Open Aif2 and get handle 
	hAif2 = CSL_aif2Open(&Aif2Obj, CSL_AIF, &aif2Param, &status);

	if ((hAif2 == NULL) || (status != CSL_SOK)) 
	{
		puts ("Error opening CSL_AIF2");
		return;
	}

	//enable AIF power and clock domain
	KeyStone_enable_PSC_module(CSL_PSC_PD_AI, CSL_PSC_LPSC_AI);

	AIF_PktDMA_init(); 	

	/******* Initialize Aif2 structures to avoid unwanted configuration *******/ 
	memset(&globalSetup, 0, sizeof(globalSetup));
	memset(&linkSetup, 0, sizeof(linkSetup));
	memset(&commonSetup, 0, sizeof(commonSetup));

	memset(&SdCommonSetup, 0, sizeof(SdCommonSetup));
	memset(&PdCommonSetup, 0, sizeof(PdCommonSetup));
	memset(&PeCommonSetup, 0, sizeof(PeCommonSetup));
	memset(&IngrDbSetup, 0, sizeof(IngrDbSetup));
	memset(&EgrDbSetup, 0, sizeof(EgrDbSetup));
	memset(&AdCommonSetup, 0, sizeof(AdCommonSetup));
	memset(&AdDioSetup, 0, sizeof(AdDioSetup));
	memset(&AtCommonSetup, 0, sizeof(AtCommonSetup));
	memset(&AtEventSetup, 0, sizeof(AtEventSetup));
	memset(&PhyTimerInit, 0, sizeof(PhyTimerInit));
	memset(&RadTimerInit, 0, sizeof(RadTimerInit));
	memset(&UlRadTimerInit, 0, sizeof(UlRadTimerInit));
	memset(&DlRadTimerInit, 0, sizeof(DlRadTimerInit));
	memset(&PhyTimerTc, 0, sizeof(PhyTimerTc));
	memset(&RadTimerTc, 0, sizeof(RadTimerTc));

	memset(&ComLinkSetup, 0, sizeof(ComLinkSetup));
	memset(&SdLinkSetup, 0, sizeof(SdLinkSetup));
	memset(&RmLinkSetup, 0, sizeof(RmLinkSetup));
	memset(&TmLinkSetup, 0, sizeof(TmLinkSetup));
	memset(&PdLinkSetup, 0, sizeof(PdLinkSetup));
	memset(&PeLinkSetup, 0, sizeof(PeLinkSetup));
	memset(&RtLinkSetup, 0, sizeof(RtLinkSetup));
	memset(&AtLinkSetup, 0, sizeof(AtLinkSetup));

	for(i=0; i<NUM_AIF2_LINK; i++)	//for all links
	{
		if(0==aifLinkCfg[i].linkEnable)
			continue;

		Aif2LinkConfig(i);

		/*do not enable PD until number for synchronized frames received*/
		aifLinkCfg[i].uiSyncFrameCnt=0;
		aifLinkCfg[i].bPDEnabled= FALSE;

		testType= aifLinkCfg[i].testType;
		/*map DIO streams for TAC/RAC/Core to AIF2 channels*/
		for(j=aifLinkCfg[i].firstAxcNum; 
			j<aifLinkCfg[i].firstAxcNum+ aifLinkCfg[i].numberAxC; j++)
		{
			if(uiDioStreamCnt[testType] >= 64)
			{
				printf("Error: DIO %d channel number > 64\n", (Uint32)testType);
				break;
			}
			AdDioSetup.IngrDioEngine[testType].DBCN[uiDioStreamCnt[testType]] = j;
			AdDioSetup.EgrDioEngine[testType].DBCN[uiDioStreamCnt[testType]++] = j;
		}
	}

	Aif2CommonConfig();

	/****** Do AIF2 HW setup (set all MMRs above) *******************/
	CSL_aif2HwSetup(hAif2, &aif2Setup);

 	for(i=0; i<NUM_AIF2_LINK; i++)	//for all links
	{
		if(0==aifLinkCfg[i].linkEnable)
			continue;

		hAif2->arg_link = i;//Select link num

		if(TEST_PATH_INTERNAL_LOOPBACK==aifLinkCfg[i].testDataPath)
		{
			//Enable Serdes loopback for link
			CSL_aif2EnDisLinkLoopback(hAif2, TRUE);
 		}

		//Enable Tx/Rx of link
		CSL_aif2EnDisTxLink(hAif2, TRUE);
		CSL_aif2EnDisRxLink(hAif2, TRUE);

	}

	//insert time delay for aif2 configuration complete
	TSC_delay_us(1);

	//AT Arm timer
	CSL_aif2AtArmTimer(hAif2 , TRUE);

	/*indicate AIF2 is initialized and running*/
	aif2Regs->EE_AIF2_RUN_CTL= 
		(1<<CSL_AIF2_EE_AIF2_RUN_STS_AIF2_GLOBAL_RUN_SHIFT)
		|(1<<CSL_AIF2_EE_AIF2_RUN_STS_AIF2_PHY_RUN_SHIFT);

	Clear_AIF2_INTC();
	Enable_AIF_error_status_interrupt();

}

