/****************************************************************************\
 *           Copyright (C) 2011, 2012 Texas Instruments Incorporated.             *
 *                           All Rights Reserved                            *
 *                                                                          *
 * GENERAL DISCLAIMER                                                       *
 * ------------------                                                       *
 * All software and related documentation is provided "AS IS" and without   *
 * warranty or support of any kind and Texas Instruments expressly disclaims*
 * all other warranties, express or implied, including, but not limited to, *
 * the implied warranties of merchantability and fitness for a particular   *
 * purpose.  Under no circumstances shall Texas Instruments be liable for   *
 * any incidental, special or consequential damages that result from the    *
 * use or inability to use the software or related documentation, even if   *
 * Texas Instruments has been advised of the liability.                     *
 ****************************************************************************
 * Created by :                                                             *
 *            Brighton Feng                                                 *
 *            Texas Instruments                                             * 
 *            August 4, 2011                                                *
Updated on June 18, 2012 for TCI6614 by Brighton
\***************************************************************************/
#ifndef AIF_TEST_CONFIG_H
#define AIF_TEST_CONFIG_H
#include <tistdtypes.h>
#include <csl_aif2.h>

/*number of 10ms frame for test*/
#define TEST_NUM_FRAME 		200

#define AIF2_TRIGGER_MANUALLY

/*Number of AIF links be used <= 6*/
#define NUM_AIF2_LINK       6

/*Number of AIF2 Channels be used <= 128*/
#define NUM_AxC_CHANNEL       (56)
#define NUM_AIF_CHANNEL       (64)

/*data type for antenna data to/from DSP core
(data type for antenna data to RAC/from TAC are fixed)*/
//#define CORE_DATA_TYPE_RSA

#define DIO_OVERHEAD                150
#define TAC_RAC_DELAY               10
#define L2_DELAY               		30
#define DDR_DELAY               	50

#define PI_WINDOW                   200
#define AXC_OFFSET_WIN              1000

//AIF timing const
#define PE1_PE2_DELAY               10
#define PE2_TM_DELAY                60
#define SERDES_2X_DELAY             24
#define SERDES_4X_DELAY             15
#define SERDES_8X_DELAY             10
#define PROPAGATION_DELAY 			1 		/*delay on board or fiber*/
#define RM_PE1_DELAY                100 	/*15~500*/
#define RM_DB_DELAY                	180 	

#ifdef AIF2_LINK_PROTOCOL_OBSAI
#define DB_PE2_DELAY                28
#define FRAME_CLOCK_COUNT           3072000
#else
#define DB_PE2_DELAY                0
#define FRAME_CLOCK_COUNT           2457600
#endif

#define WCDMA_FRAME_NUM_SLOT            (15)
#define WCDMA_SLOT_NUM_CHIP             (2560)
#define WCDMA_CHIP_NUM_CLOCK            (FRAME_CLOCK_COUNT*100/3840000)
#define WCDMA_SLOT_NUM_CLOCK    		(WCDMA_SLOT_NUM_CHIP*WCDMA_CHIP_NUM_CLOCK)
#define WCDMA_SLOT_NUM_MSG     			(WCDMA_SLOT_NUM_CHIP/4)

#define OBSAI_TYPE_WCDMA_FDD             0x2
#define OBSAI_TYPE_WCDMA_TDD             0x3
#define OBSAI_TYPE_LTE                   0x0E
#define OBSAI_TYPE_GENERIC               0x0F

#define OBSAI_MESSAGE_TYPE               OBSAI_TYPE_WCDMA_FDD

typedef enum
{
	/* normal operation */
	TEST_PATH_NORMAL_OPERATION = 0,
	/* internal loopback test */
	TEST_PATH_INTERNAL_LOOPBACK,
	/*internal redirection test, the data path is: 
	Rx Serdes -> DSP RM -> DSP TM -> TX Serdes */
	TEST_PATH_INTERNAL_REDIRECTION,
	/*external redirection test, the data path is: 
	DSP0 PE -> DSP0 TM -> DSP1 RM -> DSP1 TM -> DSP0 RM -> DSP0 PD */
	TEST_PATH_EXTERNAL_REDIRECTION,
	/* external line loopback test */
	TEST_PATH_EXTERLNAL_LINE_LOOPBACK
} TestDataPath;

typedef enum
{
	/*test (simulate) antenna data from AIF2 to RAC*/
	WCDMA_AIF2_RAC_TEST = 0,
	/*test (simulate) antenna data from TAC to AIF2*/
	WCDMA_AIF2_TAC_TEST,
	/*test AIF2 antenna data to/from DSP core buffer*/
	WCDMA_AIF2_CORE_TEST
} WcdmaAif2TestType;

typedef struct _AifConfig
{
	Uint16 				linkEnable; 	/*1: enable, 0 disable*/
	CSL_Aif2LinkRate	linkRate; 		

	TestDataPath 		testDataPath; 	
	WcdmaAif2TestType 	testType;
	Uint32 				numberAxC; 	/*number of AxC's in this link*/

#ifdef AIF2_LINK_PROTOCOL_OBSAI
	Bool  				bUseControlMsg;
#else //AIF2_LINK_PROTOCOL_CPRI
	/*control words used by application, 
	configured by row number (Ns) of the control words array:
	16<=firstCtlWordP<=Ns<=63*/
	Uint32 				firstCtlWordP; 	
	Uint32 				numCtlWord; 	/*<= 64- firstCtlWordP*/ 	
#endif
	CSL_Aif2DataWidth 	dataWidth;

	Uint32 				deltaOffset; 	
	Uint32 				piOffset; 		
	Uint32 				PeEvent1Offset;
	Uint32 				PeEvent2Offset;

	Uint32 				DBM_X; 	/*X value for Dual Bit Map configuration*/
	Uint32 				firstAxcNum; /*first channel number for AxC in this link*/
	Uint32 				controlChNum; /*channel number for control word or message*/

	CSL_Aif2LinkDataType dataFormat;
	CSL_Aif2DbDataSwap  AxCSwap;
	CSL_Aif2DbIqOrder  IQ_order;

	CSL_Aif2RtConfig 	RT_cfg;
	CSL_Aif2PeRtContol 	PE_RT_ctl;

	Bool bEnableTxSyncMater; 	/*first used link should be set as Sync master*/

	Bool bPDEnabled; 	/*enable PD after frame Sync*/
	Uint32 uiSyncFrameCnt; 	/*number of synchronized frames*/
}AifLinkConfig;

//Aif configuration for 6 links
extern AifLinkConfig      	aifLinkCfg[6];

extern Uint32 uiNumAifChannel; //Total number of channels used
extern Uint32 uiNumCtlChannel; //number of control channels used
extern Uint32 uiIgrDioOffset; 	//Ingress DIO offset

/*look up table of different link rate*/
extern unsigned char linkRate[4];

extern Uint32 uiDSPNum;

//Calculate some other AIF Parameters according above pre-set paremeters
//this function should called before EDMA, FSync and AIF configuration
void InitAifTestParameters();
#endif
