/*  ============================================================================
 *   Copyright (c) Texas Instruments Inc 2010
 *
 *   Use of this software is controlled by the terms and conditions found in the
 *   license agreement under which this software has been supplied.
* ============================================================================
Example to show the performance of DSP core access memory
* =============================================================================
 *  Revision History
 *  ===============
 *  7-Dec-2010 Brighton Feng   File Created
 *  Updated on March 30 2011 for C6678 EVM
 *  Updated on Dec 8, 2014 by Brighton Feng, add memory copy cases for SL2 and DDR
 * ============================================================================
 */

#include <csl_cacheAux.h>
#include <stdio.h>
#include <c6x.h>
#include "KeyStone_common.h"
#include "common_test.h"
#include "CPU_access_test.h"

#define 	LOAD_STORE_TIMES 	(512)

#define 	L2_TEST_BASE_ADDR 	(0x820000)
#define 	L2_END_ADDR 		(0x880000)
#define 	MAX_L2_CACHE 		(256*1024)
#define 	L2_COPY_SIZE 		((L2_END_ADDR-MAX_L2_CACHE-L2_TEST_BASE_ADDR)/2)
#define 	L2_LOAD_STORE_SIZE 	(L2_END_ADDR-L2_TEST_BASE_ADDR)

#define 	L1D_TEST_BASE_ADDR 	(0xf00000)
#define 	L1D_COPY_SIZE 		(8*1024)

#define 	CORE1_L2_TEST_BASE_ADDR	(0x11800000)

#define 	ORIGIN_SL2_TEST_BASE_ADDR 	(0x0C080000)
#define 	REMAP_SL2_TEST_BASE_ADDR 	(0x18080000)
#define 	SL2_LOAD_STORE_SIZE 		(1024*1024)
#define 	SL2_COPY_SIZE 		L2_COPY_SIZE

#define 	DDR_TEST_ADDR 		(0x88000000)
#define 	DDR_LOAD_STORE_SIZE (128*1024*1024)
#define 	DDR_PAGE_SIZE 		(1024*8)

#define 	LD_ST_TEST_OVER_HEAD 	40

//#define MemCopy8Test RepeatMemCopy8Test
void MemCopyTest()
{
	//Configure L1P as 32KB cache, set L1D as 16KB for test
	CACHE_setL1PSize(CACHE_L1_32KCACHE);
	CACHE_setL1DSize(CACHE_L1_16KCACHE);

	puts("\nDSP core 8 bytes aligned Memory Copy test: L1D->L1D");
	MemCopy8Test((unsigned long long *)L1D_TEST_BASE_ADDR,(unsigned long long *)(L1D_TEST_BASE_ADDR+L1D_COPY_SIZE), L1D_COPY_SIZE);

	puts("\nDSP core 8 bytes aligned Memory Copy test: LL2->L1D");
	MemCopy8Test((unsigned long long *)L2_TEST_BASE_ADDR,(unsigned long long *)(L1D_TEST_BASE_ADDR), L1D_COPY_SIZE);

	puts("\nDSP core 8 bytes aligned Memory Copy test: L1D->LL2");
	MemCopy8Test((unsigned long long *)L1D_TEST_BASE_ADDR,(unsigned long long *)(L2_TEST_BASE_ADDR), L1D_COPY_SIZE);


	//Configure L1 as 32KB cache
	CACHE_setL1PSize(CACHE_L1_32KCACHE);
	CACHE_setL1DSize(CACHE_L1_32KCACHE);

	puts("\nDSP core 8 bytes aligned Memory Copy test: LL2->LL2");
	MemCopy8Test((unsigned long long *)L2_TEST_BASE_ADDR,(unsigned long long *)(L2_TEST_BASE_ADDR+L2_COPY_SIZE), L2_COPY_SIZE);

	//make OTHER_CorePac_L2 nonprefetchable, noncacheable
	CACHE_wbInvAllL2(CACHE_WAIT);
	gpCGEM_regs->MAR[CORE1_L2_TEST_BASE_ADDR/16/1024/1024]=0;

	puts("\n\nDSP core 8 bytes aligned Memory Copy test: LL2->Other CorePac L2, nonprefetchable, noncacheable");
	MemCopy8Test((unsigned long long *)L2_TEST_BASE_ADDR,(unsigned long long *)CORE1_L2_TEST_BASE_ADDR, L2_COPY_SIZE);

	puts("\nDSP core 8 bytes aligned Memory Copy test: Other CorePac L2->LL2, nonprefetchable, noncacheable");
	MemCopy8Test((unsigned long long *)CORE1_L2_TEST_BASE_ADDR,(unsigned long long *)L2_TEST_BASE_ADDR, L2_COPY_SIZE);

	puts("\nDSP core 8 bytes aligned Memory Copy test: Other CorePac L2->Other CorePac L2, nonprefetchable, noncacheable");
	MemCopy8Test((unsigned long long *)CORE1_L2_TEST_BASE_ADDR,(unsigned long long *)(CORE1_L2_TEST_BASE_ADDR+L2_COPY_SIZE), L2_COPY_SIZE);

	//make OTHER_CorePac_L2 prefetchable, cacheable, use L1D only
	gpCGEM_regs->MAR[CORE1_L2_TEST_BASE_ADDR/16/1024/1024]=1|
		(1<<CSL_CGEM_MAR0_PFX_SHIFT);	
	CACHE_setL2Size(CACHE_0KCACHE);

	puts("\nDSP core 8 bytes aligned Memory Copy test: LL2->Other CorePac L2, prefetchable, 32KB L1D cache only");
	MemCopy8Test((unsigned long long *)L2_TEST_BASE_ADDR,(unsigned long long *)CORE1_L2_TEST_BASE_ADDR, L2_COPY_SIZE);

	puts("\nDSP core 8 bytes aligned Memory Copy test: Other CorePac L2->LL2, prefetchable, 32KB L1D cache only");
	MemCopy8Test((unsigned long long *)CORE1_L2_TEST_BASE_ADDR,(unsigned long long *)L2_TEST_BASE_ADDR, L2_COPY_SIZE);

	puts("\nDSP core 8 bytes aligned Memory Copy test: Other CorePac L2->Other CorePac L2, prefetchable, 32KB L1D cache only");
	MemCopy8Test((unsigned long long *)CORE1_L2_TEST_BASE_ADDR,(unsigned long long *)(CORE1_L2_TEST_BASE_ADDR+L2_COPY_SIZE), L2_COPY_SIZE);

	//use L1D and L2
	CACHE_setL2Size(CACHE_256KCACHE);

	puts("\nDSP core 8 bytes aligned Memory Copy test: LL2->Other CorePac L2, prefetchable, 32KB L1D and 256KB L2 cache");
	//Clear cache
	CACHE_wbInvAllL2(CACHE_WAIT);
	MemCopy8Test((unsigned long long *)L2_TEST_BASE_ADDR,(unsigned long long *)CORE1_L2_TEST_BASE_ADDR, L2_COPY_SIZE);

	puts("\nDSP core 8 bytes aligned Memory Copy test: Other CorePac L2->LL2, prefetchable, 32KB L1D and 256KB L2 cache");
	//Clear cache
	CACHE_wbInvAllL2(CACHE_WAIT);
	MemCopy8Test((unsigned long long *)CORE1_L2_TEST_BASE_ADDR,(unsigned long long *)L2_TEST_BASE_ADDR, L2_COPY_SIZE);

	puts("\nDSP core 8 bytes aligned Memory Copy test: Other CorePac L2->Other CorePac L2, prefetchable, 32KB L1D and 256KB L2 cache");
	//Clear cache
	CACHE_wbInvAllL2(CACHE_WAIT);
	MemCopy8Test((unsigned long long *)CORE1_L2_TEST_BASE_ADDR,(unsigned long long *)(CORE1_L2_TEST_BASE_ADDR+L2_COPY_SIZE), L2_COPY_SIZE);


	//test SL2 through prefetchable cacheable memory window
	puts("\nDSP core 8 bytes aligned Memory Copy test: LL2->SL2, prefetchable, cacheable");
	MemCopy8Test((unsigned long long *)L2_TEST_BASE_ADDR,(unsigned long long *)ORIGIN_SL2_TEST_BASE_ADDR, L2_COPY_SIZE);

	puts("\nDSP core 8 bytes aligned Memory Copy test: SL2->LL2, prefetchable, cacheable");
	MemCopy8Test((unsigned long long *)ORIGIN_SL2_TEST_BASE_ADDR,(unsigned long long *)L2_TEST_BASE_ADDR, L2_COPY_SIZE);

	puts("\nDSP core 8 bytes aligned Memory Copy test: SL2->SL2, prefetchable, cacheable");
	MemCopy8Test((unsigned long long *)ORIGIN_SL2_TEST_BASE_ADDR,(unsigned long long *)(ORIGIN_SL2_TEST_BASE_ADDR+SL2_COPY_SIZE), SL2_COPY_SIZE);

	//make remapped SL2 noncacheable, nonprefetchable
	CACHE_wbInvAllL2(CACHE_WAIT);
	gpCGEM_regs->MAR[(REMAP_SL2_TEST_BASE_ADDR/16/1024/1024)]=0;	

	//test SL2 through non-prefetchable non-cacheable remapped memory window
	puts("\n\nDSP core 8 bytes aligned Memory Copy test: LL2->remapped SL2, non-prefetchable, non-cacheable");
	MemCopy8Test((unsigned long long *)L2_TEST_BASE_ADDR,(unsigned long long *)REMAP_SL2_TEST_BASE_ADDR, L2_COPY_SIZE);

	puts("\nDSP core 8 bytes aligned Memory Copy test: remapped SL2->LL2, non-prefetchable, non-cacheable");
	MemCopy8Test((unsigned long long *)REMAP_SL2_TEST_BASE_ADDR,(unsigned long long *)L2_TEST_BASE_ADDR, L2_COPY_SIZE);

	puts("\nDSP core 8 bytes aligned Memory Copy test: remapped SL2->remapped SL2, non-prefetchable, non-cacheable");
	MemCopy8Test((unsigned long long *)REMAP_SL2_TEST_BASE_ADDR,(unsigned long long *)(REMAP_SL2_TEST_BASE_ADDR+SL2_COPY_SIZE), SL2_COPY_SIZE);

	//make remapped SL2 cacheable, prefetchable
	gpCGEM_regs->MAR[(REMAP_SL2_TEST_BASE_ADDR/16/1024/1024)]=1|
		(1<<CSL_CGEM_MAR0_PFX_SHIFT);	
	//use L1D only
	CACHE_setL2Size(CACHE_0KCACHE);
	
	//test SL2 through remapped prefetchable cacheable memory window
	puts("\nDSP core 8 bytes aligned Memory Copy test: LL2->remapped SL2, prefetchable, cacheable");
	MemCopy8Test((unsigned long long *)L2_TEST_BASE_ADDR,(unsigned long long *)REMAP_SL2_TEST_BASE_ADDR, L2_COPY_SIZE);

	puts("\nDSP core 8 bytes aligned Memory Copy test: remapped SL2->LL2, prefetchable, cacheable");
	MemCopy8Test((unsigned long long *)REMAP_SL2_TEST_BASE_ADDR,(unsigned long long *)L2_TEST_BASE_ADDR, L2_COPY_SIZE);

	puts("\nDSP core 8 bytes aligned Memory Copy test: remapped SL2->remapped SL2, prefetchable, cacheable");
	MemCopy8Test((unsigned long long *)REMAP_SL2_TEST_BASE_ADDR,(unsigned long long *)(REMAP_SL2_TEST_BASE_ADDR+SL2_COPY_SIZE), SL2_COPY_SIZE);


	//make external memory noncacheable, nonprefetchable
	CACHE_wbInvAllL2(CACHE_WAIT);
	gpCGEM_regs->MAR[(DDR_TEST_ADDR/16/1024/1024)]=0;	

	puts("\n\nCPU 8 bytes aligned Memory Copy test: SL2->DDRA, noncacheable, nonprefetchable");
	MemCopy8Test((unsigned long long *)ORIGIN_SL2_TEST_BASE_ADDR,(unsigned long long *)DDR_TEST_ADDR, L2_COPY_SIZE);

	puts("\nCPU 8 bytes aligned Memory Copy test: DDRA->SL2, noncacheable, nonprefetchable");
	MemCopy8Test((unsigned long long *)DDR_TEST_ADDR,(unsigned long long *)ORIGIN_SL2_TEST_BASE_ADDR, L2_COPY_SIZE);

	puts("\n\nDSP core 8 bytes aligned Memory Copy test: LL2->DDR, noncacheable, nonprefetchable");
	MemCopy8Test((unsigned long long *)L2_TEST_BASE_ADDR,(unsigned long long *)DDR_TEST_ADDR, L2_COPY_SIZE);

	puts("\nDSP core 8 bytes aligned Memory Copy test: DDR->LL2, noncacheable, nonprefetchable");
	MemCopy8Test((unsigned long long *)DDR_TEST_ADDR,(unsigned long long *)L2_TEST_BASE_ADDR, L2_COPY_SIZE);

	puts("\nDSP core 8 bytes aligned Memory Copy test: DDR->DDR same bank, noncacheable, nonprefetchable");
	MemCopy8Test((unsigned long long *)DDR_TEST_ADDR,(unsigned long long *)(DDR_TEST_ADDR+L2_COPY_SIZE), L2_COPY_SIZE);

	puts("\nDSP core 8 bytes aligned Memory Copy test: DDR->DDR different bank, noncacheable, nonprefetchable");
	MemCopy8Test((unsigned long long *)DDR_TEST_ADDR,(unsigned long long *)(DDR_TEST_ADDR+L2_COPY_SIZE+DDR_PAGE_SIZE), L2_COPY_SIZE);

	//make external memory prefetchable cacheable, use L1D cache only
	gpCGEM_regs->MAR[(DDR_TEST_ADDR/16/1024/1024)]=1|
		(1<<CSL_CGEM_MAR0_PFX_SHIFT);	
	CACHE_setL2Size(CACHE_0KCACHE);

	puts("\nCPU 8 bytes aligned Memory Copy test: SL2->DDRA, prefetchable, 32KB L1D cache only");
	MemCopy8Test((unsigned long long *)ORIGIN_SL2_TEST_BASE_ADDR,(unsigned long long *)DDR_TEST_ADDR, L2_COPY_SIZE);

	puts("\nCPU 8 bytes aligned Memory Copy test: DDRA->SL2, prefetchable, 32KB L1D cache only");
	MemCopy8Test((unsigned long long *)DDR_TEST_ADDR,(unsigned long long *)ORIGIN_SL2_TEST_BASE_ADDR, L2_COPY_SIZE);

	puts("\nDSP core 8 bytes aligned Memory Copy test: LL2->DDR, prefetchable, 32KB L1D cache only");
	MemCopy8Test((unsigned long long *)L2_TEST_BASE_ADDR,(unsigned long long *)DDR_TEST_ADDR, L2_COPY_SIZE);

	puts("\nDSP core 8 bytes aligned Memory Copy test: DDR->LL2, prefetchable, 32KB L1D cache only");
	MemCopy8Test((unsigned long long *)DDR_TEST_ADDR,(unsigned long long *)L2_TEST_BASE_ADDR, L2_COPY_SIZE);

	puts("\nDSP core 8 bytes aligned Memory Copy test: DDR->DDR same bank, prefetchable, 32KB L1D cache only");
	MemCopy8Test((unsigned long long *)DDR_TEST_ADDR,(unsigned long long *)(DDR_TEST_ADDR+L2_COPY_SIZE), L2_COPY_SIZE);

	puts("\nDSP core 8 bytes aligned Memory Copy test: DDR->DDR different bank, prefetchable, 32KB L1D cache only");
	MemCopy8Test((unsigned long long *)DDR_TEST_ADDR,(unsigned long long *)(DDR_TEST_ADDR+L2_COPY_SIZE+DDR_PAGE_SIZE), L2_COPY_SIZE);

	//use both L1D and L2 cache
	CACHE_setL2Size(CACHE_256KCACHE);

	puts("\nCPU 8 bytes aligned Memory Copy test: SL2->DDRA, prefetchable, 32KB L1D and 256KB L2 cache");
	//Clear cache
	CACHE_wbInvAllL2(CACHE_WAIT);
	MemCopy8Test((unsigned long long *)ORIGIN_SL2_TEST_BASE_ADDR,(unsigned long long *)(DDR_TEST_ADDR+16), L2_COPY_SIZE);

	puts("\nCPU 8 bytes aligned Memory Copy test: DDRA->SL2, prefetchable, 32KB L1D and 256KB L2 cache");
	//Clear cache
	CACHE_wbInvAllL2(CACHE_WAIT);
	MemCopy8Test((unsigned long long *)DDR_TEST_ADDR,(unsigned long long *)ORIGIN_SL2_TEST_BASE_ADDR, L2_COPY_SIZE);

	puts("\nDSP core 8 bytes aligned Memory Copy test: LL2->DDR, prefetchable, 32KB L1D and 256KB L2 cache");
	//Clear cache
	CACHE_wbInvAllL2(CACHE_WAIT);
	MemCopy8Test((unsigned long long *)L2_TEST_BASE_ADDR,(unsigned long long *)DDR_TEST_ADDR, L2_COPY_SIZE);

	puts("\nDSP core 8 bytes aligned Memory Copy test: DDR->LL2, prefetchable, 32KB L1D and 256KB L2 cache");
	//Clear cache
	CACHE_wbInvAllL2(CACHE_WAIT);
	MemCopy8Test((unsigned long long *)DDR_TEST_ADDR,(unsigned long long *)L2_TEST_BASE_ADDR, L2_COPY_SIZE);

	puts("\nDSP core 8 bytes aligned Memory Copy test: DDR->DDR same bank, prefetchable, 32KB L1D and 256KB L2 cache");
	//Clear cache
	CACHE_wbInvAllL2(CACHE_WAIT);
	MemCopy8Test((unsigned long long *)DDR_TEST_ADDR,(unsigned long long *)(DDR_TEST_ADDR+L2_COPY_SIZE), L2_COPY_SIZE);

	puts("\nDSP core 8 bytes aligned Memory Copy test: DDR->DDR different bank, prefetchable, 32KB L1D and 256KB L2 cache");
	//Clear cache
	CACHE_wbInvAllL2(CACHE_WAIT);
	MemCopy8Test((unsigned long long *)DDR_TEST_ADDR,(unsigned long long *)(DDR_TEST_ADDR+L2_COPY_SIZE+DDR_PAGE_SIZE), L2_COPY_SIZE);
}

void LoadStoreCycleTest()
{
	Int32 i;
	
	//Configure L1 as 32KB cache
	CACHE_setL1PSize(CACHE_L1_32KCACHE);
	CACHE_setL1DSize(CACHE_L1_0KCACHE);

	//LL2 memory test
	CACHE_setL2Size(CACHE_0KCACHE);
	puts("\nLL2 LDDW test: 0KB L1D cahce");
	LDDWTest(L2_TEST_BASE_ADDR, L2_LOAD_STORE_SIZE);
	puts("\nLL2 STDW test: 0KB L1D cahce");
	STDWTest(L2_TEST_BASE_ADDR, L2_LOAD_STORE_SIZE);

	//Configure L1 as 32KB cache
	CACHE_setL1PSize(CACHE_L1_32KCACHE);
	CACHE_setL1DSize(CACHE_L1_32KCACHE);

	//LL2 memory test
	CACHE_setL2Size(CACHE_0KCACHE);
	puts("\nLL2 LDDW test: 32KB L1D cahce");
	LDDWTest(L2_TEST_BASE_ADDR, L2_LOAD_STORE_SIZE);
	puts("\nLL2 STDW test: 32KB L1D cahce");
	STDWTest(L2_TEST_BASE_ADDR, L2_LOAD_STORE_SIZE);

	//make original SL2 noncacheable, nonprefetchable
	CACHE_setL1DSize(CACHE_L1_0KCACHE);
	gpCGEM_regs->MAR[(ORIGIN_SL2_TEST_BASE_ADDR/16/1024/1024)]= 1|
		(0<<CSL_CGEM_MAR0_PFX_SHIFT);		

	//test SL2 nonprefetchable noncacheable
	puts("\nSL2 LDDW test: nonprefetchable, noncacheable");
	LDDWTest((ORIGIN_SL2_TEST_BASE_ADDR), SL2_LOAD_STORE_SIZE);
	puts("\nSL2 STDW test: nonprefetchable, noncacheable");
	STDWTest((ORIGIN_SL2_TEST_BASE_ADDR), SL2_LOAD_STORE_SIZE);

	//make original SL2 cacheable, prefetchable
	CACHE_setL1DSize(CACHE_L1_32KCACHE);
	gpCGEM_regs->MAR[(ORIGIN_SL2_TEST_BASE_ADDR/16/1024/1024)]= 1|
		(1<<CSL_CGEM_MAR0_PFX_SHIFT);		

	//test SL2 through prefetchable cacheable memory window
	puts("\nSL2 LDDW test: prefetchable, cacheable");
	LDDWTest((ORIGIN_SL2_TEST_BASE_ADDR), SL2_LOAD_STORE_SIZE);
	puts("\nSL2 STDW test: prefetchable, cacheable");
	STDWTest((ORIGIN_SL2_TEST_BASE_ADDR), SL2_LOAD_STORE_SIZE);

	//make remapped SL2 noncacheable, nonprefetchable
	CACHE_wbInvAllL2(CACHE_WAIT);
	gpCGEM_regs->MAR[(REMAP_SL2_TEST_BASE_ADDR/16/1024/1024)]=0;	

	//test SL2 through non-prefetchable, non-cacheable memory window
	puts("\nremapped SL2 LDDW test: non-prefetchable, non-cacheable");
	LDDWTest((REMAP_SL2_TEST_BASE_ADDR), SL2_LOAD_STORE_SIZE);
	puts("\nremapped SL2 STDW test: non-prefetchable, non-cacheable");
	STDWTest((REMAP_SL2_TEST_BASE_ADDR), SL2_LOAD_STORE_SIZE);
	
	//make remapped SL2 cacheable, prefetchable
	gpCGEM_regs->MAR[(REMAP_SL2_TEST_BASE_ADDR/16/1024/1024)]=1|
		(1<<CSL_CGEM_MAR0_PFX_SHIFT);	
	//use L1D only
	CACHE_setL2Size(CACHE_0KCACHE);
	
	//test SL2 through prefetchable, cacheable memory window
	puts("\nremapped SL2 LDDW test: prefetchable, cacheable");
	LDDWTest((REMAP_SL2_TEST_BASE_ADDR), SL2_LOAD_STORE_SIZE);
	puts("\nremapped SL2 STDW test: prefetchable, cacheable");
	STDWTest((REMAP_SL2_TEST_BASE_ADDR), SL2_LOAD_STORE_SIZE);
	

	//make external memory nonprefetchable, noncacheable
	CACHE_wbInvAllL2(CACHE_WAIT);
	for(i=0; i< (DDR_LOAD_STORE_SIZE/(16*1024*1024)); i++)
		gpCGEM_regs->MAR[(DDR_TEST_ADDR/16/1024/1024)+i]=0;	

	puts("\nDDR LDDW test: nonprefetchable, noncacheable");
	LDDWTest((DDR_TEST_ADDR), DDR_LOAD_STORE_SIZE);
	puts("\nDDR STDW test: nonprefetchable, noncacheable");
	STDWTest((DDR_TEST_ADDR), DDR_LOAD_STORE_SIZE);
	
	//make external memory prefetchable, cacheable
	for(i=0; i< (DDR_LOAD_STORE_SIZE/(16*1024*1024)); i++)
		gpCGEM_regs->MAR[(DDR_TEST_ADDR/16/1024/1024)+i]= 1|
		(1<<CSL_CGEM_MAR0_PFX_SHIFT);	 	
	CACHE_setL2Size(CACHE_0KCACHE);

	puts("\nDDR LDDW test: prefetchable, 32KB L1D cahce");
	LDDWTest((DDR_TEST_ADDR), DDR_LOAD_STORE_SIZE);
	puts("\nDDR STDW test: prefetchable, 32KB L1D cahce");
	STDWTest((DDR_TEST_ADDR), DDR_LOAD_STORE_SIZE);
	
	//set 256KB L2 cache
	CACHE_setL2Size(CACHE_256KCACHE);

	puts("\nDDR LDDW test: prefetchable, 32KB L1D, 256KB L2 cahce");
	LDDWTest((DDR_TEST_ADDR), DDR_LOAD_STORE_SIZE);
	puts("\nDDR STDW test: prefetchable, 32KB L1D, 256KB L2 cahce");
	STDWTest((DDR_TEST_ADDR), DDR_LOAD_STORE_SIZE);
	
}

