/*  ============================================================================
 *   Copyright (c) Texas Instruments Inc 2011
 *
 *   Use of this software is controlled by the terms and conditions found in the
 *   license agreement under which this software has been supplied.
 *   ===========================================================================
 */
/** ============================================================================
Interrupts configuration for QMSS
 * =============================================================================
 *  Revision History
 *  ===============
 *  8-May-2011 Brighton Feng  file created
 * =============================================================================
 */

#include <stdio.h>
#include <c6x.h> 
#include <csl_cpintc.h>
#include "KeyStone_common.h"

int uiQ662_pend_system_event_num;
int uiQ662_pend_host_event_num;

void CIC_Init_QM(void)
{
	/* Disable Global host interrupts. */
	gpCIC0_regs->GLOBAL_ENABLE_HINT_REG= 0;

	if(C6670_EVM==gDSP_board_type
		||DUAL_NYQUIST_EVM==gDSP_board_type
		||TCI6614_EVM==gDSP_board_type)
	{	
		uiQ662_pend_system_event_num= 134;
		uiQ662_pend_host_event_num= 64;
		
		/*map Queue 662 pend interrupts events (134) to CIC0 out64*/
		KeyStone_CIC_event_map(gpCIC0_regs, 134, 64);
	}
	else if(C6678_EVM==gDSP_board_type)
	{	
		/*map Queue 662 pend interrupts events (156) to CIC0 out33*/
		uiQ662_pend_system_event_num= 156;
		uiQ662_pend_host_event_num= 33;

		KeyStone_CIC_event_map(gpCIC0_regs, 156, 33);
	}
	else
	{
		puts("Unknown DSP board type!");
		return;
	}	

	/* Enable Global host interrupts. */
	gpCIC0_regs->GLOBAL_ENABLE_HINT_REG= 1;

}

void QM_Interrupts_Init(void)
{
	CIC_Init_QM();
	
	//map high priority queue channel 0 interrupt to INT4
	//map low priority queues channel 0 interrupt to INT5
	/*on Nyquist, CIC0 out64 event number are 22 on core 0		
	on Shannon, CIC0 out33 event number are 22 on core0		
	map this event 22 to INT6 */
	gpCGEM_regs->INTMUX1 = (CSL_GEM_QM_INT_HIGH_N<<CSL_CGEM_INTMUX1_INTSEL4_SHIFT)|
		(CSL_GEM_QM_INT_LOW_0<<CSL_CGEM_INTMUX1_INTSEL5_SHIFT)|
		(22<<CSL_CGEM_INTMUX1_INTSEL6_SHIFT);

	//enable INT4, 5, 6
	CPU_interrupt_enable((1<<4)|(1<<5)|(1<<6));
}


