/******************************************************************************

  Copyright (C), 2001-2012, Texas Instrument.

 ******************************************************************************
  File Name     : Robust_DDR_SES_MP.c
  Version       : Initial Draft
  Author        : Zhan
  Created       : 2012/3/11
  Last Modified :
  Description   : DDR SES memory protection example
  Function List :
              MSMCSesMemoryBufInit
              SES_DDR_MP_test
  History       :
  1.Date        : 2012/3/11
    Author      : Zhan
    Modification: Created file

******************************************************************************/


#include <stdio.h>
#include "KeyStone_common.h"
#include "KeyStone_SRIO_init_drv.h"
#include "Robust_SRIO_Init.h"

/*----------------------------------------------*
 * external variables                           *
 *----------------------------------------------*/

/*----------------------------------------------*
 * external routine prototypes                  *
 *----------------------------------------------*/
extern void InitLsuTransfer(SRIO_LSU_Transfer * lsuTransfer,
	SRIO_Transfer_Param * transferParam, Uint32 uiLsuNum,
	Uint32 uiDstID, Uint32 uiSrcIDMap);
/*----------------------------------------------*
 * internal routine prototypes                  *
 *----------------------------------------------*/

/*----------------------------------------------*
 * project-wide global variables                *
 *----------------------------------------------*/

/*----------------------------------------------*
 * module-wide global variables                 *
 *----------------------------------------------*/
/*total 64K data for MSMC SES*/
#pragma DATA_SECTION(ddrSesMPDataDst, "ddrSesMPData_w")
#pragma DATA_ALIGN(ddrSesMPDataDst,64*1024) //align to MPAX segment bouandary
volatile char ddrSesMPDataDst[2][32768];

/* 16K data */
#pragma DATA_SECTION(ddrSesMPDataSrc, "ddrSesMPData_r")
#pragma DATA_ALIGN(ddrSesMPDataSrc,16*1024) //align to MPAX segment bouandary
volatile char ddrSesMPDataSrc[4][4096];

MPAX_Config SES_MPAX_cfg_table[]=
{
    /*BADDR                  RADDR           SegementSize            AccessPermisionMask
    32-bit virtual           36-bit physical in byte, must           Access types allowed
    base address             address right   be power of 2           in this address range
                             shift by 4                                                    */
    {(Uint32)ddrSesMPDataDst, 0,          sizeof(ddrSesMPDataDst), MP_SW|MP_UW},/*write only data*/
    {(Uint32)ddrSesMPDataSrc, 0,          sizeof(ddrSesMPDataSrc), MP_SR|MP_UR}/*read only data*/
};

/*----------------------------------------------*
 * constants                                    *
 *----------------------------------------------*/

/*----------------------------------------------*
 * macros                                       *
 *----------------------------------------------*/

/*----------------------------------------------*
 * routines' implementations                    *
 *----------------------------------------------*/

/*****************************************************************************
 Prototype    : MSMCSesMemoryBufInit
 Description  : SES test buffer initialization
 Input        : None
 Output       : None
 Return Value : 
 
  History        :
  1.Date         : 2012/10/20
    Author       : Zhan
    Modification : Created function

*****************************************************************************/
void MSMCSesMemoryBufInit()
{
	int i;
	for(i=0; i<4096; i++)
	{
		ddrSesMPDataSrc[0][i] = 0xA0;
		ddrSesMPDataSrc[1][i] = 0xC0;
		ddrSesMPDataSrc[2][i] = 0xD0;
		ddrSesMPDataSrc[3][i] = 0xE0;

	}

	for(i=0; i<32768; i++)
	{
		ddrSesMPDataDst[0][i] = 0xBB;
		ddrSesMPDataDst[1][i] = 0xBB;
	}
	return;
}

/*****************************************************************************
 Prototype    : SES_DDR_MP_test
 Description  : This test shows the MSMC memory protection among different
                master
 Input        : None
 Output       : None
 Return Value : 
 
  History        :
  1.Date         : 2012/10/29
    Author       : Zhan
    Modification : Created function

  2.Date         : 2012/10/28
    Author       : Zhan
    Modification : Add SRIO access
*****************************************************************************/
void SES_DDR_MP_test()
{
	int i;
    Uint32 uwSrioPriVid;
	SRIO_Transfer_Param transferParam;
	SRIO_LSU_Transfer lsuTransfer;    
	puts("===================-SES DDR memory protection test-==============");

	MSMCSesMemoryBufInit();

	SES_MPAX_cfg_table[0].RADDR= DDR_PHY_ADDR(SES_MPAX_cfg_table[0].BADDR);
	SES_MPAX_cfg_table[1].RADDR= DDR_PHY_ADDR(SES_MPAX_cfg_table[1].BADDR);
	KeyStone_SES_MPAX_setup(SES_MPAX_cfg_table, 1,
		sizeof(SES_MPAX_cfg_table)/sizeof(MPAX_Config), DNUM);

	/* EDMA copy data from ddrSesMPDataSrc to ddrSesMPDataDst is allowed */
	/* The first EDMA write is allowed */
	printf("EDMA copy from 0x%x (read-only) to 0x%x (write-only)...\n",
		GLOBAL_ADDR((Uint32)&ddrSesMPDataSrc[0][0]),
		GLOBAL_ADDR((Uint32)&ddrSesMPDataDst[0][0]));
	EDMA_copy(GLOBAL_ADDR((Uint32)&ddrSesMPDataSrc[0][0]),
			  GLOBAL_ADDR((Uint32)&ddrSesMPDataDst[0][0]),
			  64, EDMA_CC0_CH0, DMA_WAIT);

	/*EDMA copy data from ddrSesMPDataDst to ddrSesMPDataSrc is not allowed */
	printf("\n!!!EDMA copy from 0x%x (write-only) to 0x%x (read-only)...\n",
		GLOBAL_ADDR((Uint32)&ddrSesMPDataDst[1][0]),
		GLOBAL_ADDR((Uint32)&ddrSesMPDataSrc[1][0]));
	EDMA_copy(GLOBAL_ADDR((Uint32)&ddrSesMPDataDst[1][0]),
			  GLOBAL_ADDR((Uint32)&ddrSesMPDataSrc[1][0]),
			  32, EDMA_CC0_CH0, DMA_NO_WAIT);

	/*DMA access has some latency, and normally, exception is 
	captured about 15~45 cycles after the access violation.
	Dummy operations added here to wait for DMA access and excpetion*/
	asm(" nop 5");	asm(" nop 5");
	asm(" nop 5");	asm(" nop 5");
	asm(" nop 5");	asm(" nop 5");
	asm(" nop 5");	asm(" nop 5");
	asm(" nop 5");	asm(" nop 5");
	asm(" nop 5");	asm(" nop 5");
	asm(" nop 5");	asm(" nop 5");
	asm(" nop 5");	asm(" nop 5");
	asm(" nop 5");	asm(" nop 5");
	asm(" nop 5");	asm(" nop 5");/*NRP should be around the address of this instruction*/
	asm(" nop 5");	asm(" nop 5");
	EDMA_wait(EDMA_CC0_CH0);

	printf("\n!!!DSP core write to gpMSMC_regs->SES_MPAX_PER_PRIVID[%d].SES[0].MPAXL register at 0x%x without unlocking...\n",
		DNUM, &gpMSMC_regs->SES_MPAX_PER_PRIVID[DNUM].SES[0].MPAXL);
	gpMSMC_regs->SES_MPAX_PER_PRIVID[DNUM].SES[0].MPAXL= 0xFFFFFFFF;

	/*normally, exception is captured about 10~100 cycles after the 
	access violation. Dummy operations added here to wait for it.*/
	asm(" nop 5");
	asm(" nop 5");
	asm(" nop 5");
	asm(" nop 5");
	asm(" nop 5");
	asm(" nop 5");
	asm(" nop 5");
	asm(" nop 5"); 	/*NRP should be around the address of this instruction*/
	asm(" nop 5"); 	

	puts("--------------------SRIO SES DDR3 memory protection test---------------");

	if((C6670_EVM == gDSP_board_type))
	{	
		uwSrioPriVid = PRIVID_SRIO_C6670;
	}
	else if(C6678_EVM == gDSP_board_type)
	{
		uwSrioPriVid = PRIVID_SRIO_C6678;
	}
	else if(TCI6614_EVM == gDSP_board_type)
	{
		uwSrioPriVid = PRIVID_SRIODIO_TCI6614;
	}    
	else
	{
		puts("Unknown DSP type!");
		return;
	}    
    KeyStone_MSMC_MP_interrupt_en(1<<uwSrioPriVid);

	SES_MPAX_cfg_table[0].RADDR= DDR_PHY_ADDR(SES_MPAX_cfg_table[0].BADDR);
	SES_MPAX_cfg_table[1].RADDR= DDR_PHY_ADDR(SES_MPAX_cfg_table[1].BADDR);
	KeyStone_SES_MPAX_setup(SES_MPAX_cfg_table, 1,
		sizeof(SES_MPAX_cfg_table)/sizeof(MPAX_Config), uwSrioPriVid);  
    
	/* The first SRIO write is allowed */
	printf("SRIO copy from 0x%x (read-only) to 0x%x (write-only)...\n",
		GLOBAL_ADDR((Uint32)&ddrSesMPDataSrc[2][0]),
		GLOBAL_ADDR((Uint32)&ddrSesMPDataDst[1][0]));

	transferParam.byteCount = 32;
	transferParam.packet_type = SRIO_PKT_TYPE_NWRITE_R;
	transferParam.source = GLOBAL_ADDR((Uint32)&ddrSesMPDataSrc[2][0]);
	transferParam.dest = GLOBAL_ADDR((Uint32)&ddrSesMPDataDst[1][0]);

	InitLsuTransfer(&lsuTransfer, &transferParam, 0, 
		DSP0_SRIO_BASE_ID, 0);
	
	KeyStone_SRIO_LSU_transfer(&lsuTransfer);

	/*wait for transfer complete*/
	KeyStone_SRIO_wait_LSU_completion(0,
		lsuTransfer.transactionID, lsuTransfer.contextBit);

	/* SRIO copy data from msmcMPDataSrc to msmcMPDataSrc is not allowed */
	printf("\n!!!SRIO copy from 0x%x (read-only) to 0x%x (read-only)...\n",
		GLOBAL_ADDR((Uint32)&ddrSesMPDataSrc[2][0]),
		GLOBAL_ADDR((Uint32)&ddrSesMPDataSrc[1][0]));

	transferParam.byteCount = 32;
	transferParam.packet_type = SRIO_PKT_TYPE_NWRITE_R;
	transferParam.source = GLOBAL_ADDR((Uint32)&ddrSesMPDataSrc[2][0]);
	transferParam.dest = GLOBAL_ADDR((Uint32)&ddrSesMPDataSrc[1][0]);

	InitLsuTransfer(&lsuTransfer, &transferParam, 0, 
		DSP0_SRIO_BASE_ID, 0);
	
	KeyStone_SRIO_LSU_transfer(&lsuTransfer);

	/*add dummy nop here to wait for the exception happens*/
	for(i=0; i<10; i++)
	{
		asm(" nop 5");		asm(" nop 5");
		asm(" nop 5");		asm(" nop 5");
		asm(" nop 5");		asm(" nop 5");
		asm(" nop 5");		asm(" nop 5");
		asm(" nop 5");		asm(" nop 5");
		asm(" nop 5");		asm(" nop 5");
		asm(" nop 5");		asm(" nop 5");
		asm(" nop 5");		asm(" nop 5");
		asm(" nop 5");		asm(" nop 5");
		asm(" nop 5");		asm(" nop 5");
		asm(" nop 5");		asm(" nop 5");
		asm(" nop 5");		asm(" nop 5");
		asm(" nop 5");		asm(" nop 5");
		asm(" nop 5");		asm(" nop 5");
		asm(" nop 5");		asm(" nop 5");
		asm(" nop 5");		asm(" nop 5");
		asm(" nop 5");		asm(" nop 5");
		asm(" nop 5");		asm(" nop 5");
		asm(" nop 5");		asm(" nop 5");
		asm(" nop 5");		asm(" nop 5");
		asm(" nop 5");		asm(" nop 5");
		asm(" nop 5");		asm(" nop 5");
	}
	
	/*wait for transfer complete*/
	KeyStone_SRIO_wait_LSU_completion(0,
		lsuTransfer.transactionID, lsuTransfer.contextBit);

}

