/******************************************************************************

  Copyright (C), 2001-2012, Texas Instrument.

 ******************************************************************************
  File Name     : Robust_Reserved_space.c
  Version       : Initial Draft
  Author        : Brighton Feng
  Created       : 2012/11/7
  Last Modified :
  Description   : example to show the execptions when access reserved space.
  Function List :
  History       :

******************************************************************************/

#include <stdio.h>
#include "KeyStone_common.h"

/*----------------------------------------------*
 * external variables                           *
 *----------------------------------------------*/

/*----------------------------------------------*
 * external routine prototypes                  *
 *----------------------------------------------*/

/*----------------------------------------------*
 * internal routine prototypes                  *
 *----------------------------------------------*/

/*----------------------------------------------*
 * project-wide global variables                *
 *----------------------------------------------*/

/*----------------------------------------------*
 * module-wide global variables                 *
 *----------------------------------------------*/

/*----------------------------------------------*
 * constants                                    *
 *----------------------------------------------*/

/*----------------------------------------------*
 * macros                                       *
 *----------------------------------------------*/

/*----------------------------------------------*
 * routines' implementations                    *
 *----------------------------------------------*/
/*****************************************************************************
 Prototype    : Uint32 read_reserved_space(Uint32 *)
 Description  : read reserved address
 Input        : address
 Output       : readback value
 Return Value : readback value
 
  History        :
  1.Date         : 2012/11/7
    Author       : Brighton Feng
    Modification : Created function

*****************************************************************************/
Uint32 read_reserved_space(volatile Uint32 * addr)
{
    Uint32 value;

	printf("!!!read from reserved address 0x%x...\n", addr);
	value= *addr;

	/*normally, exception is captured about 10~100 cycles after the 
	access violation. Dummy operations added here to wait for it.*/
	asm(" nop 5");
	asm(" nop 5");
	asm(" nop 5");
	asm(" nop 5"); 	/*NRP should be around the address of this instruction*/
	asm(" nop 5"); 	

	return value;
}
/*****************************************************************************
 Prototype    : void write_reserved_space(Uint32 *)
 Description  : write reserved address
 Input        : address
 Output       : none
 Return Value : none
 
  History        :
  1.Date         : 2012/11/7
    Author       : Brighton Feng
    Modification : Created function

*****************************************************************************/
void write_reserved_space(volatile Uint32 * addr)
{
	printf("\n!!!write to reserved address 0x%x...\n", addr);
	*addr= 0;

	/*normally, exception is captured about 10~100 cycles after the 
	access violation. Dummy operations added here to wait for it.*/
	asm(" nop 5");	asm(" nop 5");
	asm(" nop 5");	asm(" nop 5");
	asm(" nop 5");	asm(" nop 5");
	asm(" nop 5");	asm(" nop 5");
	asm(" nop 5");	asm(" nop 5");
	asm(" nop 5");	asm(" nop 5");
	asm(" nop 5");	asm(" nop 5");
	asm(" nop 5");	asm(" nop 5");
	asm(" nop 5");	asm(" nop 5");
	asm(" nop 5");	asm(" nop 5");/*NRP should be around the address of this instruction*/
	asm(" nop 5");	asm(" nop 5");
}
/*****************************************************************************
 Prototype    : void write_reserved_space(Uint32 *)
 Description  : execute from reserved address

 	exception will be triggered and can not return.
 Input        : address
 Output       : none
 Return Value : none
 
  History        :
  1.Date         : 2012/11/7
    Author       : Brighton Feng
    Modification : Created function

*****************************************************************************/
void execute_reserved_space(Uint32 func_addr)
{
	void (* func)();

	func= (void (*)())func_addr;
	printf("!!!execute at reserved address 0x%x...\n", func_addr);
	func();

}

/*****************************************************************************
 Prototype    : Reserved_Space_Test
 Description  : Reserved memory Space Test example
 Input        : None
 Output       : None
 Return Value : 
 
  History        :
  1.Date         : 2012/11/7
    Author       : Brighton Feng
    Modification : Created function

*****************************************************************************/
void Reserved_Space_Test()
{
    volatile Uint32 value;

	puts("==================-Reserved space access test-=================");

	puts("--------------------access zero address------------------------");
	read_reserved_space(0);
	write_reserved_space(0);

	puts("--------------------access address exceed LL2------------------");
	read_reserved_space((Uint32 *)0x00a00000);
	write_reserved_space((Uint32 *)0x00a00000);

	puts("------------access local reserved configuration space----------");
	read_reserved_space((Uint32 *)0x01000000);
	write_reserved_space((Uint32 *)0x01000000);

	puts("---access reserved cfg space between INTC and power control----");
	read_reserved_space((Uint32 *)&gpCGEM_regs->RSVD10[0]);
	write_reserved_space((Uint32 *)&gpCGEM_regs->RSVD10[0]);

	puts("------------access global reserved configration space----------");
	read_reserved_space((Uint32 *)0x01CF0000);
	write_reserved_space((Uint32 *)0x01CF0000);

	puts("-------------access reserved XMC configuration space-----------");
	read_reserved_space((Uint32 *)&gpXMC_regs->RSVD0[128]);
	write_reserved_space((Uint32 *)&gpXMC_regs->RSVD0[128]);

	puts("-----------------access XMC unmapped space---------------------");
	read_reserved_space((Uint32 *)0x1C000000);
	write_reserved_space((Uint32 *)0x1C000000);

	puts("-----------------execute from reserved space-------------------");
	/*execute from reserved space can not safely return*/
	execute_reserved_space(0);
}

