/******************************************************************************

  Copyright (C), 2001-2012, Texas Instrument.

 ******************************************************************************
  File Name     : Robust_SL2_XMC_MP.c
  Version       : Initial Draft
  Author        : Zhan
  Created       : 2012/3/11
  Last Modified :
  Description   : SL2 XMC memory protection example
  Function List :
              SL2MemoryBufInit
              XMC_SL2_MP_test
  History       :
  1.Date        : 2012/3/11
    Author      : Zhan
    Modification: Created file

******************************************************************************/
#include <stdio.h>
#include "KeyStone_common.h"
#include "Robust_SL2_XMC_MP.h"
/*----------------------------------------------*
 * external variables                           *
 *----------------------------------------------*/

/*----------------------------------------------*
 * external routine prototypes                  *
 *----------------------------------------------*/

/*----------------------------------------------*
 * internal routine prototypes                  *
 *----------------------------------------------*/

/*----------------------------------------------*
 * project-wide global variables                *
 *----------------------------------------------*/

/*----------------------------------------------*
 * module-wide global variables                 *
 *----------------------------------------------*/
/*total 64K data for SL2*/
#pragma DATA_SECTION(sl2MPDataDst, "sl2MPData_w")
#pragma DATA_ALIGN(sl2MPDataDst,64*1024) //align to MPAX segment bouandary
volatile char sl2MPDataDst[2][32768];

/* 4K per segments */
#pragma DATA_SECTION(sl2MPDataSrc, "sl2MPData_r")
#pragma DATA_ALIGN(sl2MPDataSrc,16*1024) //align to MPAX segment bouandary
volatile char sl2MPDataSrc[4][4096];

/*memory protection and address extension configuration table*/
#pragma DATA_SECTION(SL2_MPAX_cfg_table,".far:Core_MPAX")
MPAX_Config SL2_MPAX_cfg_table[]=
{
    /*BADDR                  RADDR           SegementSize            AccessPermisionMask
    32-bit virtual           36-bit physical in byte, must           Access types allowed
    base address             address right   be power of 2           in this address range
                             shift by 4                                                    */
    {(Uint32)sl2MPDataDst,   0,              sizeof(sl2MPDataDst), MP_SW|MP_UW},/*write only data*/
    {(Uint32)sl2MPDataSrc,   0,              sizeof(sl2MPDataSrc), MP_SR|MP_UR}/*read only data*/
};

/*----------------------------------------------*
 * constants                                    *
 *----------------------------------------------*/

/*----------------------------------------------*
 * macros                                       *
 *----------------------------------------------*/

/*----------------------------------------------*
 * routines' implementations                    *
 *----------------------------------------------*/
/*****************************************************************************
 Prototype    : SL2MemoryBufInit
 Description  : SL2 XMC test buffer initialization
 Input        : None
 Output       : None
 Return Value : 
 
  History        :
  1.Date         : 2012/3/11
    Author       : Zhan
    Modification : Created function

*****************************************************************************/
void SL2MemoryBufInit()
{
	int i;
	for(i=0; i<4096; i++)
	{
		sl2MPDataSrc[0][i] = i;
		sl2MPDataSrc[1][i] = i;
		sl2MPDataSrc[2][i] = i;
		sl2MPDataSrc[3][i] = i;

	}

	for(i=0; i<32768; i++)
	{
		sl2MPDataDst[0][i] = 0xFF;
		sl2MPDataDst[1][i] = 0xFF;
	}
	return;
}
/*****************************************************************************
 Prototype    : XMC_SL2_MP_test
 Description  : SL2 memory protection test, the 64K data was used for test,
 Input        : None
 Output       : None
 Return Value : 
 
  History        :
  1.Date         : 2012/3/11
    Author       : Zhan
    Modification : Created function

*****************************************************************************/
void XMC_SL2_MP_test()
{
	volatile char readVal;

	puts("===================-XMC SL2 memory protection test-==============");

	SL2MemoryBufInit();

	SL2_MPAX_cfg_table[0].RADDR= SL2_MPAX_cfg_table[0].BADDR>>4;
	SL2_MPAX_cfg_table[1].RADDR= SL2_MPAX_cfg_table[1].BADDR>>4;
	KeyStone_XMC_MPAX_setup(SL2_MPAX_cfg_table, 14,
		sizeof(SL2_MPAX_cfg_table)/sizeof(MPAX_Config));

	/* The first write is allowed */
	printf("DSP core write to 0x%x which is write-only...\n",
		&sl2MPDataDst[0][1024]);
	sl2MPDataDst[0][1024] = 0xAA;

	/* The first read is allowed */
	printf("DSP core read from 0x%x which is read-only...\n",
		&sl2MPDataSrc[0][2056]);
	readVal = sl2MPDataSrc[0][2056];

	/* Read the sl2 trigger memory protection */
	printf("\n!!!DSP core read from 0x%x which is write-only...\n",
		&sl2MPDataDst[1][2111]);
	readVal = sl2MPDataDst[1][2111];

	/*normally, exception is captured about 10~100 cycles after the 
	access violation. Dummy operations added here to wait for it.*/
	asm(" nop 5");
	asm(" nop 5");
	asm(" nop 5"); 	
	asm(" nop 5"); /*NRP should be around the address of this instruction*/	
	asm(" nop 5"); 	

	/* Write the sl2 trigger memory protection */
	printf("\n!!!DSP core write to 0x%x which is read-only...\n",
		&sl2MPDataSrc[1][1100]);
	sl2MPDataSrc[1][1100] = 0x11;

	/*normally, exception is captured about 10~100 cycles after the 
	access violation. Dummy operations added here to wait for it.*/
	asm(" nop 5");
	asm(" nop 5");
	asm(" nop 5");
	asm(" nop 5");
	asm(" nop 5");
	asm(" nop 5"); 	/*NRP should be around the address of this instruction*/
	asm(" nop 5"); 	

	printf("\n!!!DSP core write to gpXMC_regs->XMPAX[0].XMPAXL register at 0x%x without unlocking...\n",
		&gpXMC_regs->XMPAX[0].XMPAXL);
	gpXMC_regs->XMPAX[0].XMPAXL= 0xFFFFFFFF;

	/*normally, exception is captured about 10~100 cycles after the 
	access violation. Dummy operations added here to wait for it.*/
	asm(" nop 5");
	asm(" nop 5");
	asm(" nop 5");
	asm(" nop 5");
	asm(" nop 5");
	asm(" nop 5"); 	/*NRP should be around the address of this instruction*/
	asm(" nop 5"); 	
}


