/******************************************************************************

  Copyright (C), 2001-2012, Texas Instrument.

 ******************************************************************************
  File Name     : Robust_watch_dog.c
  Version       : Initial Draft
  Author        : Brighton Feng
  Created       : 2012/10/13
  Last Modified :
  Description   : This example shows watch-dog Timer on KeyStone DSP
  Function List :
              watch_dog_test
  History       :
  1.Date        : 2012/10/13
    Author      : Brighton Feng
    Modification: Created file

******************************************************************************/
#include <c6x.h>
#include <stdio.h>
#include <csl_bootcfgAux.h>
#include "KeyStone_common.h"
/*----------------------------------------------*
 * external variables                           *
 *----------------------------------------------*/

/*----------------------------------------------*
 * external routine prototypes                  *
 *----------------------------------------------*/

/*----------------------------------------------*
 * internal routine prototypes                  *
 *----------------------------------------------*/

/*----------------------------------------------*
 * project-wide global variables                *
 *----------------------------------------------*/

/*----------------------------------------------*
 * module-wide global variables                 *
 *----------------------------------------------*/

/*----------------------------------------------*
 * constants                                    *
 *----------------------------------------------*/

/*----------------------------------------------*
 * macros                                       *
 *----------------------------------------------*/

/*----------------------------------------------*
 * routines' implementations                    *
 *----------------------------------------------*/


/*period (in millisecond) of the watch-dog timer*/
/*since the timer runs at (DSP core clock)/6
the conversion between millisecond and timer PRD value is:
PRD = (xxx_MS/1000)*DSP_CLK_HZ/6
    = xxx_MS*(DSP_CLK_HZ/1000)/6
    = xxx_MS*DSP_CLK_KHZ/6*/
#define WATCH_DOG_PERIOD_MS 	3000
    
/*****************************************************************************
 Prototype    : watch_dog_test
 Description  : set local timer (==DNUM of a core) as watch dog timer,
                service the watch-dog for sometime, 
                after stop servicing watch-dog, let it trigger NMI
 Input        : None
 Output       : None
 Return Value : 
 
  History        :
  1.Date         : 2012/10/13
    Author       : Brighton Feng
    Modification : Created function

*****************************************************************************/
void watch_dog_test()
{
	int i;
	Uint32 timerCnt;
	Timer64_Config tmrCfg;

	puts("===================-watch dog timer test-========================");
	//select output on TIMO0 from local timer.
	gpBootCfgRegs->TOUTSEL = (DNUM*2)<<CSL_BOOTCFG_TOUTSEL_TOUTSEL0_SHIFT;

	gpBootCfgRegs->RSTMUX[DNUM] = 
		(4<<CSL_BOOTCFG_RSTMUX0_RSTMUX_DELAY0_SHIFT)
		/*WD timer event trigger NMI*/
		|(WD_TRIGGER_NMI<<CSL_BOOTCFG_RSTMUX0_RSTMUX_OMODE0_SHIFT);

	puts("start watch-dog timer...");

	//configure the timer for watch-dog
	tmrCfg.timer_num= DNUM;
	tmrCfg.timerMode= TIMER_WATCH_DOG;
	tmrCfg.period= (unsigned long long)WATCH_DOG_PERIOD_MS*gDSP_Core_Speed_Hz/6000;
	tmrCfg.pulseWidth= 3;
	Timer64_Init(&tmrCfg);

	//service the watch-dog for sometime, 
	for(i=1; i<=10; i++)
	{
		timerCnt= gpTimerRegs[DNUM]->CNTLO;
		Service_Watchdog(DNUM);
		printf("service watch-dog %d times, at time counter = %u\n", 
			i, timerCnt);
	}

	printf("stop servicing watch-dog, it will timeout and trigger NMI after %d ms...\n",
		WATCH_DOG_PERIOD_MS);

	/*add dummy nop here to wait for the NMI exception happens*/
	for(i=0; i<30000000; i++)
	{
		asm(" nop 5");		asm(" nop 5");
		asm(" nop 5");		asm(" nop 5");
		asm(" nop 5");		asm(" nop 5");
		asm(" nop 5");		asm(" nop 5");
		asm(" nop 5");		asm(" nop 5");
		asm(" nop 5");		asm(" nop 5");
		asm(" nop 5");		asm(" nop 5");
		asm(" nop 5");		asm(" nop 5");
		asm(" nop 5");		asm(" nop 5");
		asm(" nop 5");		asm(" nop 5");
		asm(" nop 5");		asm(" nop 5");
		asm(" nop 5");		asm(" nop 5");
		asm(" nop 5");		asm(" nop 5");
		asm(" nop 5");		asm(" nop 5");
		asm(" nop 5");		asm(" nop 5");
		asm(" nop 5");		asm(" nop 5");
	}
}

